#!/bin/sh
# Copyright 2018 Gigaset Communications GmbH

### BEGIN INIT INFO
# Provides:          syslog.sh
# Required-Start:    mountall read-only-rootfs-hooks
# Required-Stop:
# Default-Start:     S
# Default-Stop:      0 6 7
# Short-Description: System logger.
# Description:
### END INIT INFO

source /usr/sbin/maxwell-log-functions
source /etc/bootlog.conf

bootlog_setup()
{
    test -h ${BOOTLOG_DIR}/logger && { ln -sf /bin/busybox.nosuid ${BOOTLOG_DIR}/logger; }
    test -f ${BOOTLOG_FILE} && { cat ${BOOTLOG_FILE} >> ${LOGFILE}; rm -f ${BOOTLOG_FILE}; }
}

bootlog_teardown()
{
    test -h ${BOOTLOG_DIR}/logger && { ln -sf /usr/bin/bootlogger ${BOOTLOG_DIR}/logger; }
}

SYSLOG_DAEMON_NAME="syslogd"
SYSLOG_DAEMON_PATH="/sbin/syslogd"
KLOG_DAEMON_NAME="klogd"
KLOG_DAEMON_PATH="/sbin/klogd"
SYSLOG_ARGS=()

if [ -f /etc/syslog.d/syslog-startup.conf ]; then
    source /etc/syslog.d/syslog-startup.conf
    source /etc/syslog.d/syslog.rules

    LOG_LOCAL=0
    LOG_REMOTE=0

    for destination in ${DESTINATION}; do
       test "${destination}" = "buffer"                     && { SYSLOG_ARGS+=("-C"); SYSLOG_ARGS+=("${BUFFERSIZE}"); LOG_LOCAL=1;  }
       test "${destination}" = "file" -a -n "${LOGFILE}"    && { SYSLOG_ARGS+=("-O"); SYSLOG_ARGS+=("${LOGFILE}");    LOG_LOCAL=1;  }
       test "${destination}" = "file" -a -n "${ROTATESIZE}" && { SYSLOG_ARGS+=("-s"); SYSLOG_ARGS+=("${ROTATESIZE}"); LOG_LOCAL=1;  }
       test "${destination}" = "file" -a -n "${ROTATEGENS}" && { SYSLOG_ARGS+=("-b"); SYSLOG_ARGS+=("${ROTATEGENS}"); LOG_LOCAL=1;  }
       test "${destination}" = "remote"                     && { SYSLOG_ARGS+=("-R"); SYSLOG_ARGS+=("${REMOTE}");     LOG_REMOTE=1; }
    done

    test "${LOG_LOCAL}" = "1" -a "${LOG_REMOTE}" = "1"      && SYSLOG_ARGS+=("-L")
    test "${REDUCE}" = "yes"                                && SYSLOG_ARGS+=("-S")
    test "${DROPDUPLICATES}" = "yes"                        && SYSLOG_ARGS+=("-D")
    test -n "${LOGLEVEL}"                                   && { SYSLOG_ARGS+=("-l"); SYSLOG_ARGS+=("${LOGLEVEL}"); }
    test -n "${ROTATECMD}"                                  && { SYSLOG_ARGS+=("-c"); SYSLOG_ARGS+=("$ROTATECMD"); }
else
    # default: log to 16K shm circular buffer
    SYSLOG_ARGS=("-C")
fi

case "$1" in
    start)
        log_info "Starting ${SYSLOG_DAEMON_NAME}/${KLOG_DAEMON_NAME} daemons";
        bootlog_setup
        (
            start-stop-daemon --start --background --name ${SYSLOG_DAEMON_NAME} --startas ${SYSLOG_DAEMON_PATH} -- -n "${SYSLOG_ARGS[@]}"
            log_info "Executing command: start-stop-daemon --start --background --name ${SYSLOG_DAEMON_NAME} --startas ${SYSLOG_DAEMON_PATH} -- -n \"\${SYSLOG_ARGS[@]}\""
            log_info "SYSLOG_ARGS = ${SYSLOG_ARGS[@]}"
        ) &
        (
            usleep 100000
            start-stop-daemon --start --background --name ${KLOG_DAEMON_NAME} --startas ${KLOG_DAEMON_PATH} -- -n
            log_info "Executing command: start-stop-daemon --start --background --name ${KLOG_DAEMON_NAME} --startas ${KLOG_DAEMON_PATH} -- -n"
        ) &
        ;;
    stop)
        log_info "Stopping ${SYSLOG_DAEMON_NAME}/${KLOG_DAEMON_NAME} daemons";
        (
            log_info "Executing command: start-stop-daemon --stop --name ${SYSLOG_DAEMON_NAME}"
            start-stop-daemon --stop --name ${SYSLOG_DAEMON_NAME}
        ) &
        (
            log_info "Executing command: start-stop-daemon --stop --name ${KLOG_DAEMON_NAME}"
            start-stop-daemon --stop --name ${KLOG_DAEMON_NAME}
        ) &
        bootlog_teardown
        sleep 1;
        ;;
    restart)
        log_info "Restarting ${SYSLOG_DAEMON_NAME}/${KLOG_DAEMON_NAME} daemons";
        $0 stop
        $0 start
        ;;
    *)
        echo "Usage: syslog { start | stop | restart }" >&2
        exit 1
        ;;
esac

exit 0
