#!/bin/sh
# Copyright 2016-2018 Gigaset Communications GmbH

### BEGIN INIT INFO
# Provides:          boot-complete.sh
# Required-Start:
# Required-Stop:     maxwell-teardown
# Should-Stop:       $network $portmap
# Default-Start:     5
# Default-Stop:      6 7
# Short-Description: Creates file in /var/run tmpfs to inform that bootup
#                    process is completed
### END INIT INFO

source /usr/sbin/maxwell-log-functions
source /etc/boot-complete.conf

PATH=/sbin:/bin:/usr/sbin:/usr/bin
PROPCLI=/usr/bin/propcli

# Set proper timezone
TIMEZONE=$($PROPCLI get System.DateAndTime.TimeZone --use-nvm)
if [ $? -eq 0 ]; then
	TIMEZONE=$(cut -d '.' -f 2 <<< "${TIMEZONE}")
	TIMEZONE_FILE="/usr/share/zoneinfo/${TIMEZONE}"
	if [ -f "${TIMEZONE_FILE}" ]; then
		ln -sf "${TIMEZONE_FILE}" /etc/localtime
		echo "${TIMEZONE}" > /etc/timezone
		log_info "Time zone changed to: ${TIMEZONE}"
	else
		log_warning "Wrong timezone info: $TIMEZONE"
	fi
else
	log_error "Can't get timezone from NVM"
fi

# Create boot_complete file
test "${BOOT_COMPLETE_FILE}" || { log_error "Required BOOT_COMPLETE_FILE not defined"; exit 1; }
touch "${BOOT_COMPLETE_FILE}"
