#!/bin/sh
# Copyright 2017-2018 Gigaset Communications GmbH

### BEGIN INIT INFO
# Provides:          maxwell-teardown.sh
# Required-Start:
# Required-Stop:     maxwell-teardown
# Should-Stop:       $network $portmap $syslog
# Default-Start:
# Default-Stop:      6 7
# Short-Description: Keep some settings and files during reboot
### END INIT INFO

PATH=/sbin:/bin:/usr/sbin:/usr/bin

source /etc/logrotate.d/logrotate.conf

# Paths to the watchdog statistic/backup file
WD_STATS_FILE="/var/volatile/wd_stats.json"
BACKUP_FILE="/config/wd_stats.json"

# Store uptime before reboot the device
UPTIME=$(cat /proc/uptime | awk '{print int($1)}')
echo "Current uptime is: $UPTIME"
echo $UPTIME > /data/systemstats/tmp/last_not_accounted_uptime

# Store watchdog stats file during reboot
if [ -f $WD_STATS_FILE ]; then
	cp $WD_STATS_FILE $BACKUP_FILE
fi

# Logs and logrotate paths
LOGROTATE_STATUS_FILE="/var/lib/logrotate-teardown.status"
SYSLOG_DAEMON="syslogd"

test -x ${LOGROTATE_BINARY_FILE} || { log_error "Logrotate binary ${LOGROTATE_BINARY_FILE} either not existing or not executable, skipping."; exit 1; }
test -f ${LOGROTATE_DEFAULT_CONFIG_FILE} || { log_error "Logrotate config file ${LOGROTATE_DEFAULT_CONFIG_FILE} not existing, skipping."; }

# Store maxwell log file during reboot
if [ -f ${MAXWELL_LOG_FILE} ]; then
	# In order to store all logs preceding this teardown script the execution
	# of log rotation must be delayed until syslog daemon completes gracefully
	cnt=0;
	while [[ $cnt -lt 10 ]] && ps | grep -v grep | grep ${SYSLOG_DAEMON} -q; do
		cnt=$((cnt+1)); usleep 100000;
	done

	${LOGROTATE_BINARY_FILE} -fs ${LOGROTATE_STATUS_FILE} ${LOGROTATE_DEFAULT_CONFIG_FILE}
fi
