#!/bin/sh
#
# Copyright (C) 2018 Gigaset Communications GmbH
#

FW_PATH="/lib/firmware/touch"
SYSFS_PATH="/sys/devices/lnk/touch/sitronix_ts_attrs"

start()
{
	local version
	local revision
	local fw_revision
	local fw_prefix="sitronix"

	if [ -e $FW_PATH/ -a -e $SYSFS_PATH/fwupdate ]; then
		# fw_version indicates the actual module vendor version. Versions <= 4 will work with the default
		# touch firmware. Version > 4 will probaby needs different firmware but the support for old one
		# needs to be still provided. Therefor the firmware file names will get a version flag.
		# With the help of fw_revision it can be decided if a firmware a specific touch module version
		# needs to be updated. Currently it will be only check if the revision is different but
		# this might be changed in future.
		# Most probably future modules will have already the correct fw in place but we need to be prepared.

		version=$(cat $SYSFS_PATH/fw_version)
		revision=$(tr " " . < $SYSFS_PATH/fw_revision)

		[ "$version" -gt  4 ] && fw_prefix="sitronix_v$version"

		if [ "$revision" != "unkown" -a -e "$FW_PATH/${fw_prefix}_fw.revision" ]; then

			fw_revision=$(cat "$FW_PATH/${fw_prefix}_fw.revision")

			if [ "$version" -lt  4 -o "$revision" != "$fw_revision"  ]; then
				echo 1 > $SYSFS_PATH/fwupdate &
			fi
		fi
	fi
}

if [ "$1" = "start" ]; then
	start
fi
