#!/bin/sh
# Copyright 2018 Gigaset Communications GmbH

### BEGIN INIT INFO
# Provides:          factory-reset.sh
# Required-Start:    Filesystems mounted
# Required-Stop:
# Default-Start:     S 7
# Default-Stop:
# Short-Description: Script performs required acction on factory reset
# Description:       There should be file systems already mounted before
#                    performing factory reset
### END INIT INFO

source /etc/init.d/init-functions
source /usr/share/scripts/reset-functions

get_mount_options() {
	awk -F' '  '{ print $4 }' /etc/fstab | grep "/config/appdata/${1}"
}

case "$1" in
	start)
		# runlevel variable exported by rc stript
		current_runlevel=$runlevel
		if [[ "$current_runlevel" != "S" && "$current_runlevel" != "7" ]]; then
			log_error "Executing $0 allowed only on runlevels S and 7"
		fi
		bootmode=$(sed 's/.*\(m\=[0-2]\).*/\1/' /proc/cmdline)
		if [[ "$current_runlevel" == "S" && "$bootmode" == "m=2" ]]; then
			log_info "Performing factory reset ..."
			umount -l /etc/
			umount -l /www/
			umount -l /usr/
			reset_factory_default
			mount -t overlayfs -o "$(get_mount_options "etc")" /etc
			mount -t overlayfs -o "$(get_mount_options "usr")" /usr
			mount -t overlayfs -o "$(get_mount_options "www")" /www
		elif [ "$current_runlevel" == "7" ]; then
			log_info "Performing factory reset ..."
			reset_factory_default
		fi
		;;
	stop)
		;;
	*)
		log_info "Usage is: $0 {start|stop}" >&2
		exit 1
		;;
esac

exit 0
