#!/bin/bash
#Copyright 2017-2018 Gigaset Communications GmbH

source /usr/sbin/maxwell-log-functions

function usage
{
	log_error "USAGE: -mount <mode>(ro/rw) <device>(or SYMLINK) || -umount <mount point>(directory name located in /media)."
}

function check_device_exists
{
	local device_name="$1"
	test -b /dev/${device_name}
}

function check_mount_point_exists
{
	local mount_point="$1"
	test -d "${mount_point}"
}

function check_device_mounted
{
	local device_name="$1"
	local mount_point="$2"
	local mount_entry=$(grep "${device_name} ${mount_point}" < /proc/mounts | cut -d' ' -f1,2)
	test "${mount_entry}" != ""
}

function mount_usb_storage
{
	local -i result=1
	local mode="$1"
	#storage is mounted on directory with the same name as device
	local device_name="$2"
	local device_path="/dev/${device_name}"
	local mount_point="/media/${device_name}"

	if [[ $mode != "ro" && $mode != "rw" ]]; then
		usage
	else
		check_device_exists ${device_name}
		res="$?"

		if [[ ${res} -ne 0 ]]; then
			log_error "Device ${device_name} does not exist"
		else
			check_mount_point_exists ${mount_point}
			res="$?"

			if [[ ${res} -ne 0 ]]; then
				log_error "Mount point ${mount_point} does not exist"
			else
				check_device_mounted ${device_name} ${mount_point}
				res="$?"

				if [[ ${res} -ne 0 ]]; then
					mount -o $mode $device_path $mount_point
					result=$?
				else
					mount -o remount,$mode $device_path $mount_point
					result=$?
				fi
			fi
		fi
	fi

	if [[ ${result} -eq 0 ]]; then
		log_info "Device ${device_path} has been successfully mounted at: ${mount_point}"
	else
		log_error "Device ${device_path} has not been mounted"
	fi

	return ${result}
}

function unmount_usb_storage
{
	local -i result=1
	local device_name="$1"
	local mount_point="/media/${device_name}"

	check_mount_point_exists ${mount_point}
	res="$?"

	if [[ ${res} -ne 0 ]]; then
		log_error "Mount point ${mount_point} does not exist"
	else
		check_device_mounted ${device_name} ${mount_point}
		res="$?"

		if [[ ${res} -eq 0 ]]; then
			umount $mount_point -l
			result=$?
		else
			log_info "$mount_point is not mounted"
			result=0
		fi
	fi

	if [[ ${result} -eq 0 ]]; then
		log_info "${mount_point} has been successfully unmounted"
	else
		log_error "${mount_point} has not been unmounted"
	fi

	return ${result}
}

function main
{
	local result=1
	local action="$1"
	local device_name="$2"
	local mount_mode="$3"

	case "${action}" in
		"-mount")
			mount_usb_storage ${mount_mode} ${device_name}
			result=$?
			;;
		"-umount")
			unmount_usb_storage ${device_name}
			result=$?
			;;
		*)
			usage
			;;
	esac

	return ${result}
}

main $@
