#!/bin/bash
# Copyright 2018 Gigaset Communications GmbH

source /etc/init.d/init-functions

NAME=$(basename $0 | cut -d '.' -f 1)

log_info "Running PCAP variant: $NAME"

# System tools
PROPCLI="/usr/bin/propcli"

# Common options
FILE_COUNT=1
FILE_SIZE=3
SIP_PORT_PROPERTY_NAME="SIP.Port"

case "$NAME" in
	maxwell-pcap)
		PACKETS_TILL_AUTOMATIC_STOP=30000
		FILE_PATH="/var/volatile/pcap/pcap.log"
		STATE_PROPERTY_NAME="State.Network.PCAP"
		DAEMON_ARGS="-c $PACKETS_TILL_AUTOMATIC_STOP -C $FILE_SIZE -W $FILE_COUNT -w $FILE_PATH"
		;;
	maxwell-sip-pcap)
		SIP_PORT=$($PROPCLI get $SIP_PORT_PROPERTY_NAME)
		if [ $? -ne 0 ]; then
			log_error "Can't get SIP port"
			$($PROPCLI set $STATE_PROPERTY_NAME stopped)
			exit 1
		fi

		PACKETS_TILL_AUTOMATIC_STOP=400
		FILE_PATH="/var/volatile/pcap/sip.pcap"
		STATE_PROPERTY_NAME="State.Network.SIP_PCAP"
		DAEMON_ARGS="-n port $SIP_PORT -c $PACKETS_TILL_AUTOMATIC_STOP -W $FILE_COUNT -Uw $FILE_PATH"
		;;
	*)
		log_error "Variant: $0 is not supported!"
		exit 1
		;;
esac

DAEMON="/usr/sbin/tcpdump"
PIDFILE="/var/run/pid/$NAME.pid"
USER=maxwell
GROUP=maxwell

# Define callbacks for maxwell-init-functions
on_start_succeeded()
{
	$PROPCLI set $STATE_PROPERTY_NAME started
}

on_stop_succeeded()
{
	$PROPCLI set $STATE_PROPERTY_NAME stopped
}

on_delete_succeeded()
{
	rm -f $FILE_PATH
	$PROPCLI set $STATE_PROPERTY_NAME deleted
}

source /etc/init.d/maxwell-init-functions
