#!/bin/bash
#Copyright 2017-2018 Gigaset Communications GmbH

source /usr/sbin/maxwell-log-functions

#Only one USB flash drive is supported at a time
device_name="usb_flash_drive"
mount_point="/media/${device_name}"

phonebook_maxwell_path="/data/internal/localDirectory.xml"
import_path="/maxwell_import/localDirectory.xml"
export_path="/maxwell_export/localDirectory.xml"

#file/directory localization on flash drive preceded by '/'
function check_data_path
{
	local file_path=$1
	test -e ${file_path}
}

function main
{
	local -i result=1
	local action=$1
	local full_path=""

	if [[ ${action} == "-import" ]]; then
		/etc/udev/scripts/usb_flash_drive.sh "-mount ${device_name} ro"
		res="$?"

		if [[ ${res} -eq 0 ]]; then
			full_path="${mount_point}${import_path}"

			check_data_path ${full_path}
			res="$?"

			if [[ ${res} -eq 0 ]]; then
				cp -r $full_path $phonebook_maxwell_path
				result=$?
				[[ ${result} -ne 0 ]] && log_error "Import failed during copying data."
			else
				log_error "${full_path} not found."
			fi
		else
			log_error "${device_name} is not ready. Data import failed."
		fi
	elif [[ ${action} == "-export" ]]; then
		/etc/udev/scripts/usb_flash_drive.sh "-mount ${device_name} rw"
		res="$?"

		if [[ ${res} -eq 0 ]]; then
			full_path="${mount_point}${export_path}"

			check_data_path ${phonebook_maxwell_path}
			res="$?"

			if [[ ${res} -eq 0 ]]; then
				dir=$(dirname $full_path)
				if [[ ! -d ${dir} ]]; then
					mkdir -p ${dir}
					res="$?"
				fi

				if [[ ${res} -eq 0 ]]; then
					cp -r $phonebook_maxwell_path $full_path
					result=$?
					[[ ${result} -ne 0 ]] && log_error "Export failed during copying data."
				else
					log_error "${dir} not exists and could not be created."
				fi
			else
				log_error "${phonebook_maxwell_path} not found."
			fi

			#beside of result always back to ro mode - remount
			/etc/udev/scripts/usb_flash_drive.sh "-mount ${device_name} ro"
			res="$?"

			if [[ ${res} -eq 0 ]]; then
				log_info "${device_name} has been successfully remounted back to ro mode after data export."
			else
				log_error "Switch back ${device_name} to ro mode failed."
			fi
		else
			log_error "${device_name} is not ready. Data export failed."
		fi
	else
		log_error "Wrong parameter. Required one: [-import] or [-export]"
	fi

	if [[ ${result} -eq 0 ]]; then
		log_info "Operation ${action} for ${full_path} performed successfully."
	else
		log_info "Operation ${action} FAILED!"
	fi

	return ${result};
}

main $@
