#!/bin/sh
# Copyright 2016-2018 Gigaset Communications GmbH

source /usr/sbin/maxwell-log-functions

CONFIG_FILE="/etc/firmware-update.conf"
test -f ${CONFIG_FILE} || { log_error "${CONFIG_FILE} does not exist, skipping"; exit 1; }

source ${CONFIG_FILE}

test ${FIRMWARE_UPDATE_DIR} || { log_error "Required variable FIRMWARE_UPDATE_DIR not defined, skipping"; exit 1; }
test ${FIRMWARE_UPDATE_TMP_DIR} || { log_error "Required variable FIRMWARE_UPDATE_TMP_DIR not defined, skipping"; exit 1; }
test ${FIRMWARE_UPDATE_CLEANUP_UTILITY} || { log_error "Required variable FIRMWARE_UPDATE_CLEANUP_UTILITY not defined, skipping"; exit 1; }
test ${FIRMWARE_UPDATE_LOG_FILE} || { log_error "Required variable FIRMWARE_UPDATE_LOG_FILE not defined, skipping"; exit 1; }

LOG_TAG=$(basename ${FIRMWARE_UPDATE_CLEANUP_UTILITY} .sh)
source /usr/sbin/firmware-update-log-functions

# Redirect stderr of executed programs to /dev/null
exec 2>/dev/null

result=0
cleaning_partition="ubi0:config"

verify_partition_and_cleanup()
{
    local directory=$1
    local expected_partition=$2
    local mount_partition=$(df -P ${directory} | tail -1 | cut -d' ' -f 1)

    # TODO: GM-2011 print blocking process name next to its pid
    test "${mount_partition}" == "${expected_partition}" && \
    { \
        pids=( $(lsof -t ${directory}) ) && { \
            loge "Fail: ${directory} occupied by processes: ${pids[@]}"; \
            result=1; \
        } || { \
            logi "Cleaning content of ${directory} located on ${cleaning_partition} partition"; \
            rm -rf ${directory}/*; \
        } \
    } || { \
        loge "Fail: ${directory} is not located on ${cleaning_partition} partition"; \
        result=1; \
    }

    return ${result}
}

logi "Cleaning firmware update directories"
verify_partition_and_cleanup "${FIRMWARE_UPDATE_TMP_DIR}" "${cleaning_partition}"
verify_partition_and_cleanup "${FIRMWARE_UPDATE_DIR}" "${cleaning_partition}"
logi "Clean up utils script exit code: ${result}"

exit ${result}
