#!/bin/sh
# Copyright 2015-2018 Gigaset Communications GmbH

source /usr/sbin/maxwell-log-functions

readonly CONFIG_FILE="/etc/firmware-update.conf"
test -f ${CONFIG_FILE} || { log_error "${CONFIG_FILE} does not exist, skipping"; exit 1; }

source ${CONFIG_FILE}

test ${FIRMWARE_UPDATE_DIR} || { log_error "Required variable FIRMWARE_UPDATE_DIR not defined, skipping"; exit 1; }
test ${FIRMWARE_UPDATE_EXECUTE_UTILITY} || { log_warning "Required variable FIRMWARE_UPDATE_EXECUTE_UTILITY not defined, skipping"; exit 1; }
test ${FIRMWARE_UPDATE_LOG_FILE} || { log_warning "Required variable FIRMWARE_UPDATE_LOG_FILE not defined, skipping"; exit 1; }

source /usr/sbin/firmware-update-log-functions

logi "Executing firmware upgrade verification script"

FIRMWARE_UPDATE_BINARY=${FIRMWARE_UPDATE_DIR}/update-binary.bin
# Update file name
FILE_NAME=$1
# Path to update file
FILE_PATH=$2

# verify image signature
verify_signature()
{
    if [ -x /vendor/swupdate/bin/ssign-verify ]; then
        /vendor/swupdate/bin/ssign-verify "/vendor/swupdate/" ${FIRMWARE_UPDATE_BINARY}
    else
        /usr/bin/ssign-verify "${FIRMWARE_UPDATE_BINARY}"
    fi

    if [ ! $? = 0 ]; then
        # Error: Image invalid
        return 1
    fi
    return 0
}

FILE_NAME_COUNT=$(wc -w <<< $1)
test $# -eq 2 || { loge "Script ${SCRIPT_NAME} expects 2 arguments"; exit 1; }
test ${FILE_NAME_COUNT} == 1 || {
    loge "Expected single .bin file in ${FIRMWARE_UPDATE_DIR} directory."
    loge "Have following files instead: ${FILE_NAME}"; exit 1;
}
test ${FILE_NAME##*.} == bin || {
    loge "Firmware update file with .bin extension expected";
    loge "Have following files instead: ${FILE_NAME}"; exit 1;
}
test -f "${FILE_PATH}/${FILE_NAME}" || { loge "Firmware update file along with path not found"; exit 1; }

# Copy downloaded firmware update file to firmware update storage
# where other firmware update process components expect it to be.
mv "${FILE_PATH}/${FILE_NAME}" "${FIRMWARE_UPDATE_BINARY}"
sync

if result=$(verify_signature); then
    logi "Firmware image signature verified successfully."
    if result=$(/usr/sbin/updatevalidator "${FIRMWARE_UPDATE_BINARY}"); then
        logi "Firmware image version verified successfully, update may proceed."
        # Set recovery in bootloader
        fw_setenv bootmode recovery
        sleep 2
        reboot-factory-reset
    else
        loge "Firmware update image failed version and variant verification."
        loge "${result}"
        exit 1
    fi
else
    loge "Firmware update image failed signature verification."
    loge "${result}"
    exit 1
fi

exit 0
