#!/bin/sh
# Copyright 2018 Gigaset Communications GmbH

IFCONFIG="/sbin/ifconfig"
PROPCLI="/usr/bin/propcli"
NULL_IP_ADDRESS="0.0.0.0"

source /usr/sbin/maxwell-log-functions

# only for use as a "zcip" callback script
test -z "${interface}" && { log_error "Found no 'interface' environment variable, skipping"; exit 1; }

check_ip_not_empty()  {
	test -z "${ip}" && { log_error "Found no 'ip' environment variable, skipping"; exit 1; }
}

# zcip should start on boot/resume and various media changes
case "$1" in
	init)
		# for now, zcip requires the link to be already up,
		# and it drops links when they go down.  that isn't
		# the most robust model...
		;;

	config)
		check_ip_not_empty

		${IFCONFIG} ${interface} ${ip} netmask 255.255.0.0

		if [ -x "/sbin/ip" ]; then
			/sbin/ip route add default via ${NULL_IP_ADDRESS}
		else
			/sbin/route add default gw ${NULL_IP_ADDRESS} dev ${interface} 2>/dev/null
		fi

		${PROPCLI} set NET.IP.IPv4.Address ${ip} &> /dev/null
		${PROPCLI} set NET.IP.IPv4.SubnetMask "255.255.0.0" &> /dev/null
		${PROPCLI} set NET.IP.IPv4.StandardGateway ${NULL_IP_ADDRESS} &> /dev/null
		;;

	deconfig)
		check_ip_not_empty

		${IFCONFIG} ${interface} down
		;;
esac
