angular.module('WebUI', ['ngRoute', 'ngMessages', 'ngAnimate', 'nya.bootstrap.select', 'angularFileUpload', 'ui.bootstrap']);

angular.module('WebUI').config(['$routeProvider', '$httpProvider', '$compileProvider', '$logProvider', '$animateProvider', '$provide', function ($routeProvider, $httpProvider, $compileProvider, $logProvider, $animateProvider, $provide) {
    $routeProvider
        .when('/Login', {
            templateUrl: 'pages/Login.html',

            resolve: {
                promises: ['Model', function (Model) {
                    return Model.getAll();
                }]
            },

            restricted: false
        })
        .when('/:path*', {
            templateUrl: function (args) {
                return 'pages/' + args.path + '.html';
            },

            resolve: {
                promises: ['Model', function (Model) {
                    return Model.getAll();
                }]
            },

            restricted: true
        })
        .otherwise({
            redirectTo: 'Login'
        });

    $provide.decorator('$log', ['$delegate', function ($delegate) {
        var methods = [
            'assert',
            'count',
            'debug',
            'dir',
            'error',
            'group',
            'groupCollapsed',
            'groupEnd',
            'info',
            'log',
            'profile',
            'profileEnd',
            'table',
            'time',
            'timeEnd',
            'trace',
            'warn'
        ];

        angular.forEach(methods, function (method) {
            if (!angular.isFunction($delegate[method])) {
                $delegate[method] = function() {
                    if ($logProvider.debugEnabled()) {
                        console[method].apply(console, arguments);
                    }
                };
            }
        });

        return $delegate;
    }]);

    $httpProvider.interceptors.push('HTTPInterceptor');
    $httpProvider.defaults.cache = true;
    $compileProvider.debugInfoEnabled(false);
    $logProvider.debugEnabled(true);
    $animateProvider.classNameFilter(/modal|panel/);
}]);

angular.module('WebUI').run(['$location', '$window', '$templateRequest', 'AuthenticationService', 'Status', 'SseService', function ($location, $window, $templateRequest, AuthenticationService, Status, SseService) {
    var token = $location.search().token;

    if (angular.isString(token)) {
        if (token.length === 64) {
            $window.sessionStorage.token = token;
        }

        $location.search('token', null);
    }

    $templateRequest('templates/Popup.html');

    Status.checkOffline.start();

    if (AuthenticationService.isLogged()) {
        Status.keepAlive.start();
        SseService.reconnect();
    }

    $window.addEventListener('beforeunload', function() {
        Status.status = 'unloading';
    });
}]);