angular.module('WebUI').controller('MainCtrl', ['$scope', '$q', '$log', '$location', '$window', '$http', '$timeout', 'UserService', 'AuthenticationService', 'Model', 'Navigation', 'State', 'Tools', 'LogoutTimer', 'Popup', 'Config',
    function ($scope, $q, $log, $location, $window, $http, $timeout, UserService, AuthenticationService, Model, Navigation, State, Tools, LogoutTimer, Popup, Config) {
        $scope.bowser = bowser;
        $scope.config = Config;
        $scope.state = State.state;

        $scope.parameters = Model.parameters;

        $scope.modalDeleteAccount = {};
        $scope.modalDeleteAccount.layer = null;
        $scope.modalDeleteAccount.id = null;

        $scope.menu = Model.menu;
        $scope.text = Model.text;

        $scope.logout = function () {
            UserService.logout();
        };

        $scope.placement = {
            options: [
                'top',
                'top-left',
                'top-right',
                'bottom',
                'bottom-left',
                'bottom-right',
                'left',
                'left-top',
                'left-bottom',
                'right',
                'right-top',
                'right-bottom'
            ],
            selected: 'top'
        };

        $scope.showSuccessPopup = function () {
            var popupData = {
                message: $scope.text.Saved
            };

            var promise = Popup.show(popupData, {
                priority: $scope.config.popup.priority.info,
                size: 'sm',
                closable: true
            });

            $timeout(function () {
                Popup.close(popupData.layer, popupData.id);
            }, 1500);

            return promise;
        };

        $scope.showPopup = function (text) {
            var popupData = {
                message: text
            };

            var promise = Popup.show(popupData, {
                priority: $scope.config.popup.priority.info,
                size: 'sm',
                closable: true
            });

            $timeout(function () {
                Popup.close(popupData.layer, popupData.id);
            }, 1500);

            return promise;
        };

        $scope.processForm = function (config) {
            var getChangedParameters = function (changedParameters, parametersFromCache, parameter, key) {
                if (angular.isArray(parameter)) {
                    var changedArrayParameters = [];

                    angular.forEach(parameter, function (arrayParameter, arrayIndex) {
                        changedArrayParameters[arrayIndex] = {};

                        angular.forEach(arrayParameter, function (arraySubParameter, arraySubKey) {
                            if (getChangedParameters(changedArrayParameters[arrayIndex], parametersFromCache[key][arrayIndex], arraySubParameter, arraySubKey)) {
                                changedParameters[key] = changedArrayParameters;
                            }
                        });
                    });
                } else if (key.search(/^\$.*$/) === -1) {
                    try {
                        if (!angular.equals(parameter.value, parametersFromCache[key].value)) {
                            changedParameters[key] = parameter.value;
                        }
                    } catch (e) {
                        $log.error('NVM::Error');
                        $log.debug('  event: %o', e);
                        $log.debug('  key: %o', key);
                    }
                }

                return angular.isDefined(changedParameters[key]);
            };

            config = config || {};

            var doBeforePromise = angular.isFunction(config.doBefore) ? config.doBefore() : $q.when();
            var doAfterDeferred = $q.defer();
            var doAfterPromise = doAfterDeferred.promise;

            doBeforePromise
                .then(function () {
                    if ($scope.form.$valid || config.ignoreInvalidForm) {
                        var changedParameters = {};
                        var parameters = angular.copy(config.customParameters) || angular.copy($scope.parameters);
                        var parametersFromCache = Model.getParametersFromCache();

                        angular.forEach(parameters, function (parameter, key) {
                            getChangedParameters(changedParameters, parametersFromCache, parameter, key);
                        });
                        if (config.extendParameters) {
                            angular.forEach(config.extendParameters, function (parameter, key) {
                                changedParameters[key] = parameter;
                            });
                        }

                        if (!angular.equals({}, changedParameters)) {
                            $http.post('Parameters', changedParameters)
                                .then(function (response) {
                                    if (response && response.data && response.data.message !== '#logout#') {
                                        Model.updateParametersCache(parameters);

                                        if (!config.disableSuccessPopup) {
                                            $scope.showSuccessPopup()
                                                .finally(function () {

                                                    $scope.$broadcast('Form::Set::Success', changedParameters);
                                                    doAfterDeferred.resolve();
                                                });
                                        } else {

                                            $scope.$broadcast('Form::Set::Success', changedParameters);

                                            doAfterDeferred.resolve();
                                        }
                                    } else {
                                        $scope.$broadcast('Form::Set::Success', changedParameters);
                                        $window.sessionStorage.clear();
                                        Model.removeAll(false);
                                        Navigation.History.remove();
                                        State.clear();
                                        $location.path('Login');
                                    }
                                })
                                .catch(function (response) {
                                    if (response && response.data && response.data.message === '#savepar#' && response.status === 400) {
                                        var popupData = {
                                            title: $scope.text.Error,
                                            message: $scope.text.ParametersNotStored,
                                            buttons: [
                                                {text: $scope.text.OK, value: $scope.config.button.ok}
                                            ]
                                        };

                                        Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                                    }

                                    $scope.$broadcast('Form::Set::Error', changedParameters);
                                    doAfterDeferred.reject();
                                });
                        } else {
                            doAfterDeferred.resolve();
                        }
                    } else {
                        doAfterDeferred.reject();
                    }
                })
                .catch(doAfterDeferred.reject);

            doAfterPromise
                .then(function () {
                    if (angular.isFunction(config.doAfter)) {
                        config.doAfter();
                    }
                });
        };

        $scope.processFormWithParameters = function (customParameters, config) {

            config = config || {};

            var doBeforePromise = angular.isFunction(config.doBefore) ? config.doBefore() : $q.when();
            var doAfterDeferred = $q.defer();
            var doAfterPromise = doAfterDeferred.promise;

            doBeforePromise
                .then(function () {
                    if ($scope.form.$valid || config.ignoreInvalidForm) {

                        if (!angular.equals({}, customParameters)) {
                            $http.post('Parameters', customParameters)
                                .then(function () {
                                    Model.updateParametersCache(angular.copy($scope.parameters));

                                    if (!config.disableSuccessPopup) {
                                        $scope.showSuccessPopup()
                                            .finally(function () {
                                                $scope.$broadcast('Form::Set::Success', customParameters);
                                                doAfterDeferred.resolve();
                                            });
                                    } else {
                                        $scope.$broadcast('Form::Set::Success', customParameters);
                                        doAfterDeferred.resolve();
                                    }
                                })
                                .catch(function (response) {
                                    if (response && response.data && response.data.message === '#savepar#' && response.status === 400) {
                                        var popupData = {
                                            title: $scope.text.Error,
                                            message: $scope.text.ParametersNotStored,
                                            buttons: [
                                                {text: $scope.text.OK, value: $scope.config.button.ok}
                                            ]
                                        };

                                        Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                                    }

                                    $scope.$broadcast('Form::Set::Error', customParameters);
                                    doAfterDeferred.reject();
                                });
                        } else {
                            doAfterDeferred.resolve();
                        }
                    } else {
                        doAfterDeferred.reject();
                    }
                })
                .catch(doAfterDeferred.reject);

            doAfterPromise
                .then(function () {
                    if (angular.isFunction(config.doAfter)) {
                        config.doAfter();
                    }
                });
        };

        $scope.cancelForm = function (data) {
            Model.restoreParametersFromCache();
            if (data === undefined) {
                $scope.$broadcast('Form::Cancel');
            } else {
                $scope.$broadcast('Form::Cancel', {url: data.url});
            }
        };

        $scope.getPictureName = function () {
            pictureName = '';
            switch ($scope.parameters.PhoneModel.value) {
                case Config.phoneModel['Maxwell 4']:
                    pictureName = 'Maxwell4.png';
                    break;
                case Config.phoneModel['Maxwell 3']:
                case Config.phoneModel['Maxwell 2']:
                    pictureName = 'Maxwell3.png';
                    break;
                case Config.phoneModel['Maxwell Basic']:
                    pictureName = 'Maxwellb.png';
                    break;
            }
            return pictureName
        };

        var updateExtensionModulesRouting = function () {
            $location.path(Navigation.Menu.getUrl(Model.menu.menu, 'Settings/DeskPhone/KeysAndLEDsEMs/KeysAndLEDs'));
        };

        $scope.isExtensionModuleEnabled = function () {
            if ($location.path().indexOf("KeysAndLEDsEMs") !== -1) {
                if ($location.path().indexOf("KeysAndLEDsEMs/KeysAndLEDs") !== -1) {
                }
                else if (!$scope.state['ExtensionBoard.1.Status'] || $scope.parameters.MenuExtensionModule1.value !== '1') {
                    $location.path(Navigation.Menu.getUrl(Model.menu.menu, 'Settings/DeskPhone/KeysAndLEDsEMs/KeysAndLEDs'));
                }
                else if ($location.path().indexOf("ExtensionModule2") !== -1 && (!$scope.state['ExtensionBoard.2.Status'] || $scope.parameters.MenuExtensionModule2.value !== '1')) {
                    $location.path(Navigation.Menu.getUrl(Model.menu.menu, 'Settings/DeskPhone/KeysAndLEDsEMs/ExtensionModule1'));
                }
                else if ($location.path().indexOf("ExtensionModule3") !== -1 && (!$scope.state['ExtensionBoard.3.Status'] || $scope.parameters.MenuExtensionModule3.value !== '1')) {
                    $location.path(Navigation.Menu.getUrl(Model.menu.menu, 'Settings/DeskPhone/KeysAndLEDsEMs/ExtensionModule2'));
                }
            }
            return $scope.state['ExtensionBoard.1.Status'];
        };

        $scope.isLocked = function (flags) {
            return Tools.checkFlag(flags, $scope.config.flags.ui_lock);
        };

        $scope.isRequired = function (flags) {
            return Tools.checkFlag(flags, $scope.config.flags.required);
        };

        var availableMnemonics = [];

        $scope.initMnemonics = function () {
            availableMnemonics.push($scope.parameters.LDAPFirstName.value);
            availableMnemonics.push($scope.parameters.LDAPSurname.value);
            availableMnemonics.push($scope.parameters.LDAPPhoneHome.value);
            availableMnemonics.push($scope.parameters.LDAPPhoneOffice.value);
            availableMnemonics.push($scope.parameters.LDAPPhoneMobile.value);
            availableMnemonics.push($scope.parameters.LDAPEmail.value);
            availableMnemonics.push($scope.parameters.LDAPFax.value);
            availableMnemonics.push($scope.parameters.LDAPCompany.value);
            availableMnemonics.push($scope.parameters.LDAPStreet.value);
            availableMnemonics.push($scope.parameters.LDAPCity.value);
            availableMnemonics.push($scope.parameters.LDAPZIP.value);
            availableMnemonics.push($scope.parameters.LDAPCountry.value);
        };

        $scope.changeMnemonic = function (mnemonicName, oldMnemonicName) {
            var index = availableMnemonics.indexOf(oldMnemonicName);
            if (index !== -1) {
                availableMnemonics[index] = mnemonicName;
            } else {
                availableMnemonics.push(mnemonicName);
            }
        };

        $scope.validateDisplayFormat = function (newValue) {
            var re = /%(\w+)/g;
            var m;
            var missingMnemonics = [];

            do {
                m = re.exec(newValue);
                if (m) {
                    var mnemonic = m[1];
                    if (availableMnemonics.indexOf(mnemonic) === -1) {
                        missingMnemonics.push(mnemonic);
                    }
                }
            } while (m);

            if (missingMnemonics.length > 0) {
                $scope.form.LDAPDisplayFormat.$setValidity('mnemonic', false);
                $scope.missingMnemonics = missingMnemonics;
            } else {
                $scope.form.LDAPDisplayFormat.$setValidity('mnemonic', true);
                $scope.missingMnemonics = [];
            }
        };

        $scope.changeValue = function (parameterName, value) {
            var parameter = Tools.getPropertyByString($scope.parameters, parameterName);

            if (parameter) {
                $scope.changeMnemonic(value, parameter.value);
                $scope.validateDisplayFormat($scope.parameters.LDAPDisplayFormat.value);
                parameter.value = value;
            }
        };

        $scope.updateValue = function (parameterName, value) {
            var parameter = Tools.getPropertyByString($scope.parameters, parameterName);

            if (parameter) {
                $scope.changeMnemonic(parameter.value, value);
                $scope.validateDisplayFormat($scope.parameters.LDAPDisplayFormat.value);
            }
        };

        $scope.getTextWithIndex = function (textKey, index) {
            return $scope.text[textKey + ':' + index] || index;
        };

        $scope.checkMenuCondition = Navigation.Menu.checkMenuCondition;

        $scope.isLogged = AuthenticationService.isLogged;
        $scope.isAdminLogged = AuthenticationService.isAdminLogged;
        $scope.isUserLogged = AuthenticationService.isUserLogged;

        $scope.$on('$routeChangeStart', function (event, nextRoute, currentRoute) {
            if (nextRoute.restricted && !AuthenticationService.isLogged()) {
                if (angular.isUndefined(currentRoute)) {
                    $location.path('Login');
                } else {
                    event.preventDefault();
                }
            } else if (!nextRoute.restricted && AuthenticationService.isLogged()) {
                event.preventDefault();

                if (angular.isUndefined(currentRoute)) {
                    var lastUrl = $window.sessionStorage.lastUrl;

                    if (lastUrl) {
                        $location.path(lastUrl);
                    } else {
                        UserService.logout();
                    }
                }
            } else if (nextRoute.restricted && Model.menu && Model.menu.menu && Model.menu.profile) {
                var allLinks = Model.menu.menu.concat(Model.menu.profile),
                    urlFromMenu = Navigation.Menu.getUrl(allLinks, nextRoute.params.path);

                if (urlFromMenu) {
                    $location.path(urlFromMenu);
                } else {
                    event.preventDefault();
                }
            }
        });

        $scope.isBroadsoftPhoneSystem = function () {
            return ($scope.parameters.PhoneSystem.value == $scope.config.phoneSystem['BROADSOFT']);
        };

        $scope.isCentilePhoneSystem = function () {
            return ($scope.parameters.PhoneSystem.value == $scope.config.phoneSystem['CENTILE']);
        };

        $scope.isVodafonePhoneSystem = function () {
            return ($scope.parameters.PhoneSystem.value == $scope.config.phoneSystem['VODAFONE']);
        };

        $scope.isVodiaPhoneSystem = function () {
            return ($scope.parameters.PhoneSystem.value == $scope.config.phoneSystem['VODIA'])
        }

        $scope.$on('$routeChangeSuccess', function (event, currentRoute, previousRoute) {
            $scope.parameters = Model.parameters;
            $scope.menu = Model.menu;
            $scope.text = Model.text;

            if ((/^Profile\/.*$/).test(currentRoute.params.path)) {
                $scope.activeMenu = $scope.activeMenu || [0];
                $scope.activeMenu.profilePage = true;
            } else {
                $scope.activeMenu = [];

                if (angular.isArray($scope.menu.menu) && $scope.menu.menu.length > 0) {
                    Navigation.Menu.checkActiveMenu($scope.menu.menu, $scope.activeMenu, currentRoute.params.path);
                    $scope.activeMenu = $scope.activeMenu.reverse();
                }
            }

            Navigation.History.update();
            State.stop();

            $scope.htmlTooltip = '<strong>' + $scope.text.AllowedSigns + ':</strong><br>' +
                $scope.text.CapitalLetters + ': A-Z<br>' +
                $scope.text.SmallLetters + ': a-z<br>' +
                $scope.text.Digits + ': 0-9<br>' +
                $scope.text.SpecialSigns + ':' + '\+\&\@\_\#\*\"\'\-\/\\\=\%\$\~\^\ \.\,\?\!\:\;\|\(\)\<\>\[\]\{\}]';

            LogoutTimer.restart();
        });

        $scope.$on('$routeChangeError', function (event, currentRoute, previousRoute, rejection) {
            $log.error('$routeChangeError');
            $log.debug('  event: %o', event);
            $log.debug('  currentRoute: %o', currentRoute);
            $log.debug('  previousRoute: %o', previousRoute);
            $log.debug('  rejection: %o', rejection);

            UserService.logout();
        });

        $scope.$on('Connection::Success', function (event, response) {
            if (response && response.config && response.config.restartLogoutTimer) {
                LogoutTimer.restart();
            }
        });

        $scope.$on('Connection::Error', function (event, response) {
            $log.error('Connection::Error');
            $log.debug('  event: %o', event);
            $log.debug('  response: %o', response);

            var popupData = {
                title: $scope.text.Error,
                buttons: [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ]
            };

            if (response && response.config && response.config.restartLogoutTimer) {
                LogoutTimer.restart();
            }

            if (response.data.message === '#admlog#' && response.status === 403) {
                popupData.message = $scope.text.AccessDenied;
            } else if (angular.isUndefined(response.data.message) || !(/^#[a-z]+#$/gi).test(response.data.message)) {
                popupData.message = $scope.text.ServerError;
            }

            var deferred = $q.defer();

            if (response.status === 401 || response.status === 403) {
                UserService.logout()
                    .finally(deferred.resolve);
            } else {
                deferred.resolve();
            }

            deferred.promise
                .then(function () {
                    if (angular.isString(popupData.message)) {
                        Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                    }
                });
        });

        $scope.$on('Connection::Offline', function (event, popupData) {
            popupData.message = $scope.text.ConnectionLost;
            Popup.show(popupData, {
                priority: $scope.config.popup.priority.error,
                layer: $scope.config.popup.layer.important
            });
        });

        $scope.$on('Connection::Online', function (event, popupData) {
            Popup.close(popupData.layer, popupData.id);
        });

        $scope.$on('Connection::AdminLogged', function (event) {
            var popupData = {
                title: $scope.text.Error,
                buttons: [{text: $scope.text.OK, value: $scope.config.button.ok}],
                message: $scope.text.AccessDenied
            };

            Popup.show(popupData, {priority: $scope.config.popup.priority.error})
                .then(function (popupButton) {
                    if (angular.isDefined(popupButton)) {
                        switch (popupButton) {
                            case $scope.config.button.ok:
                                UserService.logout();
                                break;
                        }
                    }
                });
        });

        $scope.$on('Language::Update', function (event, data) {
            Model.removeText(true);
            Model.getText()
                .then(function () {
                    $scope.text = Model.text;
                });
        });

        $scope.$on('State::Update', function (event, data) {
            let _state = JSON.parse(data);
            State.get({name: _state.name})
        });

        $scope.needToBeUpdateAfterDeleteAccount = {
            'CallDiverPage': false,
            'DialingPlansPage': false
        }

        $scope.getParametesRelatedWithDeletingAccount = function (pageName, popupData) {
            CALL_DIVERT_PAGE = 'CallDiverPage';
            DIALLING_PLANS_PAGE = 'DialingPlansPage';
            CONNECTIONS_PAGE = 'ConnectionsPage';

            if ($scope.needToBeUpdateAfterDeleteAccount[pageName] || pageName == CONNECTIONS_PAGE) {
                switch (pageName) {
                    case CALL_DIVERT_PAGE:
                        parametersToBeUpdated = ['CallDivertAll[]', 'CallDivertBusy[]', 'CallDivertNoAnswer[]'];
                        break;
                    case DIALLING_PLANS_PAGE:
                        parametersToBeUpdated = ['DiallingPlans[]'];
                        break;
                    case CONNECTIONS_PAGE:
                        parametersToBeUpdated = ['SIP[]'];
                        $scope.needToBeUpdateAfterDeleteAccount[CALL_DIVERT_PAGE] = true;
                        $scope.needToBeUpdateAfterDeleteAccount[DIALLING_PLANS_PAGE] = true;
                        break;
                    default:
                        console.log("Wrong Page Name!");
                        return false;
                }
                $q.all(parametersToBeUpdated.map(function (parameterName) {
                    return Model.getParameters({
                        removeCache: true,
                        parameterName: parameterName
                    })
                })).then(function () {
                    $scope.$broadcast('Parameters::SIP::AccountRemoved');
                    angular.isObject(popupData) ? Popup.close(popupData.layer, popupData.id) : null;
                    $scope.needToBeUpdateAfterDeleteAccount[pageName] = false;

                });
            }
        };

        $scope.$on('Parameters::SIP::AccountRemoved', function (event, data) {
            updateViewWithScopeParameters($scope.removedAccount)
        });

        $scope.getSIPAccounts = function () {
            accountArray = [];
            for (index = 0; index < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; index++) {
                accountArray.push($scope.parameters.SIP[index])
            }
            return accountArray
        }

        $scope.isMaxwell4 = function () {
            return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 4']);
        }

        $scope.isMaxwell3 = function () {
            return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 3']);
        }

        $scope.isMaxwell2 = function () {
            return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 2']);
        }

        $scope.inputType = [];

        $scope.hideShowPassword = function (element) {
            if ($scope.inputType[element] == 'password')
                $scope.inputType[element] = 'text';
            else
                $scope.inputType[element] = 'password';
        };

        $scope.tmpHideSectionIfMax4 = function () {
            return $scope.parameters.PhoneConfig.TmpHideNonFunctionalParts
        };
    }])
    .directive('tooltipSpecialPopup', function () {
        return {
            restrict: 'EA',
            replace: true,
            scope: {content: '@', placement: '@', animation: '&', isOpen: '&'},
            templateUrl: 'templates/tooltip.tpl.html'
        };
    })

    .directive('tooltipSpecial', ['$tooltip', function ($tooltip) {
        return $tooltip('tooltipSpecial', 'tooltip', 'mouseenter');
    }]);

angular.module('WebUI').controller('ChangeLanguageCtrl', ['$scope', 'Model', function ($scope, Model) {
    $scope.$on('Form::Set::Success', function (event, data) {
        Model.removeText(true);
        Model.getText()
            .then(function () {
                $scope.$parent.$parent.text = Model.text;
            });
    });
}]);

angular.module('WebUI').controller('ChangePasswordCtrl', ['$scope', '$window', 'UserService', 'Popup', function ($scope, $window, UserService, Popup) {
    var passwordFieldNameDef = {
            user: 'UserPassword',
            admin: 'AdminPassword'
        },

        textIdDef = {
            user: 'NewPasswordUser',
            admin: 'NewPasswordAdmin'
        };

    $scope.passwordFieldName = passwordFieldNameDef[$window.sessionStorage.type];
    $scope.textId = textIdDef[$window.sessionStorage.type];

    $scope.parameters[$scope.passwordFieldName].value = '';
    $scope.repeatPassword = '';

    $scope.$on('Form::Set::Success', function (event, data) {
        UserService.logout();
    });

    $scope.$on('Form::Cancel', function (event, data) {
        $scope.parameters[$scope.passwordFieldName].value = '';
        $scope.repeatPassword = '';
    });

}]);

angular.module('WebUI').controller('ChangePINCtrl', ['$scope', '$http', 'Popup', function ($scope, $http, Popup) {
    $scope.currentPIN = '';
    $scope.newPIN = '';
    $scope.repeatPIN = '';

    $scope.setNewPIN = function () {
        if ($scope.currentPIN && $scope.newPIN && $scope.newPIN === $scope.repeatPIN) {
            $http.post('PIN', {current: $scope.currentPIN, new: $scope.newPIN})
                .then(function (response) {
                    if (response && response.data && response.data === '#invalidPin#') {
                        var popupData = {
                            title: $scope.text.Error,
                            message: $scope.text.InvalidPIN,
                            buttons: [
                                {text: $scope.text.OK, value: $scope.config.button.ok}
                            ]
                        };

                        Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                    } else {
                        $scope.showSuccessPopup();
                    }
                });
        }
    };

    $scope.isNewPinEqualsOldErr = function () {
        if ($scope.form.CurrentPIN.$viewValue == $scope.form.NewPIN.$viewValue) {
            $scope.form.NewPIN.$setValidity('newPinEqualsOld', false);
        }
        else {
            $scope.form.NewPIN.$setValidity('newPinEqualsOld', true);
        }
    }

    $scope.$on('Form::Cancel', function (event, data) {
        $scope.currentPIN = '';
        $scope.newPIN = '';
        $scope.repeatPIN = '';
    });
}]);

angular.module('WebUI').controller('LoginCtrl', ['$scope', 'Model', 'UserService', 'Popup', 'Status', function ($scope, Model, UserService, Popup, Status) {
    $scope.password = '';
    $scope.loginInProgress = false;
    Status.checkOffline.stop();
    Status.keepAlive.stop();
    $scope.loginItems=['Admin','User'];
    $scope.selectedLogin =$scope.loginItems[0];

    $scope.login = function () {
        if (!$scope.loginInProgress && $scope.password.length > 0) {
            $scope.loginInProgress = true;
            Status.checkOffline.start();
            Status.keepAlive.start();
            UserService.login($scope.password, $scope.selectedLogin)
                .catch(function (response) {
                    $scope.loginInProgress = false;
                    if (response && response.data && response.data.message === '#passerr#' && response.status === 401) {
                        var popupData = {
                            title: $scope.text.Error,
                            message: $scope.text.InvalidPassword,
                            buttons: [
                                {text: $scope.text.OK, value: $scope.config.button.ok}
                            ]
                        };

                        Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                    }
                });
        }
    };

    $scope.$on('Form::Set::Success', function (event, data) {
        Model.removeText(true);
        Model.getText()
            .then(function () {
                $scope.$parent.$parent.text = Model.text;
            });
    });
}]);

angular.module('WebUI').controller('FileUploadCtrl', ['$rootScope', '$scope', '$window', '$log', 'FileUploader', 'LogoutTimer', 'Status', function ($rootScope, $scope, $window, $log, FileUploader, LogoutTimer, Status) {
    $scope.uploader = $scope.uploader || new FileUploader();

    var onAfterAddingFile = $scope.uploader.onAfterAddingFile,
        onWhenAddingFileFailed = $scope.uploader.onWhenAddingFileFailed,
        onAfterAddingAll = $scope.uploader.onAfterAddingAll,
        onBeforeUploadItem = $scope.uploader.onBeforeUploadItem,
        onProgressItem = $scope.uploader.onProgressItem,
        onSuccessItem = $scope.uploader.onSuccessItem,
        onErrorItem = $scope.uploader.onErrorItem,
        onCancelItem = $scope.uploader.onCancelItem,
        onCompleteItem = $scope.uploader.onCompleteItem,
        onProgressAll = $scope.uploader.onProgressAll,
        onCompleteAll = $scope.uploader.onCompleteAll;

    FileUploader.FileSelect.prototype.isEmptyAfterSelection = function () {
        return true;
    };

    $scope.uploader.uploadWithKeepAlive = function (item) {
        Status.keepAlive.startOnce()
            .then(function () {
                if (item) {
                    $scope.uploader.uploadItem(item);
                } else {
                    $scope.uploader.uploadAll();
                }
            });
    };

    $scope.uploader.filters.push({
        name: 'globalFileSize',
        fn: function (file) {
            return file.size < $scope.config.maxUploadFileSize;
        }
    });

    $scope.uploader.onAfterAddingFile = function (item) {
        $log.info('onAfterAddingFile: %s', item.file.name);
        $log.debug('  item: %o', item);

        onAfterAddingFile(item);
    };

    $scope.uploader.onWhenAddingFileFailed = function (item, filter, options) {
        $log.warn('onWhenAddingFileFailed: %s (%s)', item.name, filter.name);
        $log.debug('  item: %o', item);
        $log.debug('  filter: %o', filter);
        $log.debug('  options: %o', options);

        onWhenAddingFileFailed(item, filter, options);
    };

    $scope.uploader.onAfterAddingAll = function (addedItems) {
        $log.info('onAfterAddingAll');
        $log.debug('  addedItems: %o', addedItems);

        onAfterAddingAll(addedItems);
    };

    $scope.uploader.onBeforeUploadItem = function (item) {
        $log.info('onBeforeUploadItem: %s', item.file.name);
        $log.debug('  item: %o', item);

        onBeforeUploadItem(item);

        item.headers = item.headers || {};
        item.headers.Authorization = 'Bearer ' + $window.sessionStorage.token;

        Status.checkOffline.stop();
    };

    $scope.uploader.onProgressItem = function (item, progress) {
        $log.info('onProgressItem: %s (%d%%)', item.file.name, progress);
        $log.debug('  item: %o', item);
        $log.debug('  progress: %d', progress);

        onProgressItem(item, progress);
    };

    $scope.uploader.onSuccessItem = function (item, response, status, headers) {
        $log.info('onSuccessItem: %s', item.file.name);
        $log.debug('  item: %o', item);
        $log.debug('  response: %o', response);
        $log.debug('  status: %d', status);
        $log.debug('  headers: %o', headers);

        onSuccessItem(item, response, status, headers);
    };

    $scope.uploader.onErrorItem = function (item, response, status, headers) {
        $log.error('onErrorItem: %s', item.file.name);
        $log.debug('  item: %o', item);
        $log.debug('  response: %o', response);
        $log.debug('  status: %d', status);
        $log.debug('  headers: %o', headers);

        onErrorItem(item, response, status, headers);

        if (status > 0) {
            $rootScope.$broadcast('Connection::Error', {
                data: angular.copy(response),
                status: status
            });
        }
    };

    $scope.uploader.onCancelItem = function (item, response, status, headers) {
        $log.info('onCancelItem: %s', item.file.name);
        $log.debug('  item: %o', item);
        $log.debug('  response: %o', response);
        $log.debug('  status: %d', status);
        $log.debug('  headers: %o', headers);

        onCancelItem(item, response, status, headers);
    };

    $scope.uploader.onCompleteItem = function (item, response, status, headers) {
        $log.info('onCompleteItem: %s', item.file.name);
        $log.debug('  item: %o', item);
        $log.debug('  response: %o', response);
        $log.debug('  status: %d', status);
        $log.debug('  headers: %o', headers);

        onCompleteItem(item, response, status, headers);
    };

    $scope.uploader.onProgressAll = function (progress) {
        $log.info('onProgressAll: %d', progress);
        $log.debug('  progress: %d', progress);

        onProgressAll(progress);
    };

    $scope.uploader.onCompleteAll = function () {
        $log.info('onCompleteAll');

        onCompleteAll();

        Status.checkOffline.start();
    };

    $scope.uploader.removeNotUploadingItems = function () {
        var removableItems = $scope.uploader.queue.filter(function (item) {
            return !item.isUploading;
        });

        angular.forEach(removableItems, function (item) {
            item.remove();
        });
    };

    $scope.$watch('uploader.isUploading', function (newValue, oldValue) {
        if (newValue === true && oldValue === false) {
            LogoutTimer.cancel();
        }

        if (newValue === false && oldValue === true) {
            LogoutTimer.restart();
        }
    });

    $scope.$on('Form::Cancel', function (event, data) {
        $scope.uploader.removeNotUploadingItems();
    });

    $scope.$on('$destroy', function (event, data) {
        $scope.uploader.cancelAll();
    });
}]);

angular.module('WebUI').controller('ConnectionsCtrl', ['$scope', '$q', 'Popup', 'Model', 'State', '$rootScope', '$http', function ($scope, $q, Popup, Model, State, $rootScope, $http) {
    PAGE_NAME = 'ConnectionsPage';
    $scope.sipAccountsBeforeRemoveFromScope = $scope.parameters.SIP;
    $scope.removedAccount = null;
    $scope.accountEdited = null;
    $scope.defaultAccount = null;

    var popupData = {};

    getSIPAccountsFromCache = function () {
        sipAccountsArray = []
        parameterFromCache = Model.getParametersFromCache()
        for (accountIndex = 0; accountIndex < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; accountIndex++) {
            sipAccountsArray.push(parameterFromCache.SIP[accountIndex])
        }
        return sipAccountsArray
    };

    getSIPAccountFromScope = function () {
        sipAccountArray = [];
        for (accountIndex = 0; accountIndex < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; accountIndex++) {
            sipAccountArray.push(angular.copy($scope.parameters.SIP[accountIndex]))
        }
        return sipAccountArray;
    };

    updateViewWithScopeParameters = function (removedAccount) {
        $scope.parameters.SIP = Model.getParametersFromCache()['SIP'];
        $scope.parameters['DefaultAccount'].value = $scope.defaultAccount;
        Model.updateParametersCache($scope.parameters)
    };

    $scope.isConfigured = function (account) {

        return account.AuthenticationName.value !== '' &&
            account.Domain.value !== '' &&
            account.RegistrationServerRefreshTime.value !== '';
    };

    $scope.editAccount = function (accountId) {
        if (accountId == $scope.parameters.DefaultAccount.value) {
            $scope.parameters.SIP[accountId].Active.value = "1";
        }

        var isAnyActive = false;
        angular.forEach($scope.parameters.SIP, function (sipAccount, key) {
            if (sipAccount.Active.value == 1) {
                isAnyActive = true;
            }
        });
        if (!isAnyActive) {
            $scope.parameters.SIP[accountId].Active.value = "1";
            $scope.parameters.DefaultAccount.value = accountId;
        }

        $scope.accountEdited = accountId;
    };

    $scope.handleDNSQueryChange = function (accountId) {
        if ($scope.parameters.SIP[accountId].DNSQuery.value !== $scope.parameters.SIP[accountId].DNSQuery.options[0]) {
            $scope.parameters.SIP[accountId].FailoverServerEnabled.restoreFromCache();
            $scope.parameters.SIP[accountId].FailoverServerAddress.restoreFromCache();
            $scope.parameters.SIP[accountId].FailoverServerPort.restoreFromCache();
        }
    };

    $scope.handleFailoverServerEnabledChange = function (accountId) {
        if ($scope.parameters.SIP[accountId].FailoverServerEnabled.value === $scope.config.switch.no) {
            $scope.parameters.SIP[accountId].FailoverServerAddress.restoreFromCache();
            $scope.parameters.SIP[accountId].FailoverServerPort.restoreFromCache();
        }
    };

    $scope.handleSTUNEnabledChange = function (accountId) {
        if ($scope.parameters.SIP[accountId].STUNEnabled.value === $scope.config.switch.no) {
            $scope.parameters.SIP[accountId].STUNServerAddress.restoreFromCache();
            $scope.parameters.SIP[accountId].STUNServerPort.restoreFromCache();
            $scope.parameters.SIP[accountId].STUNRefreshTime.restoreFromCache();
        }
    };

    $scope.toggleAccountActive = function (accountId) {
        if (accountId !== null) {
            var parametersGroup = [
                $scope.parameters.SIP[accountId].AuthenticationName.value,
                $scope.parameters.SIP[accountId].AuthenticationPassword.value,
                $scope.parameters.SIP[accountId].Domain.value,
                $scope.parameters.SIP[accountId].RegistrationServerRefreshTime.value
            ];

            var allDefined = parametersGroup.every(function (element) {
                return (angular.isDefined(element) && element !== '');
            });

            $scope.parameters.SIP[accountId].Active.value = allDefined ? $scope.config.checkbox.checked : $scope.config.checkbox.unchecked;

            $scope.setDefaultSendAccount(accountId, false);
        }
    };

    State.getConnectionsStates();

    $scope.deleteAccount = function (accountId) {
        $scope.sipAccountsBeforeRemoveFromScope = getSIPAccountFromScope();
        $scope.parametersCacheBeforeRemove = Model.getParametersFromCache();
        $scope.removedAccount = accountId;

        showPopup('confirmation')
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            showPopup('wait');
                            $http.post('DeleteAccount', {accountId: accountId})
                                .then(function (res) {
                                    $scope.defaultAccount = res.data['defaultAccount'];
                                    waitAndUpdateCacheAndView(accountId, res.data['defaultAccount']);
                                })
                    }
                }
            })
    };

    waitAndUpdateCacheAndView = function (accountId, defaultAccount) {
        counter = 0;
        var interval = setInterval(function () {
            if (!$scope.isConfigured($scope.parameters.SIP[accountId])) {
                $scope.setDefaultAccount(accountId, true);
                updateCacheAfterDeletingAccount(accountId, defaultAccount)
                Popup.close(popupData.layer, popupData.id)
                clearInterval(interval)
            }
            if (counter == 2 * $scope.config.deleteAccountTimeout) {
                clearInterval(interval)
                $scope.getParametesRelatedWithDeletingAccount(PAGE_NAME, popupData);
            }
            counter++;
        }, 500);
    };

    updateCacheAfterDeletingAccount = function (accountId, defaultValue) {
        accountId = accountId.toString()
        parametersFromCache = Model.getParametersFromCache();

        parametersFromCache['SIP'] = $scope.parametersCacheBeforeRemove['SIP'];
        parametersFromCache['SIP'][accountId] = $scope.parameters['SIP'][accountId];
        parametersFromCache['SIP'][accountId].Active.value = 0;
        parametersFromCache['DefaultAccount'].value = defaultValue;

        Model.updateParametersCache(parametersFromCache)

    };

    isActive = function (account) {
        return account.Active.value == 1;

    };

    isEditedAndIsNotActive = function (accountEdited) {
        return $scope.parameters.SIP[accountEdited].Active.value !== $scope.config.checkbox.checked && $scope.parameters.DefaultAccount.value == accountEdited;
    };

    isDefaultAccountNotActive = function (accountEdited) {
        return ($scope.parameters.SIP[accountEdited].Active.value === $scope.config.checkbox.checked && $scope.parameters.DefaultAccount.value == 0);
    };

    $scope.setDefaultAccount = function (accountEdited, isDeletingAccount) {

        if (isDeletingAccount) {
            $scope.parameters.DefaultAccount.value = findNewDefaultConnectionNew(accountEdited, isDeletingAccount)
        } else if (isEditedAndIsNotActive(accountEdited) || isDefaultAccountNotActive(accountEdited)) {
            $scope.parameters.DefaultAccount.value = findNewDefaultConnectionNew(accountEdited, isDeletingAccount);
        }
    };

    findNewDefaultConnectionNew = function (accountEdited, isDeletingAccount) {

        defaulAccount = 0;

        if (isDeletingAccount) {
            sipAccounts = $scope.sipAccountsBeforeRemoveFromScope
        } else {
            sipAccounts = $scope.parameters.SIP
        }

        for (accountIndex = 0; accountIndex < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; accountIndex++) {
            var isNextDefaultAccount = (isActive(sipAccounts[accountIndex]) && $scope.isConfigured(sipAccounts[accountIndex]))
            if (isNextDefaultAccount) {
                if (isDeletingAccount && accountEdited == accountIndex) {

                } else {
                    defaulAccount = accountIndex;
                    break;
                }

            }
        }
        return defaulAccount;
    }

    showPopup = function (type) {
        switch (type) {
            case 'confirmation':
                popupData.title = $scope.text.DeleteConnection;
                popupData.message = $scope.text.DeleteConnectionPopup;
                popupData.buttons = [
                    {text: $scope.text.Yes, value: $scope.config.button.yes},
                    {text: $scope.text.No, value: $scope.config.button.no}
                ];
                param = {priority: $scope.config.popup.priority.user};
                break;
            case 'wait':
                popupData.message = $scope.text.PleaseWait;
                popupData.buttons = [];
                Popup.show(popupData, {priority: $scope.config.popup.priority.info});
                param = {priority: $scope.config.popup.priority.info};
                break;
            case 'Error':
                popupData.title = $scope.text.DeleteConnection;
                popupData.message = $scope.text.DeleteConnectionError;
                popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                param = {priority: $scope.config.popup.priority.user, force: true};
                break;
            case 'ServerError':
                popupData.title = $scope.text.DeleteConnection;
                popupData.message = $scope.text.ServerError;
                popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                param = {priority: $scope.config.popup.priority.user, force: true};
                break;
        }
        return Popup.show(popupData, param);
    };

    $scope.$on('Form::Set::Success', function (event, data) {
        if ($scope.accountEdited !== null) {
            $scope.accountEdited = null;
        }
    });

    $scope.$on('Form::Cancel', function (event, data) {
        $scope.accountEdited = null;
    });

}]);

angular.module('WebUI').controller('StatusConnectionsCtrl', ['$scope', 'State', function ($scope, State) {

    State.getConnectionsStates()
    State.getExtensionModulesStates()

}]);

angular.module('WebUI').controller('IPCtrl', ['$scope', 'Popup', function ($scope, Popup) {
    $scope.$on('Form::Set::Success', function (event, data) {
        var popupData = {
            title: $scope.text.IP,
            message: $scope.text.NetworkInterfaceTerminated
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.user});
    });

    $scope.handleNetworkTypeChange = function () {
        if ($scope.parameters.NetworkType.value === $scope.parameters.NetworkType.options[0]) {
            $scope.parameters.IPv6Address.restoreFromCache();
            $scope.parameters.IPv6PrefixLength.restoreFromCache();
            $scope.parameters.IPv6PreferredDNSServer.restoreFromCache();
            $scope.parameters.IPv6AlternateDNSServer.restoreFromCache();
        }

        if ($scope.parameters.NetworkType.value === $scope.parameters.NetworkType.options[1]) {
            $scope.parameters.IPv4Address.restoreFromCache();
            $scope.parameters.IPv4SubnetMask.restoreFromCache();
            $scope.parameters.IPv4StandardGateway.restoreFromCache();
            $scope.parameters.IPv4PreferredDNSServer.restoreFromCache();
            $scope.parameters.IPv4AlternateDNSServer.restoreFromCache();
        }
    };

    $scope.handleIPAddressTypeChange = function () {
        if ($scope.parameters.IPAddressType.value === $scope.parameters.IPAddressType.options[1]) {
            $scope.parameters.IPv4Address.restoreFromCache();
            $scope.parameters.IPv4SubnetMask.restoreFromCache();
            $scope.parameters.IPv4StandardGateway.restoreFromCache();
            $scope.parameters.IPv4PreferredDNSServer.restoreFromCache();
            $scope.parameters.IPv4AlternateDNSServer.restoreFromCache();
        }
    };
}]);

angular.module('WebUI').controller('LANCtrl', ['$scope', 'Popup', function ($scope, Popup) {

    var vlanSettings = null;

    $scope.processFormLan = function () {
        if (vlanSettings == '1' || vlanSettings == '2') {
            $scope.processForm({
                extendParameters: {
                    VLANLocked: '2'
                },
                ignoreInvalidForm: true,
            });
        } else if (vlanSettings == '0') {
            $scope.processForm({
                extendParameters: {
                    VLANLocked: '0'
                },
                ignoreInvalidForm: true,
            });
        } else {
            $scope.processForm();
        }
    }

    $scope.$on('Form::Set::Success', function (event, data) {
        var popupData = {
            title: $scope.text.IP,
            message: $scope.text.NetworkInterfaceTerminated
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.user});
    });

    $scope.handleProxyServerActiveChange = function () {
        if ($scope.parameters.ProxyServerActive.value === $scope.parameters.ProxyServerActive.options[1]) {
            $scope.parameters.ProxyServerAddress.restoreFromCache();
            $scope.parameters.ProxyServerPort.restoreFromCache();
        }
    };

    $scope.handleVLANTaggingChange = function () {
        if ($scope.parameters.VLANTagging.value === $scope.parameters.VLANTagging.options[0]) {
            $scope.parameters.VLANIdentifierLAN.restoreFromCache();
            $scope.parameters.VLANPriorityLAN.restoreFromCache();
            $scope.parameters.VLANIdentifierPC.restoreFromCache();
            $scope.parameters.VLANPriorityPC.restoreFromCache();
        }

        if ($scope.parameters.VLANTagging.value === $scope.parameters.VLANTagging.options[1]) {
            $scope.parameters.VLANIdentifierPC.restoreFromCache();
            $scope.parameters.VLANPriorityPC.restoreFromCache();
        }
        $scope.setVlanSettings();
    };

    $scope.arePCandLANVLANIdentifiersTheSameErr = function () {
        if ($scope.parameters.VLANTagging.value === $scope.parameters.VLANTagging.options[2] &&
            $scope.parameters.VLANIdentifierLAN.value === $scope.parameters.VLANIdentifierPC.value) {
            $scope.form.VLANIdentifierPC.$setValidity('pcAndLANIDsAreEqual', false);
            $scope.form.VLANIdentifierLAN.$setValidity('pcAndLANIDsAreEqual', false);
        }
        else {
            $scope.form.VLANIdentifierPC.$setValidity('pcAndLANIDsAreEqual', true);
            $scope.form.VLANIdentifierLAN.$setValidity('pcAndLANIDsAreEqual', true);
        }
        $scope.setVlanSettings();
    }

    $scope.setVlanSettings = function () {
        vlanSettings = $scope.parameters.VLANTagging.value;
    }
}]);

angular.module('WebUI').controller('RebootAndResetCtrl', ['$scope', 'State', 'UserService', 'Popup', function ($scope, State, UserService, Popup) {
    var popupData = {
        buttons: [
            {text: $scope.text.Yes, value: $scope.config.button.yes},
            {text: $scope.text.No, value: $scope.config.button.no}
        ]
    };

    $scope.reboot = function () {
        popupData.title = $scope.text.Reboot;
        popupData.message = $scope.text.RebootPopup;

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            State.set({'System.Reboot': 5})
                                .then(function () {
                                    UserService.logout();
                                });
                            break;
                    }
                }
            });
    };

    $scope.reset = function () {
        popupData.title = $scope.text.Reset;
        popupData.message = $scope.text.ResetPopup;

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            State.set({'System.Reset': 5})
                                .then(function () {
                                    UserService.logout();
                                });
                            break;
                    }
                }
            });
    };
}]);

angular.module('WebUI').controller('ProvisioningAndConfigurationCtrl', ['$scope', 'State', 'Popup', '$timeout', function ($scope, State, Popup, $timeout) {

    showProvisioningInProgresPopUp = function () {
        Popup.show({
            message: $scope.text.ProvisioningInProgress
        }, {priority: $scope.config.popup.priority.info, size: 'sm', closable: true});
    }

    var startProvisioning = function (popupData) {
        State.set({'Provisioning': $scope.config.provisioningState['REQUEST']});
        showProvisioningInProgresPopUp();

        $scope.longProvisioningTimer = $timeout(function () {
            var popupData = {
                title: $scope.text.AutoConfiguration,
                message: $scope.text.AutoConfigurationTakesTooLong,
                buttons: [
                    {text: $scope.text.Yes, value: $scope.config.button.yes},
                    {text: $scope.text.No, value: $scope.config.button.no}
                ]
            };

            if (State.state['Provisioning'] == $scope.config.provisioningState['REQUEST']) {
                Popup.show(popupData, {priority: $scope.config.popup.priority.user})
                    .then(function (popupButton) {
                        if (angular.isDefined(popupButton)) {
                            switch (popupButton) {
                                case $scope.config.button.yes:
                                    break;
                                case $scope.config.button.no:
                                    State.state['Provisioning'] == $scope.config.provisioningState['REQUEST'] ? showProvisioningInProgresPopUp() : null;
                                    break;
                            }
                        }
                    })
            }
        }, $scope.config.provisioningPopupTimeout);
    }

    $scope.$on('Parameters::Get::ProvisioningState', function (event, state) {
        $timeout.cancel($scope.longProvisioningTimer);
        var popupDataFinish = {};

        if (state == $scope.config.provisioningState['SUCCESS'] || state == $scope.config.provisioningState['PART_SUCCESS']) {
            popupDataFinish.message = $scope.text.ProvisioningSuccessful;
            State.stop('Provisioning');
            popupDataFinish.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
        } else if (state === $scope.config.provisioningState['FAIL']) {
            getHttpErrorCode()
        }
        Popup.show(popupDataFinish, {priority: $scope.config.popup.priority.user})
    });

    $scope.startAutoConfiguration = function () {
        var popupData = {
            title: $scope.text.StartAutoConfiguration,
            message: $scope.text.StartAutoConfigurationPopup,
            buttons: [
                {text: $scope.text.Yes, value: $scope.config.button.yes},
                {text: $scope.text.No, value: $scope.config.button.no}
            ]
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            var config = {};
                            config.doAfter = startProvisioning(popupData);
                            $scope.processForm(config);
                            break;
                    }
                }
            });
    };

    getHttpErrorCode = function () {
        State.get({name: 'Provisioning.HttpErrorCode', interval: 5})
            .then(null, null, function (response_http_code) {
                var popupDataFinish = {};
                console.log(response_http_code.data['Provisioning.HttpErrorCode']);
                State.stop('Provisioning.HttpErrorCode');
                State.clear('Provisioning.HttpErrorCode');
                if (response_http_code.data['Provisioning.HttpErrorCode'] >= 400 && response_http_code.data['Provisioning.HttpErrorCode'] < 600) {
                    State.stop('Provisioning');
                    popupDataFinish.title = $scope.text.Error;
                    popupDataFinish.message = $scope.text.ProvisioningFailedCode + response_http_code.data['Provisioning.HttpErrorCode'];
                    popupDataFinish.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                    Popup.show(popupDataFinish, {priority: $scope.config.popup.priority.user});
                } else {
                    popupDataFinish.title = $scope.text.Error;
                    popupDataFinish.message = $scope.text.ProvisioningFailed;
                    State.stop('Provisioning');
                    popupDataFinish.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                    Popup.show(popupDataFinish, {priority: $scope.config.popup.priority.user});
                }
            });
    }
}]);

angular.module('WebUI').controller('SaveAndRestoreCtrl', ['$scope', '$http', 'FileUploader', 'State', 'Popup', 'Model', 'Tools', 'SseService', '$filter', function ($scope, $http, FileUploader, State, Popup, Model, Tools, SseService, $filter) {
    var popupData = {};

    var saveSettings = function (url, filename) {
        popupData.title = $scope.text.SaveAndRestore;
        popupData.message = $scope.text.DevicePreparingSettings;
        popupData.buttons = [];

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.get(url, {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            responseType: 'blob',
            restartLogoutTimer: true
        })
            .then(function (response) {
                Popup.close(popupData.layer, popupData.id);
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), filename);
            })
            .catch(function () {
                popupData.title = $scope.text.Error;
                popupData.message = $scope.text.ServerError;
                popupData.buttons = [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ];

                Popup.show(popupData, {priority: $scope.config.popup.priority.error});
            });
    };

    $scope.uploader = new FileUploader({
        url: 'RestoreSettings',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['cfg'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });

    $scope.uploader.onBeforeUploadItem = function (item) {
        popupData.title = $scope.text.SaveAndRestore;
        popupData.message = $scope.text.PleaseWaitSettingsRestored;
        popupData.buttons = [];

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    };

    $scope.uploader.onWhenAddingFileFailed = function (item) {
        popupData.title = $scope.text.SaveAndRestore;
        popupData.message = $filter('Replacement')($scope.text.FileIsNotSupported, [item.name]);
        popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    }

    var refreshParameters = function () {
        Model.removeParameters(true);
        Model.getParameters({removeCache: true})
            .then(function () {
                Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                SseService.start();
            });
    };
    var monitorState = function () {
        SseService.stop();
        State.get({name: 'Lighttpd.Ready', interval: 5})
            .then(null, null, function (response) {
                if (response && response.data && response.data['Lighttpd.Ready']) {
                    switch (response.data['Lighttpd.Ready']) {
                        case '1':
                            State.stop('Lighttpd.Ready');
                            refreshParameters();
                            break;
                        case '0':
                        default:
                            break;
                    }
                }
            });
    };

    $scope.uploader.onCompleteItem = function (item, response, status, headers) {
        popupData.buttons = [
            {text: $scope.text.OK, value: $scope.config.button.ok}
        ];
        //restore started
        if (response && response.message === '#restoresettings#') {
            switch (status) {
                case 200://ok
                    popupData.title = $scope.text.SaveAndRestore;
                    popupData.message = $scope.text.SettingsRestored;
                    break;

                case 400://bad request
                    popupData.title = $scope.text.Error;
                    popupData.message = $scope.text.FileIsCorupt;
                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                    break;

                case 405://method not allowed
                    popupData.title = $scope.text.Error;
                    popupData.message = $scope.text.ForbiddenToRestoreAllSettings;
                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                    break;

                default:
                    popupData.title = $scope.text.Error;
                    popupData.message = $scope.text.ServerError;
                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
            }
        } else {
            popupData.title = $scope.text.Error;
            popupData.message = $scope.text.ServerError;
            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
        }
        //wait until settings are really restored
        if (status == 200) {
            monitorState();
        }
    };

    $scope.saveAllSettings = function () {
        saveSettings('SaveAllSettings', 'AllSettings.cfg');
    };

    $scope.savePersonalizedSettings = function () {
        saveSettings('SavePersonalizedSettings', 'PersonalizedSettings.cfg');
    };
}]);

angular.module('WebUI').controller('LocalPhonebookCtrl', ['$scope', '$http', '$filter', 'FileUploader', 'Popup', 'State', 'Model', 'Tools', function ($scope, $http, $filter, FileUploader, Popup, State, Model, Tools) {
    var popupData = {};

    $scope.downloadPhonebook = function (){
        var downloadPhonebookURL = '/LocalPhonebook/Download';
        popupData.title = $scope.text.LocalPhonebook;
        popupData.message = $scope.text.LocalPhoneBookDownloading;
        popupData.buttons = [];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.post(downloadPhonebookURL)
            .then(function (response) {
                Popup.close(popupData.layer, popupData.id);
            })
            .catch(function () {
                popupData.title = $scope.text.Error;
                popupData.message = $scope.text.ServerError;
                popupData.buttons = [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ];
                Popup.show(popupData, {priority: $scope.config.popup.priority.error});
            });

    }

    var saveSettings = function (url, filename) {
        popupData.title = $scope.text.SaveAndRestore;
        popupData.message = $scope.text.DevicePreparingSettings;
        popupData.buttons = [];

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.get(url, {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            responseType: 'blob',
            restartLogoutTimer: true
        })
            .then(function (response) {
                Popup.close(popupData.layer, popupData.id);
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), filename);
            })
            .catch(function () {
                popupData.title = $scope.text.Error;
                popupData.message = $scope.text.ServerError;
                popupData.buttons = [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ];

                Popup.show(popupData, {priority: $scope.config.popup.priority.error});
            });
    };

    $scope.isUSBSectionVisible = function () {
        return $scope.parameters.PhoneConfig.UsbSupport == true && State.state['System.USB.Device'] == 1
    }

    $scope.uploader = new FileUploader({
        url: 'LocalPhonebook',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['xml', 'csv'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });

    $scope.uploader.onWhenAddingFileFailed = function (item) {
        popupData.title = $scope.text.LoadPhonebookFromPC;
        popupData.message = $filter('Replacement')($scope.text.FileIsNotSupported, [item.name]);
        popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    };

    var monitorState = function () {
        State.get({name: 'LocalDirectoryUpdate.Verify', interval: 2})
            .then(null, null, function (response) {
                if (response && response.data && response.data['LocalDirectoryUpdate.Verify']) {
                    switch (response.data['LocalDirectoryUpdate.Verify']) {
                        case 'request':
                            break;
                        case 'success':
                            $scope.showPopup($scope.text.LocalPhonebookImported);
                            break;
                        case 'fail':
                        case 'fail_LimitReached':
                        case 'fail_FileCorrupted':
                        case 'fail_FileDoesNotExist':
                        case 'fail_Internal':
                        default:
                            popupData.title = $scope.text.LoadPhonebookFromPC;
                            popupData.message = $scope.text.LocalPhonebookFailed;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            break;
                    }

                    if (response.data['LocalDirectoryUpdate.Verify'] !== 'request') {
                        State.stop('LocalDirectoryUpdate.Verify');
                    }
                }
            });
    };

    $scope.uploader.onCompleteItem = function (item, response, status, headers) {
        popupData.buttons = [
            {text: $scope.text.OK, value: $scope.config.button.ok}
        ];

        if (response && response.message === '#importphonebook#') {
            if (status === 200) {
                monitorState();
            }
            else {
                popupData.title = $scope.text.LoadPhonebookFromPC;
                popupData.message = $scope.text.FileIsCorupt;
                Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
            }
        } else {
            popupData.title = $scope.text.LoadPhonebookFromPC;
            popupData.message = $scope.text.ServerError;
            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
        }
    };

    $scope.saveLocalPhonebook = function () {
        saveSettings('SaveLocalPhonebook', 'LocalPhonebook.xml');
    };

    $scope.TransferLocalPhonebook = function (command) {
        var popupData = {
            buttons: [
                {text: $scope.text.Yes, value: $scope.config.button.yes},
                {text: $scope.text.No, value: $scope.config.button.no}
            ]
        };
        popupData.title = $scope.text.PhonebookUsb;
        popupData.message = $scope.text.ConfirmLocalPhonebookTransfer;

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            popupData.message = $scope.text.PleaseWait;
                            popupData.buttons = [];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            $http.post('/LocalPhonebook/' + command)
                                .then(function (response) {
                                    Popup.close(popupData.layer, popupData.id);
                                    switch (response.data.message) {
                                        case '#cmdok#':
                                            popupData.message = $scope.text.LocalPhonebookTransferred;
                                            break;
                                        case '#cmderr#':
                                        default:
                                            popupData.message = $scope.text.LocalPhonebookTransferFailed;
                                    }
                                    popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                                })
                                .catch(function (response) {
                                    popupData.message = $scope.text.LocalPhonebookTransferFailed;
                                    popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                                });
                            break;
                    }
                }
            });
    };

    State.getSystemUSBDeviceState();
    $scope.time_ = new Date(2001, 1, 1, $scope.parameters.SystemLocalPhonebookUpdateTime.value.split(":")[0], $scope.parameters.SystemLocalPhonebookUpdateTime.value.split(":")[1], 0, 0);

    $scope.time = function (value) {
        if (angular.isDefined(value)) {
            $scope.parameters.SystemLocalPhonebookUpdateTime.value = value.getHours() + ":" + value.getMinutes();
            $scope.time_.setHours(value.getHours());
            $scope.time_.setMinutes(value.getMinutes());
        } else {
            return $scope.time_;
        }
    };
}]);

angular.module('WebUI').controller('DeviceCtrl', ['$scope', '$http', 'State', 'Model', function ($scope, $http, State, Model) {
    var getInvalidCertificates = function () {
        var promise = $http.get('InvalidCertificates', {cache: false});

        promise
            .then(function (response) {
                $scope.InvalidCertificates = response.data;
            });

        return promise;
    };
    var getCurrentTime = function () {
        $http.get('CurrentTime', {cache: false})
            .then(function (resp) {
                $scope.parameters.Timestamp.value = resp.data;
                Model.updateParametersCache();
            });

    }

    var getMacId = function () {
        $http.get('MacId')
            .then(function (res) {
                $scope.parameters.MACid = {
                    "value": res.data[0].value
                }
                Model.updateParametersCache();
            })
    }

    $scope.isProvisioningOrDHCP = function () {
        return ($scope.parameters.AutoAdjustTime.value === 'NTPServiceEnabled' || $scope.parameters.AutoAdjustTime.value === "NTPServiceDHCP")
    }

    getInvalidCertificates();

    $scope.lastSynchronization = 0;

    $http.get('NetStatus', {cache: false})
        .then(function (response) {
            if (response && response.data) {
                $scope._saveNetStatusDataFromResponse(response)
            }

        });

    $scope._saveNetStatusDataFromResponse = function (response) {
        $scope.lanPort = $scope.getNetStatusDescription(response.data.LANPort)
        $scope.pcPort = $scope.getNetStatusDescription(response.data.PCPort)
    }

    $scope.getNetStatusDescription = function (status) {
        return (status.isConnected == 1) ? $scope.text['NetStatusSpeed:' + status.speed] + ' ' + $scope.text['NetStatusDuplex:' + status.duplex] : $scope.text.NotConnected
    };

    $scope.TimeServer = ($scope.parameters.TimeServerProvisioning.value === "") ? $scope.parameters.TimeServer.value : $scope.parameters.TimeServerProvisioning.value;

    State.getNetMacAddressState();
    State.getNtpLastSyncState();
    $scope.$on("State::Update", function (event, args) {
        var StateJson = JSON.parse(args);
        if (StateJson.name == "System.TimeChanged" || StateJson.name == "NTPLastSync") {
            getCurrentTime();
        }
    });
    getCurrentTime();
    getMacId();


}]);

angular.module('WebUI').controller('SecurityCtrl', ['$scope', '$http', '$filter', 'FileUploader', 'State', 'Popup', 'Tools', '$window', 'Status', function ($scope, $http, $filter, FileUploader, State, Popup, Tools, $window, Status) {
    var getCertificates = function () {
        var promise = $http.get('Certificates', {cache: false});

        promise
            .then(function (response) {
                $scope.certificates = response.data;
            });

        return promise;
    };

    var processCertificates = function (certificateFileNames, action) {
        var stateName = 'Security.Certificates.' + action;

        var popupData = {
            title: $scope.text.Security,
            message: $scope.text.PleaseWait
        };

        var createStateObject = function (certificateFileName) {
            var state = {};
            state[stateName] = certificateFileName;
            return state;
        };

        var sendCertificates = function () {
            State.set(createStateObject(certificateFileNames.shift()))
                .then(function () {
                    State.get({name: stateName, interval: 2})
                        .then(null, null, function (response) {
                            if (response && response.data && (response.data[stateName] === 'success' || response.data[stateName] === 'fail')) {
                                State.stop(stateName);

                                if (certificateFileNames.length > 0) {
                                    sendCertificates();
                                } else {
                                    getCertificates()
                                        .then(function () {
                                            Popup.close(popupData.layer, popupData.id);
                                            initCertificates();
                                        });
                                }
                            }
                        });
                });
        };

        certificateFileNames = angular.isArray(certificateFileNames) ? certificateFileNames : [certificateFileNames];

        if (certificateFileNames.length > 0) {
            Popup.show(popupData, {priority: $scope.config.popup.priority.info, chain: true});
            sendCertificates();
        }
    };

    var initCertificates = function () {
        $scope.certificateServer = null;
        $scope.certificateCA = null;
        $scope.certificatePending = null;
        $scope.certificateDetails = null;
        $scope.certificateDetailsType = null;
        $scope.certificateClientKeyUpload = false;
    };

    var uploadedCertificates = [];
    var uploadedCertKey = []

    var handleCertificateId = function (certificateId) {
        var id = certificateId;
        if (typeof $scope.certificates[certificateId] === 'undefined') {
            for (var index = 0, l = $scope.certificates.length; index < l; index++) {
                if ($scope.getCertificateName($scope.certificates[index]) == certificateId) {
                    id = index;
                    break;
                }
            }
        }

        return id;
    };

    $scope.isInvalidCertificate = function (certificateId) {
        return (certificateId !== null && $scope.certificates[handleCertificateId(certificateId)].status === 'invalid');
    };

    $scope.isAcceptedOrRejectedCertificate = function (certificate) {
        return function (certificate) {
            return (certificate && (certificate.status === 'accepted' || certificate.status === 'rejected'));
        };
    };

    $scope.isPendingOrInvalidCertificate = function (certificate) {
        return function (certificate) {
            return (certificate && (certificate.status === 'pending' || certificate.status === 'invalid'));
        };
    };

    $scope.getCertificateName = function (certificate) {
        return (certificate && certificate.subject ? certificate.subject.CN || certificate.subject.OU || certificate.subject.N : '');
    };

    $scope.acceptCertificate = function (certificateId) {
        processCertificates($scope.certificates[handleCertificateId(certificateId)].file, 'Accept');
    };

    $scope.rejectCertificate = function (certificateId) {
        processCertificates($scope.certificates[handleCertificateId(certificateId)].file, 'Reject');
    };

    $scope.removeCertificate = function (certificateId) {
        processCertificates($scope.certificates[handleCertificateId(certificateId)].file, 'Delete');
    };

    $scope.showDetails = function (certificateId, type) {
        $scope.certificateDetailsType = type;
        $scope.changeDetails(certificateId, type);
    };

    $scope.closeDetails = function () {
        $scope.certificateDetailsType = null;
    };

    $scope.changeDetails = function (certificateId, type) {
        if ($scope.certificateDetailsType === type) {
            $scope.certificateDetails = $scope.certificates[handleCertificateId(certificateId)];
        }
    };

    $scope.uploader = new FileUploader({
        url: 'Certificates',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['cer', 'crt', 'crl', 'der', 'pem'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });

    $scope.uploaderCert = new FileUploader({
        url: 'CertificatesClient'
    });


    $scope.uploader.onSuccessItem = function (item, response, status, headers) {
        var certificateFileName = response.file || item.file.name;

        if (certificateFileName) {
            uploadedCertificates.push(certificateFileName);
        }
    };

    $scope.uploader.onErrorItem = function (item, response, status, headers) {
        if (status === 400 && response && response.message) {
            var popupData = {
                title: $scope.text.Security,
                buttons: [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ]
            };

            switch (response.message) {
                case '#fileType#':
                    popupData.message = $filter('Replacement')($scope.text.FileIsNotSupported, [item.file.name]);
                    break;
            }

            Popup.show(popupData, {priority: $scope.config.popup.priority.info, chain: true});
        }
    };

    $scope.uploader.onCompleteAll = function () {
        processCertificates(uploadedCertificates, 'Accept');
    };

    $scope.uploaderCert.uploadAll = function () {
        uploadedCertKey.forEach(function (item) {
            $scope.uploaderCert.uploadItem(item)
        })

    }
    $scope.uploaderCert.onAfterAddingFile = function (item) {
        if (item.alias == 'cert') {
            item.formData.push({fileType: "cert"})
        } else {
            item.formData.push({fileType: "key"})
        }
        if ($scope.uploaderCert.queue.length > 1) {
            item.formData.push({isLastFile: true});
        }
        uploadedCertKey.push(item)
    }
    $scope.uploaderCert.onErrorItem = $scope.uploader.onErrorItem
    $scope.uploaderCert.onBeforeUploadItem = function (item) {
        item.headers = item.headers || {};
        item.headers.Authorization = 'Bearer ' + $window.sessionStorage.token;
        if ($scope.uploaderCert.queue.length == 1 && item.alias == 'cert') {
            item.formData.push({isLastFile: true});
        }
        Status.checkOffline.stop();
    };
    $scope.uploaderCert.removeNotUploadingItems = function () {
        console.log($scope.uploaderCert.queue.length);
        var removableItems = $scope.uploaderCert.queue.filter(function (item) {
            return !item.isUploading;
        });

        angular.forEach(removableItems, function (item) {
            var index = uploadedCertKey.indexOf(item)
            uploadedCertKey.splice(index, 1);
            item.remove();
        });
    }
    $scope.uploaderCert.onCompleteAll = function () {
        uploadedCertKey = [];
    };

    getCertificates()
        .then(initCertificates);
}]);

angular.module('WebUI').controller('AudioCtrl', ['$scope', 'Tools', function ($scope, Tools) {
    $scope.selectedCodec = $scope.availableCodec = null;

    $scope.selectedCodecUp = function () {
        var selectedCodecValid = $scope.selectedCodec !== null &&
            $scope.parameters.SelectedCodecs.value.indexOf($scope.selectedCodec) > 0;

        if (selectedCodecValid) {
            Tools.swapArrayElements($scope.parameters.SelectedCodecs.value, $scope.selectedCodec, -1);
        }
    };

    $scope.selectedCodecDown = function () {
        var selectedCodecValid = $scope.selectedCodec !== null &&
            $scope.parameters.SelectedCodecs.value.indexOf($scope.selectedCodec) >= 0 &&
            $scope.parameters.SelectedCodecs.value.indexOf($scope.selectedCodec) < $scope.parameters.SelectedCodecs.value.length - 1;

        if (selectedCodecValid) {
            Tools.swapArrayElements($scope.parameters.SelectedCodecs.value, $scope.selectedCodec, 1);
        }
    };

    $scope.selectedCodecRight = function () {
        var selectedCodecValid = $scope.selectedCodec !== null &&
            $scope.parameters.SelectedCodecs.value.indexOf($scope.selectedCodec) !== -1;

        if (selectedCodecValid) {
            $scope.parameters.SelectedCodecs.value.splice($scope.parameters.SelectedCodecs.value.indexOf($scope.selectedCodec), 1);
            $scope.selectedCodec = null;
        }
    };

    $scope.availableCodecLeft = function () {
        var availableCodecValid = $scope.availableCodec !== null &&
            $scope.parameters.AvailableCodecs.options.indexOf($scope.availableCodec) !== -1;

        if (availableCodecValid) {
            $scope.parameters.SelectedCodecs.value.unshift($scope.availableCodec);
            $scope.availableCodec = null;
        }
    };

    $scope.getNotSelectedCodecs = function (codec) {
        var parsedValue = Number(codec);
        return isNaN(parsedValue) || $scope.parameters.SelectedCodecs.value.indexOf(parsedValue) === -1;
    };

    $scope.handleVoiceQualityChange = function () {
        if ($scope.parameters.VoiceQuality.value === $scope.parameters.VoiceQuality.options[0]) {
            $scope.parameters.SelectedCodecs.value = [$scope.config.codecs['G.722'], $scope.config.codecs['G.711 alaw'], $scope.config.codecs['G.711 ulaw']];
        }

        if ($scope.parameters.VoiceQuality.value === $scope.parameters.VoiceQuality.options[1]) {
            $scope.parameters.SelectedCodecs.value = [$scope.config.codecs['iLBC'], $scope.config.codecs['G.726-32'], $scope.config.codecs['G.729']];
        }

        $scope.selectedCodec = $scope.availableCodec = null;
    };

    $scope.deselectCodecInList = function (list) {
        switch (list) {
            case 'SelectedCodecs':
                $scope.selectedCodec = null;
                break;

            case 'AvailableCodecs':
                $scope.availableCodec = null;
                break;
        }
    };
}]);

angular.module('WebUI').controller('PhoneSystemCtrl', ['$scope', '$http', function ($scope, $http) {
    currentPhoneSystem = $scope.parameters.PhoneSystem.value;

    $http.get('PhoneSystemDefaultsVal', {cache: true})
        .then(function (response) {
            if (response && response.data) {
                phoneSystemsDefaultValues = response.data;
            }
        });

    $scope.handlePhoneSystemChange = function () {
        phoneSystemWithVisibleParameters = $scope.config.phoneSystem['BROADSOFT'];

        if($scope.parameters.PhoneSystem.value == currentPhoneSystem && phoneSystemWithVisibleParameters != currentPhoneSystem){
            angular.forEach(phoneSystemsDefaultValues, function(parameter) {
                $scope.parameters[parameter.webui].restoreFromCache();
            } )
        }else{
            angular.forEach(phoneSystemsDefaultValues, function(parameter){
                $scope.parameters[parameter.webui].value = parameter.defaultvalues;
            })
        }
    };

    $scope.$on('Form::Set::Success', function (event, data) {

        if(currentPhoneSystem == $scope.config.phoneSystem["BINTEC-ELMEG_PBX"] && $scope.parameters.PhoneSystem.value != $scope.config.phoneSystem["BINTEC-ELMEG_PBX"]){

            functionKeyIndexes = getArrayOfKeysSetToPbxFeature($scope.parameters.FunctionKeys);
            softKeyIndexes = getArrayOfKeysSetToPbxFeature($scope.parameters.SoftKeys);

            if(functionKeyIndexes.length || softKeyIndexes.length){
                $http.post('ResetPbxFeatureKeys',{
                    'functionKeys': functionKeyIndexes,
                    'softKeys': softKeyIndexes
                });
            }
        }
        currentPhoneSystem = $scope.parameters.PhoneSystem.value;
    });

    $scope.isBLFcreateAutomatically = function () {
        return ($scope.parameters.AutomaticFKFilling.value == '1');
    };

    getArrayOfKeysSetToPbxFeature = function(groupOfKeys) {
        tmpArray = []
        angular.forEach(groupOfKeys,function (key,index){
            if(key.Type.value == $scope.config.functionKeyType.PbxFeature) {
                tmpArray.push(index)
            }
        });
        return tmpArray
    }

}]);

angular.module('WebUI').controller('DateAndTimeCtrl', ['$scope', '$http', 'State', function ($scope, $http, State) {
    var timestampMoment, timeMoment, dateMoment, offsetHours, offsetDays;
    $scope.tooltipText = null;

    var updateTime = function (oldMoment, newMoment) {
        oldMoment.hour(newMoment.hour());
        oldMoment.minute(newMoment.minute());
        oldMoment.second(newMoment.second());
    };

    var updateDate = function (oldMoment, newMoment) {
        oldMoment.year(newMoment.year());
        oldMoment.month(newMoment.month());
        oldMoment.date(newMoment.date());
    };

    var getTimezoneDSTFromMoment = function (timezone, srcMoment) {
        timezone = timezone.split('.')[1];

        var timezoneDST = 0;

        if (moment.tz(srcMoment, timezone).isDST() && $scope.parameters.AutoAdjustClockForDST.value === 'AutomatedDaylightSavingEnabled') {
            timezoneDST = 1;
        }

        return timezoneDST;
    };

    var getTimezoneOffsetFromMoment = function (timezone, srcMoment) {
        timezone = timezone.split('.')[1];

        var timezoneOffset = moment.tz(srcMoment, timezone).utcOffset() / 60;

        if (moment.tz(srcMoment, timezone).isDST() && $scope.parameters.AutoAdjustClockForDST.value === 'AutomatedDaylightSavingDisabled') {
            timezoneOffset--;
        }

        return timezoneOffset;
    };

    var updateMoments = function () {
        timestampMoment = moment.unix(Number($scope.parameters.Timestamp.value)).utc();
        timeMoment = moment();
        dateMoment = moment();

        timestampMoment.second(0);

        updateTime(timeMoment, timestampMoment);
        updateDate(dateMoment, timestampMoment);

        offsetHours = getTimezoneOffsetFromMoment($scope.parameters.TimeZone.value, timestampMoment);
        offsetDays = Math.floor((timeMoment.hour() + offsetHours) / 24);
        timeMoment.add(offsetHours, 'h');
        dateMoment.add(offsetDays, 'd');

        timestampMoment.utcOffset(offsetHours);
    };

    var updateModel = function (type, newMoment) {
        var newTimezoneDST, oldTimezoneDST, timezoneDeltaDST, timezoneOffset;

        oldTimezoneDST = getTimezoneDSTFromMoment($scope.parameters.TimeZone.value, timestampMoment);

        switch (type) {
            case 'time':
                updateTime(timeMoment, newMoment);
                updateTime(timestampMoment, newMoment);
                break;
            case 'date':
                updateDate(dateMoment, newMoment);
                updateDate(timestampMoment, newMoment);
                break;
        }

        newTimezoneDST = getTimezoneDSTFromMoment($scope.parameters.TimeZone.value, timestampMoment);

        timezoneDeltaDST = newTimezoneDST - oldTimezoneDST;
        timeMoment.add(timezoneDeltaDST, 'h');

        timezoneOffset = getTimezoneOffsetFromMoment($scope.parameters.TimeZone.value, timestampMoment);
        timestampMoment.utcOffset(timezoneOffset);

        $scope.parameters.Timestamp.value = timestampMoment.format('X');
    };

    updateMoments();

    $scope.getTimezoneText = function (textKey, index) {
        var timezone = index.split('.')[1];
        var timezoneMoment = moment(timestampMoment).tz(timezone);
        var limit = 12;

        while (limit-- && timezoneMoment.isDST()) {
            timezoneMoment.add(1, 'M');
        }

        var offset = timezoneMoment.format('Z');
        offset = (offset === '+00:00') ? '(GMT) ' : '(GMT' + offset + ') ';

        return offset + $scope.getTextWithIndex(textKey, index);
    };

    $scope.time = function (value) {
        if (angular.isDefined(value)) {
            var newMoment = moment(value);
            var timeDiff = Math.abs(newMoment.diff(timeMoment, 'hours'));

            if (timeDiff <= 12) {
                updateModel('time', newMoment);
            }
        } else {
            return timeMoment.valueOf();
        }
    };

    $scope.date = function (value) {
        if (angular.isDefined(value)) {
            updateModel('date', moment(value));
        } else {
            return dateMoment.toDate();
        }
    };

    $scope.toggleDatepicker = function ($event) {
        $event.preventDefault();
        $event.stopPropagation();
        $scope.datepickerOpened = !$scope.datepickerOpened;
    };

    $scope.minDate = new Date('2015-01-01T00:00:00Z');
    $scope.maxDate = new Date('2037-12-31T23:59:59Z');

    $scope.handleTimeZoneChange = $scope.handleAutoAdjustClockForDSTChange = function () {
        updateMoments();
    };

    $scope.handleAutoAdjustTimeChange = function () {
        if ($scope.parameters.AutoAdjustTime.value === 'NTPServiceEnabled') {
            $scope.parameters.Timestamp.restoreFromCache();
        }

        if ($scope.parameters.AutoAdjustTime.value === 'NTPServiceDisabled') {
            $scope.parameters.TimeServer.restoreFromCache();
            $scope.parameters.Timestamp.value = Math.round(+new Date() / 1000);
            updateMoments();
        }
    };

    $scope.$on('Form::Cancel', function (event, data) {
        updateMoments();
    });

    $scope.isProvisioningOrDHCP = function () {
        return ($scope.parameters.AutoAdjustTime.value == 'NTPServiceEnabled' || $scope.parameters.AutoAdjustTime.value == 'NTPServiceDHCP');
    }

    $scope.getNtpServerValue = function () {
        let _ntpServerValue = null;

        if ($scope.parameters.AutoAdjustTime.value == 'NTPServiceEnabled') {
            if ($scope.parameters.TimeServerProvisioning.value != '') {
                _ntpServerValue = $scope.parameters.TimeServerProvisioning.value;
            }
            else if ($scope.parameters.TimeServerProvisioning.value == '' && $scope.parameters.TimeServerDHCP.value != '') {
                _ntpServerValue = $scope.parameters.TimeServerDHCP.value;
                $scope.tooltipText = $scope.text.DHCPValue;
            }
            else {
                _ntpServerValue = $scope.parameters.TimeServer.value;
                $scope.tooltipText = $scope.text.ManualValue;
            }
        }
        else if ($scope.parameters.AutoAdjustTime.value == 'NTPServiceDHCP') {
            if ($scope.parameters.TimeServerDHCP.value != '') {
                _ntpServerValue = $scope.parameters.TimeServerDHCP.value;
            }
            else if ($scope.parameters.TimeServerDHCP.value == '' && $scope.parameters.TimeServerProvisioning.value != '') {
                _ntpServerValue = $scope.parameters.TimeServerProvisioning.value;
                $scope.tooltipText = $scope.text.ProvisioningValue;
            }
            else {
                _ntpServerValue = $scope.parameters.TimeServer.value;
                $scope.tooltipText = $scope.text.ManualValue;
            }
        }
        return _ntpServerValue;
    }

    State.getNtpLastSyncState();
}]);

angular.module('WebUI').controller('CorporateCtrl', ['$scope', function ($scope) {
    $scope.options = {
        LDAPFirstName: ['givenName', 'none'],
        LDAPSurname: ['sn', 'cn', 'displayName', 'none'],
        LDAPPhoneHome: ['homePhone', 'telephoneNumber', 'none'],
        LDAPPhoneOffice: ['telephoneNumber', 'none'],
        LDAPPhoneMobile: ['mobile', 'none'],
        LDAPEmail: ['mail', 'none'],
        LDAPFax: ['facsimileTelephoneNumber', 'none'],
        LDAPCompany: ['company', 'o', 'ou', 'none'],
        LDAPStreet: ['street', 'none'],
        LDAPCity: ['l', 'postalAddress', 'none'],
        LDAPZIP: ['postalCode', 'none'],
        LDAPCountry: ['friendlyCountryName', 'c', 'none']
    };
}]);

angular.module('WebUI').controller('XmlCtrl', ['$scope', 'Tools', function ($scope, Tools) {
    $scope.options = {
        XmlNumberFilter: ['bp', 'in', 'mb', 'hm']
    };

    $scope.changeValue = function (parameterName, value) {
        var parameter = Tools.getPropertyByString($scope.parameters, parameterName);

        if (parameter) {
            parameter.value = value;
        }
    };
}]);

angular.module('WebUI').controller('StorageAllocationCtrl', ['$scope', '$http', function ($scope, $http) {
    $scope.RingtonesSize = 0;
    $scope.ImagesSize = 0;
    $scope.MaxLimitStorage = 5 * 1024 * 1024;

    var updateStatus = function (response) {
        if (response && response.data && response.data.ringtones && response.data.pictures) {
            $scope.RingtonesSize = response.data.ringtones.size;
            $scope.ImagesSize = response.data.pictures.size;
        }
    };

    $http.get('StorageStatus', {cache: false})
        .then(updateStatus);

}]);

angular.module('WebUI').controller('PopupCtrl', ['$scope', '$modalInstance', 'data', function ($scope, $modalInstance, data) {
    $scope.data = data;
    $scope.button = function (value) {
        $modalInstance.close(value);
    };
}]);

angular.module('WebUI').controller('FirmwareUpdateCtrl', ['$scope', '$filter', 'FileUploader', 'Popup', 'State', 'UserService', function ($scope, $filter, FileUploader, Popup, State, UserService) {
    var popupData = {};

    var verifyFirmware = function () {
        State.get({name: 'FirmwareUpdate.Verify', interval: 2})
            .then(null, null, function (response) {
                if (response && response.data && response.data['FirmwareUpdate.Verify']) {
                    switch (response.data['FirmwareUpdate.Verify']) {
                        case 'request':
                            break;

                        case 'success':
                        case 'current':
                            popupData.title = $scope.text.FirmwareUpdate;
                            popupData.message = response.data['FirmwareUpdate.Verify'] === 'success'
                                ? $scope.text.ConfirmUpdate
                                : $scope.text.FirmwareIsCurrent;
                            popupData.buttons = response.data['FirmwareUpdate.Verify'] === 'success'
                                ? [{text: $scope.text.OK, value: $scope.config.button.ok}]
                                : [{text: $scope.text.Yes, value: $scope.config.button.yes},
                                    {text: $scope.text.No, value: $scope.config.button.no}];

                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true})
                                .then(function (popupButton) {
                                    switch (popupButton) {
                                        case $scope.config.button.ok:
                                        case $scope.config.button.yes:
                                            State.set({'FirmwareUpdate.UpdateSystem': 'request'})
                                                .then(UserService.logout);
                                            break;
                                        case  $scope.config.button.no:
                                            State.set({'FirmwareUpdate.UpdateSystem': 'deleteFirmware'})
                                            break;
                                    }
                                });
                            break;

                        case 'wrongtype':
                            popupData.title = $scope.text.Error;
                            popupData.message = $scope.text.FirmwareIsWrong;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            break;

                        case 'fail':
                        default:
                            popupData.title = $scope.text.Error;
                            popupData.message = $scope.text.FileIsCorupt;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            break;
                    }

                    if (response.data['FirmwareUpdate.Verify'] !== 'request') {
                        State.stop('FirmwareUpdate.Verify');
                    }
                }
            });
    };

    $scope.uploader = new FileUploader();
    $scope.uploadFirmware = function (item) {
        State.set({'FirmwareUpdate.UpdateSystem': 'deleteFirmware'})
            .then(function (item) {
                $scope.deleteFirmware = true;
                State.get({name: 'FirmwareUpdate.UpdateSystem', interval: 1})
                    .then(null, null, function (response, item) {
                        if (response && response.data &&
                            response.data['FirmwareUpdate.UpdateSystem'] ===
                            "deleteDone") {
                            State.stop('FirmwareUpdate.UpdateSystem');
                            $scope.deleteFirmware = false;
                            $scope.uploader.uploadWithKeepAlive(item);
                        }
                    });
            });
    };
    $scope.uploader.onCompleteItem = function (item, response, status, headers) {
        if (response && response.message === '#firmwareupdate#') {
            if (status === 200) {
                verifyFirmware();

                popupData.title = $scope.text.FirmwareUpdate;
                popupData.message = $scope.text.PleaseWaitFirmwareUpdateUploadFile;
                popupData.buttons = [];
            } else {
                popupData.title = $scope.text.Error;
                popupData.message = $scope.text.FileIsCorupt;
                popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
            }

            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
        }
    };
    $scope.resetFirmwareUpdateStates = function () {
        State.set({'FirmwareUpdate.UpdateSystem': 'reset'});
        State.set({'FirmwareUpdate.Verify': 'reset'});
    };
    $scope.NewFirmwareVersion = '';
    $scope.activeSearchForNewSoftware = false;
    var monitorNewFirmware = function () {
        State.get({name: 'FirmwareUpdate.NewVersionNumber', interval: 2})
            .then(null, null, function (response) {
                if (response && response.data && response.data['FirmwareUpdate.NewVersionNumber']) {
                    State.stop('FirmwareUpdate.NewVersionNumber');
                    $scope.NewFirmwareVersion = response.data['FirmwareUpdate.NewVersionNumber'];
                    popupData.title = $scope.text.FirmwareUpdate;
                    popupData.message = $filter('Replacement')($scope.text.ConfirmNewFirmwareUpdate, [$scope.NewFirmwareVersion]);
                    popupData.buttons = [{
                        text: $scope.text.Yes,
                        value: $scope.config.button.yes
                    }, {text: $scope.text.No, value: $scope.config.button.no}];

                    Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true})
                        .then(function (popupButton) {
                            switch (popupButton) {
                                case $scope.config.button.yes:
                                    State.set({'FirmwareUpdate.UpdateSystem': 'request'})
                                        .then(UserService.logout);
                                    break;
                                case  $scope.config.button.no:
                                    State.set({'FirmwareUpdate.UpdateSystem': 'deleteFirmware'})
                                    break;
                            }
                        });
                }
            });
    };

    var monitorFirmwareVerification = function () {
        State.get({name: 'FirmwareUpdate.Verify', interval: 2})
            .then(null, null, function (response) {
                if (response && response.data && response.data['FirmwareUpdate.Verify']) {
                    switch (response.data['FirmwareUpdate.Verify']) {
                        case 'request':
                            break;

                        case 'success':
                        case 'current':
                            monitorNewFirmware();
                            break;

                        case 'fail':
                            popupData.title = $scope.text.Error;
                            popupData.message = $scope.text.FileIsCorupt;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            break;
                        default:
                            popupData.title = $scope.text.Error;
                            popupData.message = $scope.text.ServerError;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            break;
                    }

                    if (response.data['FirmwareUpdate.Verify'] !== 'request') {
                        State.stop('FirmwareUpdate.Verify');
                        $scope.activeSearchForNewSoftware = false;
                    }
                }
            });

    };

    var monitorAutomaticUpdate = function () {
        State.get({name: 'FirmwareUpdate.TriggerAuto', interval: 2})
            .then(null, null, function (response) {
                if (response && response.data && response.data['FirmwareUpdate.TriggerAuto']) {
                    switch (response.data['FirmwareUpdate.TriggerAuto']) {
                        case 'request':
                            break;

                        case 'success':
                            Popup.close(popupData.layer, popupData.id);
                            $scope.showPopup($scope.text.FileUploaded);
                            popupData.title = $scope.text.FirmwareVerification;
                            popupData.message = $scope.text.PleaseWait;
                            popupData.buttons = [];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});

                            monitorFirmwareVerification();
                            break;

                        case 'no_version':
                            Popup.close(popupData.layer, popupData.id);
                            popupData.title = $scope.text.SearchForNewSoftware;
                            popupData.message = $scope.text.NotNewFirmwareAvailable;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});

                            $scope.activeSearchForNewSoftware = false;
                            break;
                        default:
                            popupData.title = $scope.text.Error;
                            popupData.message = $scope.text.ServerError;
                            popupData.buttons = [{text: $scope.text.OK, value: $scope.config.button.ok}];
                            Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                            $scope.activeSearchForNewSoftware = false;
                            break;
                    }

                    if (response.data['FirmwareUpdate.TriggerAuto'] !== 'request') {
                        State.stop('FirmwareUpdate.TriggerAuto');
                    }
                }
            });
    };

    $scope.startSearchForNewSoftware = function (item) {
        $scope.activeSearchForNewSoftware = true;

        popupData.title = $scope.text.SearchForNewSoftware;
        popupData.message = $scope.text.PleaseWait;
        popupData.buttons = [];
        Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});

        State.set({'FirmwareUpdate.TriggerAuto': 'request'})
            .then(function () {
                monitorAutomaticUpdate();
            });
    };
}]);

angular.module('WebUI').controller('CallHistoryCtrl', ['$scope', '$http', 'Popup', 'State', function ($scope, $http, Popup, State) {
    var popupData = {};
    $scope.FILTER_ALL = 3;
    $scope.FILTER_ACCEPTED = 0;
    $scope.FILTER_MISSED = 1;
    $scope.FILTER_OUTGOING = 2;

    $scope.activeFilter = $scope.FILTER_ALL;
    $scope.calls = {allSelected: $scope.config.checkbox.unchecked, selectedItemsCnt: 0};

    $scope.toggleFilter = function (value) {
        $scope.activeFilter = value;
        $scope.calls.allSelected = $scope.config.checkbox.unchecked;
    };

    $scope.isFiltered = function () {
        return function (call) {
            return (call && (call.Type === $scope.activeFilter || $scope.activeFilter === $scope.FILTER_ALL));
        };
    };

    $scope.toggleSelectAll = function (calls) {
        angular.forEach($scope.callList, function (call, index) {
            if (call.isSelected !== $scope.calls.allSelected && (call.Type === $scope.activeFilter || $scope.activeFilter === $scope.FILTER_ALL)) {
                if ($scope.calls.allSelected === $scope.config.checkbox.checked) {
                    $scope.calls.selectedItemsCnt++;
                } else {
                    $scope.calls.selectedItemsCnt--;
                }
                call.isSelected = $scope.calls.allSelected;
            }
        });
    };

    $scope.toggleSelect = function (call) {
        if (call.isSelected === $scope.config.checkbox.checked) {
            $scope.calls.selectedItemsCnt++;
        } else {
            $scope.calls.allSelected = $scope.config.checkbox.unchecked
            $scope.calls.selectedItemsCnt--;
        }
    };

    $scope.canDelete = function () {
        return $scope.calls.selectedItemsCnt > 0;
    };

    $scope.deleteEntries = function () {
        popupData.title = "";
        popupData.message = $scope.text.DeleteSelectedEntriesPopup;
        popupData.buttons = [
            {text: $scope.text.Yes, value: $scope.config.button.yes},
            {text: $scope.text.No, value: $scope.config.button.no}
        ];

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            //delete all
                            if ($scope.calls.allSelected === $scope.config.checkbox.checked) {
                                $http.post('CallHistory', {deleteAll: true, type: $scope.activeFilter})
                                    .then(function (response) {
                                        $scope.refreshAll()
                                    });
                            } else { //delete selected only
                                var ids = [];
                                var idxs = [];
                                angular.forEach($scope.callList, function (call, index) {
                                    if (call.isSelected === $scope.config.checkbox.checked) {
                                        ids.push(call.id);
                                        idxs.push(index);
                                    }
                                });
                                if (ids.length > 0) {
                                    $http.post('CallHistory', {deleteAll: false, ids: ids})
                                        .then(function (response) {
                                            //remove from list
                                            while (idxs.length) {
                                                $scope.callList.splice(idxs.pop(), 1);
                                            }
                                        });
                                }
                            }
                            $scope.calls.allSelected = $scope.config.checkbox.unchecked;
                            $scope.calls.selectedItemsCnt = 0;
                            break;
                    }
                }
            });
    };

    var toTimeZone = function (time, zone) {
        var format = 'YYYY-MM-DD HH:mm:ss';
        var timemoment = moment.utc(time, format);

        return timemoment.tz(zone).format(format);
    }

    $scope.refreshAll = function () {
        var popupData = {
            title: $scope.text.CallHistory,
            message: $scope.text.PleaseWait
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.get('CallHistory', {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            restartLogoutTimer: true
        })
            .then(function (response) {
                Popup.close(popupData.layer, popupData.id);
                $scope.callList = [];

                angular.forEach(response.data.entries, function (call, index) {
                    var startTime = (call.startTime.replace("T", " "))
                    var endTime = (call.endTime.replace("T", " "));
                    var duration = moment(endTime, "YYYY-MM-DD HH:mm:ss").diff(moment(startTime, "YYYY-MM-DD HH:mm:ss")) / 1000;
                    var timeZoneName = $scope.parameters.TimeZone.value.split('.')[1];

                    $scope.callList.push({
                        accountId: call.accountId,
                        anonymous: call.anonymous,
                        CallerID: call.cnip,
                        direction: call.direction,//in, out
                        EndTime: toTimeZone(endTime, timeZoneName),
                        firstName: call.firstName,
                        group: call.group,
                        id: call.id,
                        line: call.line,
                        mailbox: call.mailbox,
                        PhoneNumber: call.number,
                        numberType: call.numberType,//"mobile", "home", "unknown"
                        sipID: call.sipID,
                        StartTime: toTimeZone(startTime, timeZoneName),
                        status: call.status,
                        surname: call.surname,
                        Type: call.group,
                        isSelected: $scope.config.checkbox.unchecked,
                        Duration: duration
                    });
                    $scope.calls.allSelected = $scope.config.checkbox.unchecked;
                    $scope.calls.selectedItemsCnt = 0;
                });
            });
    };

    $scope.refreshAll();

    //refresh call history after clicking on menu
    $scope.$on('Form::Cancel', function (event, data) {
        if (data.url.indexOf('CallHistory') !== -1) {
            $scope.refreshAll();
        }
    });
}]);

angular.module('WebUI').controller('PhoneWebServerCtrl', ['$scope', '$window', '$timeout', 'Popup', function ($scope, $window, $timeout, Popup) {
    $scope.$on('Form::Set::Success', function (event, data) {
        var protocolDef = {'HTTP': 'http:', 'HTTPS': 'https:', 'HTTP_HTTPS': $window.location.protocol},
            portDef = {'http:': $scope.parameters.HTTPPort.value, 'https:': $scope.parameters.HTTPSPort.value},
            defaultPortDef = {'http:': 80, 'https:': 443},
            protocol = protocolDef[$scope.parameters.HTTPConnectionType.value],
            port = portDef[protocol],
            defaultPort = defaultPortDef[protocol];

        var popupData = {
            title: $scope.text.PhoneWebServer,
            message: $scope.text.PleaseWait
        };

        if (angular.isDefined(data.HTTPConnectionType) || angular.isDefined(data.HTTPPort) || angular.isDefined(data.HTTPSPort)) {
            Popup.show(popupData, {priority: $scope.config.popup.priority.info, chain: true});

            $timeout(function () {
                Popup.close(popupData.layer, popupData.id);

                if ($window.location.protocol !== protocol || ($window.location.port ? $window.location.port !== port : port !== defaultPort)) {
                    $window.location.href = protocol + '//' +
                        $window.location.hostname + ':' + port +
                        $window.location.pathname +
                        $window.location.hash +
                        '?token=' + $window.sessionStorage.token;
                }
            }, 5000);
        }
    });

    $scope.handleHTTPConnectionTypeChange = function () {
        if ($scope.parameters.HTTPConnectionType.value === 'HTTP') {
            $scope.parameters.HTTPSPort.restoreFromCache();
        }

        if ($scope.parameters.HTTPConnectionType.value === 'HTTPS') {
            $scope.parameters.HTTPPort.restoreFromCache();
        }
    };
}]);

angular.module('WebUI').controller('DoNotDisturbCtrl', ['$scope', '$q', 'Popup', 'Tools', function ($scope, $q, Popup, Tools) {
    var popupData = {
        title: $scope.text.DoNotDisturb
    };

    var allEntriesAreUnique = function (newEntry) {
        var compareObject = {
            PhoneNumber: angular.isDefined(newEntry) ? newEntry.PhoneNumber : null
        };

        return Tools.List.hasUniqueEntries($scope.parameters.DnD, $scope.DnD.Default, compareObject, angular.isDefined(newEntry));
    };

    var updateConfiguredDnDList = function () {
        $scope.configuredDnD = Tools.List.getConfiguredEntries($scope.parameters.DnD, $scope.DnD.Default);
        $scope.numberOfEditableDnD = Tools.List.getEditableEntries($scope.parameters.DnD, $scope.DnD.Default).length;
    };

    $scope.DnD = {
        NewEntry: {},
        Default: {
            Name: '',
            PhoneNumber: ''
        }
    };

    Tools.List.copyEntry($scope.DnD.NewEntry, $scope.DnD.Default);

    $scope.isEntryLocked = Tools.List.isEntryLocked;

    $scope.addEntry = function () {
        if (allEntriesAreUnique($scope.DnD.NewEntry)) {
            var freeEntryIndex = Tools.List.getFreeEntryIndex($scope.parameters.DnD, $scope.DnD.Default);

            if (freeEntryIndex !== null) {
                Tools.List.copyEntry($scope.parameters.DnD[freeEntryIndex], $scope.DnD.NewEntry);
                Tools.List.copyEntry($scope.DnD.NewEntry, $scope.DnD.Default);
                $scope.processForm({
                    customParameters: {
                        DnD: Tools.List.prepareSingleEntry($scope.parameters.DnD, freeEntryIndex)
                    },
                    ignoreInvalidForm: true
                });

                updateConfiguredDnDList();
            }
        } else {
            popupData.message = $scope.text.PhoneNumberIsContained;
            popupData.buttons = [
                {text: $scope.text.OK, value: $scope.config.button.ok}
            ];
            Popup.show(popupData, {priority: $scope.config.popup.priority.user});
        }
    };

    $scope.deleteEntry = function (entry) {
        Tools.List.deleteEntry(entry, $scope.DnD.Default);
        $scope.processForm({
            customParameters: {
                DnD: Tools.List.prepareSingleEntry($scope.parameters.DnD, $scope.parameters.DnD.indexOf(entry))
            },
            ignoreInvalidForm: true
        });

        updateConfiguredDnDList();
    };

    $scope.deleteAllEntries = function () {
        popupData.message = $scope.text.DeleteAllEntriesPopup;
        popupData.buttons = [
            {text: $scope.text.Yes, value: $scope.config.button.yes},
            {text: $scope.text.No, value: $scope.config.button.no}
        ];

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                if (angular.isDefined(popupButton)) {
                    switch (popupButton) {
                        case $scope.config.button.yes:
                            Tools.List.deleteAllEntries($scope.parameters.DnD, $scope.DnD.Default);
                            $scope.processForm({
                                customParameters: {
                                    DnD: $scope.parameters.DnD
                                },
                                ignoreInvalidForm: true
                            });

                            updateConfiguredDnDList();
                            break;
                    }
                }
            });
    };

    $scope.customValidation = function () {
        if (allEntriesAreUnique()) {
            return $q.when();
        } else {
            popupData.message = $scope.text.PhoneNumberIsContained;
            popupData.buttons = [
                {text: $scope.text.OK, value: $scope.config.button.ok}
            ];
            Popup.show(popupData, {priority: $scope.config.popup.priority.user});

            return $q.reject();
        }
    };

    $scope.handleDnDListActiveChange = function () {
        if ($scope.parameters.DnDListActive.value === $scope.config.switch.no) {
            Tools.List.copyEntry($scope.DnD.NewEntry, $scope.DnD.Default);
            angular.forEach($scope.parameters.DnD, function (entry) {
                entry.Name.restoreFromCache();
                entry.PhoneNumber.restoreFromCache();
            });
        }
    };

    $scope.$on('Form::Cancel', function (event, data) {
        Tools.List.copyEntry($scope.DnD.NewEntry, $scope.DnD.Default);
    });

    updateConfiguredDnDList();
}]);

angular.module('WebUI').controller('KeysAndLEDsCtrl', ['$scope', 'Model', 'State', function ($scope, Model, State) {
    var ALL_PHONEBOOKS = '-1';

    $scope.FUNCTION_KEY_SIDE_LEFT = 'Left';
    $scope.FUNCTION_KEY_SIDE_RIGHT = 'Right';

    $scope.FUNCTION_KEY_KEYS_PER_SIDE = 4;
    $scope.FUNCTION_KEY_SIDE_IDLE = 'IdleScreen';
    $scope.FUNCTION_KEY_SIDE_SCREEN_1 = 'Screen1';
    $scope.FUNCTION_KEY_SIDE_SCREEN_2 = 'Screen2';
    $scope.FUNCTION_KEY_SECTION_COUNT = ($scope.isMaxwell2() || $scope.isMaxwell3())
        ? [$scope.FUNCTION_KEY_SIDE_LEFT, $scope.FUNCTION_KEY_SIDE_RIGHT,] :
        [$scope.FUNCTION_KEY_SIDE_IDLE,
            $scope.FUNCTION_KEY_SIDE_SCREEN_1,
            $scope.FUNCTION_KEY_SIDE_SCREEN_2];


    $scope.FUNCTION_KEY_KEYS_PER_SIDE_IDLE = 6;
    $scope.FUNCTION_KEY_KEYS_PER_SIDE_SCREEN = 22;

    $scope.CALL_DIVERT_ALL_CALLS = '1';
    $scope.CALL_DIVERT_NO_ANSWER = '2';
    $scope.CALL_DIVERT_WHEN_BUSY = '3';

    $scope.XSI_PERSONAL = '7';
    $scope.XSI_GROUP = '8';
    $scope.XSI_GROUP_COMMON = '9';
    $scope.XSI_ENTERPRISE = '10';
    $scope.XSI_ENTERPRISE_COMMON = '11';

    $scope.LED_COLOR_RED = '0';
    $scope.LED_COLOR_YELLOW = '1';
    $scope.LED_COLOR_GREEN = '2';
    $scope.LED_COLOR_STANDARD = '3';

    $scope.callDivertTypes = [$scope.CALL_DIVERT_ALL_CALLS, $scope.CALL_DIVERT_NO_ANSWER, $scope.CALL_DIVERT_WHEN_BUSY];

    $scope.editedKeyId = null;

    $scope.getActiveConnections = function () {
        var activeConnections = [];

        angular.forEach($scope.parameters.SIP, function (connection, index) {
            if (connection.Active.value === $scope.config.checkbox.checked) {
                activeConnections.push(index);
            }
        });

        return activeConnections;
    };

    $scope.isFunctionKeyIn = function (editedKeyType, options) {
        return (editedKeyType !== null ? (options.indexOf(editedKeyType) !== -1) : false);
    };

    $scope.checkDisabledElement = function (element) {
        return ($scope.parameters.WebUICallDivertDisable.value === '1' && element === $scope.config.functionKeyType.CallDivert);
    }

    $scope.getColorClass = function (key) {
        return ((!$scope.isMaxwell2() && !$scope.isVodafonePhoneSystem()) ? {
            KeymenuGrey: !$scope.isLEDColorSelectionAvailable(key.Type.value) ||
                key.Color.value == $scope.LED_COLOR_STANDARD,
            KeymenuRed: key.Color.value == $scope.LED_COLOR_RED,
            KeymenuYellow: key.Color.value == $scope.LED_COLOR_YELLOW,
            KeymenuGreen: key.Color.value == $scope.LED_COLOR_GREEN
        } : {});
    }

    $scope.getCurrentQuickDialKeyTextBoxName = function (keyId, switchValue) {
        var name = "QuickDial" + keyId;
        if (switchValue == '1') {
            name += "NumberValue";
        }
        else if (switchValue == '2') {
            name += "FacValue";
        }
        else if (switchValue == '3') {
            name += "ActionUrlValue";
        }

        return name;
    }

    $scope.getCurrentProgrammableKeyTextBoxName = function (keyName, selectedOption) {
        var name = '';
        if (keyName == 'messages') {
            if (selectedOption == $scope.parameters.ProgrammableKeysMessagesType.options[1]) {
                name = 'MessagesCodeURLEnable';
            }
            else {
                name = 'MessagesActionURLEnable';
            }
        }
        else if (keyName == 'dndEnable') {
            if (selectedOption == $scope.parameters.ProgrammableKeysDNDType.options[1]) {
                name = 'DNDCodeURLEnable';
            }
            else if (selectedOption == $scope.parameters.ProgrammableKeysDNDType.options[2]) {
                name = 'DNDActionCodeEnable';
            }
        }
        else if (keyName == 'dndDisable') {
            if (selectedOption == $scope.parameters.ProgrammableKeysDNDType.options[1]) {
                name = 'DNDCodeURLDisable';
            }
            else if (selectedOption == $scope.parameters.ProgrammableKeysDNDType.options[2]) {
                name = 'DNDActionCodeDisable';
            }
        }
        else if (keyName == 'hold') {
            if (selectedOption == $scope.parameters.ProgrammableKeysHoldType.options[1]) {
                name = 'HoldCodeURL';
            }
            else {
                name = 'HoldActionURL';
            }
        }
        else if (keyName == 'conference') {
            if (selectedOption == $scope.parameters.ProgrammableKeysConferenceType.options[1]) {
                name = 'ConferenceCodeURL';
            }
            else if (selectedOption == $scope.parameters.ProgrammableKeysConferenceType.options[2]) {
                name = 'ConferenceActionURL';
            }
            else if (selectedOption == $scope.parameters.ProgrammableKeysConferenceType.options[3]) {
                name = 'ConferenceFAC';
            }
        }
        return name;
    }
    $scope.isMaxwell2 = function () {
        return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 2']);
    }
    $scope.updateTooltipTitle = function (key, sideId, keyId) {
        displayName = key.DisplayName.value;
        tdId = "#tdKey" + sideId + keyId;
        connetionName = $scope.parameters.SIP[key.Connection.value].AccountName.value;
        if (key.DisplayName.value.length == 0) {
            textToDisplay = "";
        } else {
            textToDisplay = $scope.text.Key + " " + (keyId + 1) + "\n" + $scope.text.Name + ": " + displayName;
            if ($scope.parameters.SIP[key.Connection.value].Active.value != 0) {
                textToDisplay += "\n" + $scope.text.Connection + ": " + connetionName;
            }
        }
        $(tdId).prop('title', textToDisplay);
    }
    $scope.handleFunctionKeysTypeChange = function (sideId, editedKeyId) {
        editedKeyId = Number(editedKeyId);
        if (sideId === $scope.FUNCTION_KEY_SIDE_RIGHT) {
            editedKeyId += $scope.FUNCTION_KEY_KEYS_PER_SIDE;
        }
        else if (sideId === $scope.FUNCTION_KEY_SIDE_SCREEN_1) {
            editedKeyId += $scope.FUNCTION_KEY_KEYS_PER_SIDE_IDLE
        }
        else if (sideId === $scope.FUNCTION_KEY_SIDE_SCREEN_2) {
            editedKeyId += $scope.FUNCTION_KEY_KEYS_PER_SIDE_SCREEN
        }

        if (!isNaN(editedKeyId) && $scope.parameters.FunctionKeys[editedKeyId]) {
            var parametersFromCache = Model.getParametersFromCache();
            parametersNames = ['CallDivertType','Connection','CallPickupCode','DTMFCode','DisableCode','DisplayName','EnableCode','PhoneNumber','URL'];

            if ($scope.parameters.FunctionKeys[editedKeyId].Type.value === parametersFromCache.FunctionKeys[editedKeyId].Type.value) {

                angular.forEach(parametersNames,function(paramName) {
                    $scope.parameters.FunctionKeys[editedKeyId][paramName].restoreFromCache();
                });

                $scope.handleCallDivertTypeChange(editedKeyId);
            }
            else {
                angular.forEach(parametersNames.slice(2, parametersNames.length),function(paramName) {
                    $scope.parameters.FunctionKeys[editedKeyId][paramName].value = '';
                });
                $scope.parameters.FunctionKeys[editedKeyId].Silent.value = 0;
                $scope.parameters.FunctionKeys[editedKeyId].CallDivertType.restoreFromCache();
                $scope.parameters.FunctionKeys[editedKeyId].Connection.restoreFromCache();

                $scope.handleConnectionChange();
            }
        }
    };

    $scope.handleSoftKeyTypeChanged = function(index){
        var parametersFromCache = Model.getParametersFromCache();
        parameterNames = ['PhoneNumber','URL', 'EnableCode', 'DisableCode', 'EnableName','DisableName']
        parameterNamesRestoreFromCache = ['Connection', 'Action','Silent'];

        tmpArray = parameterNames.concat(parameterNamesRestoreFromCache);
        angular.forEach(tmpArray, function (paramName){
            $scope.parameters.SoftKeys[index][paramName].value = parametersFromCache.SoftKeys[index][paramName].value;
        });

        if($scope.parameters.SoftKeys[index].Type.value != parametersFromCache.SoftKeys[index].Type.value){
            angular.forEach(parameterNames, function (paramName){
                $scope.parameters.SoftKeys[index][paramName].value = '';
            })
        }
    };

    $scope.isVisibleCallDivertTargetNumberForCurrentPhoneSystem = function () {
        switch ($scope.parameters.PhoneSystem.value) {
            case $scope.config.phoneSystem['BROADSOFT']:
                return true;
                break;
            case $scope.config.phoneSystem['CENTILE']:
                return true;
                break;
            case $scope.config.phoneSystem['VODIA']:
                return true;
                break;
            case $scope.config.phoneSystem['TELES']:
                return true;
                break;
            case $scope.config.phoneSystem['Gigaset T640/T440']:
                return true;
                break;
            case $scope.config.phoneSystem['STARFACE']:
                return true;
                break;
            default:
                return false;
        }
    }

    $scope.isBLFAutomaticllyCreated = function (sideId, editedKeyId) {

        if (sideId === $scope.FUNCTION_KEY_SIDE_RIGHT) {
            editedKeyId += $scope.FUNCTION_KEY_KEYS_PER_SIDE;
        }

        return ($scope.isBroadsoftPhoneSystem() && $scope.parameters.FunctionKeys[editedKeyId].AutomaticallyFilled.value == '1');
    };

    $scope.isKeyTypeDisabled = function (functionKeyType) {
        if ((functionKeyType == $scope.config.functionKeyType.SharedLine && !($scope.isBroadsoftPhoneSystem() || $scope.isCentilePhoneSystem() || $scope.isVodiaPhoneSystem())) ||
            (functionKeyType == $scope.config.functionKeyType.CallDivert && ($scope.isVodafonePhoneSystem()))) {
            return true
        }
        return false
    }

    $scope.getDirectoryKeyOptions = function () {
        if ($scope.parameters.PhoneSystem.value !== $scope.config.phoneSystem['VODAFONE']) {
            return $scope.parameters.ProgrammableKeysDirectoryType.options
        } else {
            return [ALL_PHONEBOOKS]
        }
    };

    $scope.directoryProgrammableKeyOptions = $scope.getDirectoryKeyOptions().filter(function (option) {
        if (!($scope.isBroadsoftPhoneSystem() || $scope.isCentilePhoneSystem() || $scope.isVodiaPhoneSystem())) {
            return option !== $scope.XSI_PERSONAL && option !== $scope.XSI_GROUP &&
                option !== $scope.XSI_GROUP_COMMON && option !== $scope.XSI_ENTERPRISE &&
                option !== $scope.XSI_ENTERPRISE_COMMON;
        }
        return option;
    });

    $scope.expanded = [{
        Left: [false, false, false, false],
        Right: [false, false, false, false],
        IdleScreen: [false, false, false, false, false, false],
        Screen1: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false],
        Screen2: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]

    }]
    $scope.keyHasInputError = [{
        Left: [false, false, false, false],
        Right: [false, false, false, false],
        IdleScreen: [false, false, false, false, false, false],
        Screen1: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false],
        Screen2: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]

    }]

    $scope.toggle = function (sideId, keyId) {
        if ($scope.isCollapsed(sideId, keyId) == false) {
            if ($scope.areInputsErrorsExist(sideId, keyId) == true) {
                $scope.keyHasInputError[$scope.activePage][sideId][keyId] = true;
            } else {
                $scope.keyHasInputError[$scope.activePage][sideId][keyId] = false;
            }
        }
        if ($scope.keyHasInputError[$scope.activePage][sideId][keyId] == false) {
            $scope.expanded[$scope.activePage][sideId][keyId] = !$scope.expanded[$scope.activePage][sideId][keyId];
        }
    };

    $scope.isTextBoxError = function (error) {
        for (var key in error) {
            if (error[key] == true) {
                return true;
            } else {
                return false;
            }
        }
    };

    $scope.isLEDColorSelectionAvailable = function (keyType) {
        return keyType == $scope.config.functionKeyType.Line || keyType == $scope.config.functionKeyType.Blf ||
            keyType == $scope.config.functionKeyType.SipInfo || keyType == $scope.config.functionKeyType.Fac ||
            keyType == $scope.config.functionKeyType.PbxFeature;
    }

    $scope.isStandardOptionVisible = function (keyType) {
        return keyType == $scope.config.functionKeyType.Line || keyType == $scope.config.functionKeyType.Blf || keyType == $scope.config.functionKeyType.PbxFeature;
    }

    $scope.areTextBoxesValidated = function (sideId, keyId, selectedFunction) {
        result = true;
        if (selectedFunction == $scope.config.functionKeyType.QuickDial || selectedFunction == $scope.config.functionKeyType.Blf) {
            if ($scope.isTextBoxError($scope.form["PhoneNumber" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction != $scope.config.functionKeyType.NoFunction && selectedFunction != $scope.config.functionKeyType.Line) {
            if ($scope.isTextBoxError($scope.form["DisplayName" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Blf) {
            if ($scope.isTextBoxError($scope.form["CallPickupCode" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.SipUri || selectedFunction == $scope.config.functionKeyType.ActionUrl) {
            if ($scope.isTextBoxError($scope.form["URL" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Dtmf) {
            if ($scope.isTextBoxError($scope.form["DTMF" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Fac) {
            if ($scope.isTextBoxError($scope.form["EnableCode" + sideId + keyId].$error) == true || $scope.isTextBoxError($scope.form["DisableCode" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        return result;
    };

    $scope.areInputsErrorsExist = function (sideId, keyId) {
        dropDownListBox = "selectType" + sideId + keyId;
        selectedFunction = document.getElementById(dropDownListBox).value;
        result = !$scope.areTextBoxesValidated(sideId, keyId, selectedFunction);
        return result;
    };

    $scope.activePage = 0;
    $scope.togglePage = function (tabIndex) {
        $scope.activePage = tabIndex;
    }
    $scope.isCollapsed = function (sideId, keyId) {
        return !$scope.expanded[$scope.activePage][sideId][keyId];
    };

    $scope.handleQuickDialTypeChange = function (editedKeyId) {
        editedKeyId = Number(editedKeyId);
        var parametersFromCache = Model.getParametersFromCache();

        if (!isNaN(editedKeyId) && $scope.parameters.QuickDialKeys[editedKeyId]) {

            if ($scope.parameters.QuickDialKeys[editedKeyId].Type.value === parametersFromCache.QuickDialKeys[editedKeyId].Type.value) {

                $scope.parameters.QuickDialKeys[editedKeyId].Number.restoreFromCache();
                $scope.parameters.QuickDialKeys[editedKeyId].FAC.restoreFromCache();
                $scope.parameters.QuickDialKeys[editedKeyId].ActionURL.restoreFromCache();

            } else {
                $scope.parameters.QuickDialKeys[editedKeyId].Number.value = "";
                $scope.parameters.QuickDialKeys[editedKeyId].FAC.value = "";
                $scope.parameters.QuickDialKeys[editedKeyId].ActionURL.value = "";

            }
        }
    }

    $scope.handleCallDivertTypeChange = function (editedKeyId) {
        var connectionId = Number($scope.parameters.FunctionKeys[editedKeyId].Connection.value);

        if (!isNaN(connectionId) && $scope.parameters.CallDivertNoAnswer[connectionId]) {
            $scope.parameters.CallDivertNoAnswer[connectionId].Delay.restoreFromCache();
        }
    };

    $scope.handleConnectionChange = function () {
        angular.forEach($scope.parameters.CallDivertNoAnswer, function (callDivertNoAnswer) {
            callDivertNoAnswer.Delay.value = 120;
        });
    };

    $scope.$on('Form::Set::Success', function (event, data) {
        $scope.editedKeyId = null;
    });

    $scope.$on('Form::Cancel', function (event, data) {
        $scope.editedKeyId = null;
    });

    $scope.generateKeyForPage = function (startIndex, numberOfKeys) {
        var parametersArr = []

        for (var i = startIndex; i < numberOfKeys; i++) {
            parametersArr.push($scope.parameters.FunctionKeys[i]);
        }
        return parametersArr;
    }

    $scope.getSideParameters = function (sideId) {
        if (sideId === $scope.FUNCTION_KEY_SIDE_LEFT) {
            return $scope.generateKeyForPage(0, 4)
        } else if ((sideId === $scope.FUNCTION_KEY_SIDE_RIGHT)) {
            return $scope.generateKeyForPage(4, 8)
        } else if (sideId === $scope.FUNCTION_KEY_SIDE_IDLE) {
            return $scope.generateKeyForPage(0, 6)
        } else if (sideId === $scope.FUNCTION_KEY_SIDE_SCREEN_1) {
            return $scope.generateKeyForPage(6, 22)
        } else if (sideId === $scope.FUNCTION_KEY_SIDE_SCREEN_2) {
            return $scope.generateKeyForPage(22, 38)
        }
    };

    State.getExtensionModulesStates();
}]);

angular.module('WebUI').controller('DiallingPlansCtrl', ['$scope', 'Tools', function ($scope, Tools) {
    PAGE_NAME = 'DialingPlansPage';
    $scope.getParametesRelatedWithDeletingAccount(PAGE_NAME, null);

    var updateConfiguredDiallingPlansList = function () {
        $scope.configuredDiallingPlans = Tools.List.getConfiguredEntries($scope.parameters.DiallingPlans, $scope.DiallingPlans.Default.Model);
    };

    var updateConfiguredCallBarringList = function () {
        $scope.configuredCallBarring = Tools.List.getConfiguredEntries($scope.parameters.CallBarring, $scope.CallBarring.Default.Model);
    };

    $scope.getActiveConnections = function () {
        var activeConnections = [];

        angular.forEach($scope.parameters.SIP, function (connection, index) {
            if (connection.Active.value === $scope.config.checkbox.checked) {
                activeConnections.push(String(index));
            }
        });

        return activeConnections;
    };

    $scope.DiallingPlans = {
        NewEntry: {},
        Default: {
            Model: {
                PhoneNumber: '',
                UseAreaCode: $scope.config.checkbox.unchecked,
                Connection: '0',
                Comment: '',
                Active: $scope.config.checkbox.unchecked
            },
            View: {
                PhoneNumber: '',
                UseAreaCode: $scope.config.checkbox.unchecked,
                Connection: $scope.getActiveConnections()[0] || '0',
                Comment: '',
                Active: $scope.config.checkbox.checked
            }
        }
    };

    $scope.CallBarring = {
        NewEntry: {},
        Default: {
            Model: {
                PhoneNumber: '',
                Comment: '',
                Blocked: $scope.config.checkbox.unchecked
            },
            View: {
                PhoneNumber: '',
                Comment: '',
                Blocked: $scope.config.checkbox.checked
            }
        }
    };

    Tools.List.copyEntry($scope.DiallingPlans.NewEntry, $scope.DiallingPlans.Default.View);
    Tools.List.copyEntry($scope.CallBarring.NewEntry, $scope.CallBarring.Default.View);

    $scope.isEntryLocked = Tools.List.isEntryLocked;

    $scope.addEntry = function (listName) {
        var freeEntryIndex = Tools.List.getFreeEntryIndex($scope.parameters[listName], $scope[listName].Default.Model);
        var customParameters = {};

        if (freeEntryIndex !== null) {
            Tools.List.copyEntry($scope.parameters[listName][freeEntryIndex], $scope[listName].NewEntry);
            Tools.List.copyEntry($scope[listName].NewEntry, $scope[listName].Default.View);
            customParameters[listName] = Tools.List.prepareSingleEntry($scope.parameters[listName], freeEntryIndex);

            $scope.processForm({
                customParameters: customParameters,
                ignoreInvalidForm: true
            });

            if (listName === 'DiallingPlans') {
                updateConfiguredDiallingPlansList();
            }

            if (listName === 'CallBarring') {
                updateConfiguredCallBarringList();
            }
        }
    };

    $scope.deleteEntry = function (listName, entry) {
        var customParameters = {};

        Tools.List.deleteEntry(entry, $scope[listName].Default.Model);
        customParameters[listName] = Tools.List.prepareSingleEntry($scope.parameters[listName], $scope.parameters[listName].indexOf(entry));

        $scope.processForm({
            customParameters: customParameters,
            ignoreInvalidForm: true
        });

        if (listName === 'DiallingPlans') {
            updateConfiguredDiallingPlansList();
        }

        if (listName === 'CallBarring') {
            updateConfiguredCallBarringList();
        }
    };

    $scope.$on('Form::Cancel', function () {
        Tools.List.copyEntry($scope.DiallingPlans.NewEntry, $scope.DiallingPlans.Default.View);
        Tools.List.copyEntry($scope.CallBarring.NewEntry, $scope.CallBarring.Default.View);
    });

    updateConfiguredDiallingPlansList();
    updateConfiguredCallBarringList();
}]);

angular.module('WebUI').controller('ExtensionModuleCtrl', ['$scope', 'Tools', 'Model', function ($scope, Tools, Model) {

    $scope.CALL_DIVERT_ALL_CALLS = '1';
    $scope.CALL_DIVERT_NO_ANSWER = '2';
    $scope.CALL_DIVERT_WHEN_BUSY = '3';
    $scope.FUNCTION_KEY_SIDE_LEFT = 'Left';
    $scope.FUNCTION_KEY_SIDE_RIGHT = 'Right';
    $scope.callDivertTypes = [$scope.CALL_DIVERT_ALL_CALLS, $scope.CALL_DIVERT_NO_ANSWER, $scope.CALL_DIVERT_WHEN_BUSY];

    $scope.LED_COLOR_RED = '0';
    $scope.LED_COLOR_YELLOW = '1';
    $scope.LED_COLOR_GREEN = '2';
    $scope.LED_COLOR_STANDARD = '3';

    $scope.isTextBoxError = function (error) {
        for (var key in error) {
            if (error[key] == true) {
                return true;
            } else {
                return false;
            }
        }
    };

    $scope.areTextBoxesValidated = function (sideId, keyId, selectedFunction) {
        result = true;
        if (selectedFunction == $scope.config.functionKeyType.QuickDial || selectedFunction == $scope.config.functionKeyType.Blf) {
            if ($scope.isTextBoxError($scope.form["PhoneNumber" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction != $scope.config.functionKeyType.NoFunction) {
            if ($scope.isTextBoxError($scope.form["DisplayName" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Blf) {
            if ($scope.isTextBoxError($scope.form["CallPickupCode" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.SipUri || selectedFunction == $scope.config.functionKeyType.ActionUrl) {
            if ($scope.isTextBoxError($scope.form["URL" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Dtmf) {
            if ($scope.isTextBoxError($scope.form["DTMF" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        if (selectedFunction == $scope.config.functionKeyType.Fac) {
            if ($scope.isTextBoxError($scope.form["EnableCode" + sideId + keyId].$error) == true || $scope.isTextBoxError($scope.form["DisableCode" + sideId + keyId].$error) == true) {
                result = false;
            }
        }
        return result;
    };

    $scope.areInputsErrorsExist = function (sideId, keyId) {
        dropDownListBox = "selectType" + sideId + keyId;
        selectedFunction = document.getElementById(dropDownListBox).value;
        result = !$scope.areTextBoxesValidated(sideId, keyId, selectedFunction);
        return result;
    };

    $scope.getActiveConnections = function () {
        var activeConnections = [];

        angular.forEach($scope.parameters.SIP, function (connection, index) {
            if (connection.Active.value === $scope.config.checkbox.checked) {
                activeConnections.push(index);
            }
        });

        return activeConnections;
    };
    $scope.isFunctionKeyIn = function (type, options) {
        return (options.indexOf(type) !== -1);
    };

    var getExtModId = function (moduleId, pageId, sideId, keyId) {
        return moduleId * 58 + pageId * 29 + sideId * 14 + keyId + parseInt($scope.parameters.PhoneConfig.ExtMod);
    };

    $scope.updateTooltipTitle = function (key, sideId, keyId) {
        displayName = key.DisplayName.value;
        tdId = "#tdKey" + sideId + keyId;
        connetionName = $scope.parameters.SIP[key.Connection.value].AccountName.value;
        if (key.DisplayName.value.length == 0) {
            textToDisplay = "";
        } else {
            textToDisplay = $scope.text.Key + " " + (keyId + 1) + "\n" + $scope.text.Name + ": " + displayName;
            if ($scope.parameters.SIP[key.Connection.value].Active.value != 0) {
                textToDisplay += "\n" + $scope.text.Connection + ": " + connetionName;
            }
        }
        $(tdId).prop('title', textToDisplay);
    }

    $scope.isBLFAutomaticllyCreatedExt = function (sideId, editedKeyId) {
        editedKeyId = Number(editedKeyId);
        sideId = Number(sideId === 'Left' ? 0 : 1);
        var paramId = getExtModId($scope.activeExtensionModuleId, $scope.activePage, sideId, editedKeyId);
        return ($scope.isBroadsoftPhoneSystem() && $scope.parameters.FunctionKeys[paramId].AutomaticallyFilled.value === '1');
    };

    $scope.isVodafonePhoneSystem = function () {
        return ($scope.parameters.PhoneSystem.value == $scope.config.phoneSystem['VODAFONE']);
    };

    $scope.getColorClass = function (key) {
        return (!$scope.isVodafonePhoneSystem() ? {
            KeymenuGrey: !$scope.isLEDColorSelectionAvailable(key.Type.value) || key.Color.value == $scope.LED_COLOR_STANDARD,
            KeymenuRed: key.Color.value == $scope.LED_COLOR_RED,
            KeymenuYellow: key.Color.value == $scope.LED_COLOR_YELLOW,
            KeymenuGreen: key.Color.value == $scope.LED_COLOR_GREEN
        } : {KeymenuGrey: true});
    }

    $scope.isLEDColorSelectionAvailable = function (keyType) {
        return keyType == $scope.config.functionKeyType.Line || keyType == $scope.config.functionKeyType.Blf ||
            keyType == $scope.config.functionKeyType.SipInfo || keyType == $scope.config.functionKeyType.Fac ||
            keyType == $scope.config.functionKeyType.PbxFeature;
    }

    $scope.isStandardOptionVisible = function (keyType) {
        return keyType == $scope.config.functionKeyType.Line || keyType == $scope.config.functionKeyType.Blf || keyType == $scope.config.functionKeyType.PbxFeature;
    }

    $scope.handleFunctionKeysTypeChange = function (sideId, editedKeyId) {
        editedKeyId = Number(editedKeyId);
        sideId = Number(sideId === 'Left' ? 0 : 1);
        var paramId = getExtModId($scope.activeExtensionModuleId, $scope.activePage, sideId, editedKeyId);
        var parametersFromCache = Model.getParametersFromCache();

        if (!isNaN(paramId) && $scope.parameters.FunctionKeys[paramId]) {
            if ($scope.parameters.FunctionKeys[paramId].Type.value === parametersFromCache.FunctionKeys[paramId].Type.value) {
                $scope.parameters.FunctionKeys[paramId].CallPickupCode.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].Connection.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].DTMFCode.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].DisableCode.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].DisplayName.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].EnableCode.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].PhoneNumber.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].URL.restoreFromCache();
                $scope.parameters.FunctionKeys[paramId].Silent.restoreFromCache();
            } else {
                $scope.parameters.FunctionKeys[paramId].CallPickupCode.value = "";
                $scope.parameters.FunctionKeys[paramId].Connection.value = 0;
                $scope.parameters.FunctionKeys[paramId].DTMFCode.value = "";
                $scope.parameters.FunctionKeys[paramId].DisableCode.value = "";
                $scope.parameters.FunctionKeys[paramId].DisplayName.value = "";
                $scope.parameters.FunctionKeys[paramId].EnableCode.value = "";
                $scope.parameters.FunctionKeys[paramId].PhoneNumber.value = "";
                $scope.parameters.FunctionKeys[paramId].URL.value = "";
                $scope.parameters.FunctionKeys[paramId].Silent.value = 0;
            }
        }
    };

    $scope.expanded = [{
        Left: [false, false, false, false, false, false, false, false, false, false, false, false, false, false],
        Right: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]
    },
        {
            Left: [false, false, false, false, false, false, false, false, false, false, false, false, false, false],
            Right: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]
        }]

    $scope.keyHasInputError = [{
        Left: [false, false, false, false, false, false, false, false, false, false, false, false, false, false],
        Right: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]
    },
        {
            Left: [false, false, false, false, false, false, false, false, false, false, false, false, false, false],
            Right: [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false]
        }]

    $scope.toggle = function (sideId, keyId) {
        if ($scope.isCollapsed(sideId, keyId) == false) {
            if ($scope.areInputsErrorsExist(sideId, keyId) == true) {
                $scope.keyHasInputError[$scope.activePage][sideId][keyId] = true;
            } else {
                $scope.keyHasInputError[$scope.activePage][sideId][keyId] = false;
            }
        }
        if ($scope.keyHasInputError[$scope.activePage][sideId][keyId] == false) {
            $scope.expanded[$scope.activePage][sideId][keyId] = !$scope.expanded[$scope.activePage][sideId][keyId];
        }
    };

    $scope.activePage = 0;
    $scope.togglePage = function (tabIndex) {
        $scope.activePage = tabIndex;
    }
    $scope.isCollapsed = function (sideId, keyId) {
        return !$scope.expanded[$scope.activePage][sideId][keyId];
    };

    $scope.init = function (extensionModuleId) {
        $scope.activeExtensionModuleId = extensionModuleId;
    };

    //init model from NVM
    $scope.configuredKeys = {
        Left: [],
        Right: []
    };
    var updateConfiguredKeys = function () {
        //3 modules supported
        for (var moduleId = 0; moduleId < 3; moduleId++) {
            $scope.configuredKeys.Left[moduleId] = [];
            $scope.configuredKeys.Right[moduleId] = [];
            //2 pages per module
            for (var pageId = 0; pageId < 2; pageId++) {
                $scope.configuredKeys.Left[moduleId][pageId] = [];
                $scope.configuredKeys.Right[moduleId][pageId] = [];

                //14 left keys + 1 on the bottom for page toggling (not configurable )
                for (var i = 0; i < $scope.config.extensionModuleCount.Left; i++) {
                    $scope.configuredKeys.Left[moduleId][pageId].push($scope.parameters.FunctionKeys[getExtModId(moduleId, pageId, 0, i)]);
                }
                //15 right keys
                for (var i = 0; i < $scope.config.extensionModuleCount.Right; i++) {
                    $scope.configuredKeys.Right[moduleId][pageId].push($scope.parameters.FunctionKeys[getExtModId(moduleId, pageId, 1, i)]);
                }
            }
        }
    };
    updateConfiguredKeys();
}]);

angular.module('WebUI').controller('DoorStationsCtrl', ['$scope', 'Tools', function ($scope, Tools) {

    $scope.$parent.$parent.inputType[0] = 'password';
    $scope.$parent.$parent.inputType[1] = 'password';

    var updateDoorStationsList = function () {
        $scope.configuredDoorStations = Tools.List.getConfiguredEntries($scope.parameters.DoorStations, $scope.DoorStations.Default);
    };

    $scope.newEntryExpanded = false;
    $scope.expanded = [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false];
    $scope.showPassword = [false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false];

    $scope.toggleNewEntryExpanded = function () {
        $scope.newEntryExpanded = !$scope.newEntryExpanded;
    }

    $scope.toggle = function (doorStationId) {
        $scope.expanded[doorStationId] = !$scope.expanded[doorStationId];
    }

    $scope.toggleShowPassword = function (doorStationId) {
        $scope.showPassword[doorStationId] = !$scope.showPassword[doorStationId];
    };

    $scope.isCollapsed = function (doorStationId) {
        return !$scope.expanded[doorStationId];
    };

    $scope.DoorStations = {
        NewEntry: {},
        Default: {
            Name: '',
            CameraURL: '',
            DTMFCode: '',
            PictureRefreshTime: '500',
            SIPID: '',
            Username: '',
            Password: ''
        }
    };

    Tools.List.copyEntry($scope.DoorStations.NewEntry, $scope.DoorStations.Default);

    $scope.addEntry = function (listName) {
        var freeEntryIndex = Tools.List.getFreeEntryIndex($scope.parameters[listName], $scope[listName].Default);
        var customParameters = {};

        if (freeEntryIndex !== null) {
            Tools.List.copyEntry($scope.parameters[listName][freeEntryIndex], $scope[listName].NewEntry);
            Tools.List.copyEntry($scope[listName].NewEntry, $scope[listName].Default);
            customParameters[listName] = Tools.List.prepareSingleEntry($scope.parameters[listName], freeEntryIndex);
            $scope.processForm({customParameters: customParameters, ignoreInvalidForm: true});
            updateDoorStationsList();
            $scope.toggleNewEntryExpanded();
        }
    };

    $scope.showAdvancedOptions = function () {
        return ($scope.parameters.PhoneConfig.DoorStation.advancedOptions);
    }

    $scope.deleteEntry = function (listName, entry) {
        var customParameters = {};

        $scope.expanded[$scope.configuredDoorStations.indexOf(entry)] = false;
        Tools.List.deleteEntry(entry, $scope[listName].Default);

        customParameters[listName] = Tools.List.prepareSingleEntry($scope.parameters[listName], $scope.parameters[listName].indexOf(entry));
        $scope.processForm({customParameters: customParameters, ignoreInvalidForm: true});
        updateDoorStationsList();
    };

    $scope.$on('Form::Cancel', function () {
        Tools.List.copyEntry($scope.DoorStations.NewEntry, $scope.DoorStations.Default);
    });

    updateDoorStationsList();
}]);

angular.module('WebUI').controller('SystemLoggingCtrl', ['$scope', '$http', 'Popup', 'State', '$timeout', function ($scope, $http, Popup, State, $timeout) {
    MAXWELL_LOG_ID = 'maxwellLog';
    KERNEL_LOG_ID = 'kernelLog';
    $scope.cliData = "";
    $scope.kernelLogData = "";
    var lastLine = 0;
    var lastUpdate = 0;
    var SYSLOG_POLL_INTERVAL = 2000;
    var stop = false;

    $scope.accordions = {
        maxwellLog:
            {
                title: "Maxwell Logs",
                open: false,
                intervalHandler: null
            }
        ,
        kernelLog:
            {
                title: "Kernel Logs",
                open: false,
                intervalHandler: null
            }
    };

    $scope.toggle = function (accordionId) {

        accordionToHide = accordionId == MAXWELL_LOG_ID ? KERNEL_LOG_ID : MAXWELL_LOG_ID;
        $scope.accordions[accordionId].open = !$scope.accordions[accordionId].open;
        if ($scope.accordions[accordionId].open) {
            $scope.accordions[accordionToHide].open = false;
            run(accordionId);
            stop(accordionToHide);
        } else {
            stop(accordionId);
        }
    };

    $scope.getSysLog = function () {
        $http.get('SysLog', {cache: false, params: {'lastLine': lastLine, 'lastUpdate': lastUpdate}})
            .then(function (response) {
                if (response && response.data
                    && response.data.syslogAppend.length > 0) {
                    $scope.cliData += response.data.syslogAppend;
                    lastLine = response.data.lastLine;
                    lastUpdate = response.data.lastUpdate;
                }
            });
    };

    $scope.getKernelLog = function () {
        $http.get('KernelLog', {cache: false, params: {'lastLine': lastLine, 'lastUpdate': lastUpdate}})
            .then(function (response) {
                if (response && response.data
                    && response.data.syslogAppend.length > 0) {
                    $scope.kernelLogData += response.data.syslogAppend;
                    lastLine = response.data.lastLine;
                    lastUpdate = response.data.lastUpdate;
                }
            });
    };

    run = function (accordionId) {
        if (accordionId == MAXWELL_LOG_ID) {
            $scope.accordions.maxwellLog.intervalHandler = setInterval(function () {
                $scope.getSysLog();
            }, SYSLOG_POLL_INTERVAL)

        } else if (accordionId == KERNEL_LOG_ID) {
            $scope.accordions.kernelLog.intervalHandler = setInterval(function () {
                $scope.getKernelLog();
            }, SYSLOG_POLL_INTERVAL)

        }
    };

    stop = function (accordionId) {
        if (accordionId == MAXWELL_LOG_ID) {
            clearInterval($scope.accordions.maxwellLog.intervalHandler);
        } else if (accordionId == KERNEL_LOG_ID) {
            clearInterval($scope.accordions.kernelLog.intervalHandler);
        } else {
            clearInterval($scope.accordions.maxwellLog.intervalHandler);
            clearInterval($scope.accordions.kernelLog.intervalHandler);
        }
    };

    $scope.DownloadLogs = function () {
        var popupData = {
            title: $scope.text.Download_expert,
            message: $scope.text.DownloadLogs
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.get('Logs', {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            responseType: 'blob',
            restartLogoutTimer: true
        })
            .then(function (response) {
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), 'maxwell-logs.tar.gz');
                Popup.close(popupData.layer, popupData.id);
            })
            .catch(function (e) {
                console.log(e)
                popupData.title = $scope.text.Error_expert;
                popupData.message = $scope.text.FileIsCorupt_expert;
                popupData.buttons = [
                    {text: $scope.text.OK_expert, value: $scope.config.button.ok}
                ];

                Popup.show(popupData, {priority: $scope.config.popup.priority.error, force: true});
            })

    }
    $scope.$on('$destroy', stop);
}]);

angular.module('WebUI').controller('SysDumpCtrl', ['$scope', '$http', 'Popup', 'State', function ($scope, $http, Popup, State) {

    $scope.MakeSysDumpSendPC = function () {
        var popupData = {
            title: $scope.text.SysDumpPrepare,
            message: $scope.text.PrepareSysdump
        };

        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.get('SysDump', {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            responseType: 'blob',
            restartLogoutTimer: true
        })
            .then(function (response) {
                Popup.close(popupData.layer, popupData.id);
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), 'Sysdump.zip');
            })
            .catch(function () {
                popupData.title = $scope.text.Error_expert;
                popupData.message = $scope.text.FileIsCorupt_expert;
                popupData.buttons = [
                    {text: $scope.text.OK_expert, value: $scope.config.button.ok}
                ];

                Popup.show(popupData, {priority: $scope.config.popup.priority.error, force: true});
            });
    };

}]);

angular.module('WebUI').controller('CoreDumpsCtrl', ['$scope', '$http', 'Popup', 'State', function ($scope, $http, Popup, State) {

    $scope.coreDumpPrepare = false;

    $scope.MakeCoreDumpsSendPC = function () {
      var popupData = {
        title: $scope.text.CoreDumpsPrepare,
        message: $scope.text.PrepareCoreDumps
       }
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
        $scope.coreDumpPrepare = true;
        $http.get('CoreDumps', {
            cache: false,
            timeout: $scope.config.timers.defaultHttpTimeout * 1000,
            responseType: 'blob',
            restartLogoutTimer: true
        })
            .then(function (response) {
                popupData.message = $scope.text.CoreDumpArePrepared;
                popupData.buttons = [{text: $scope.text.OK_expert, value: $scope.config.button.ok}];
                Popup.show(popupData, {priority: $scope.config.popup.priority.user, force: true});
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), 'CoreDumps.zip');
                $scope.coreDumpPrepare = false;
            })
            .catch(function () {
                popupData.title = $scope.text.Error_expert;
                popupData.message = $scope.text.FileIsCorupt_expert;
                popupData.buttons = [
                    {text: $scope.text.OK_expert, value: $scope.config.button.ok}
                ];
                Popup.show(popupData, {priority: $scope.config.popup.priority.error, force: true});
                $scope.coreDumpPrepare = false;
            });
    };
}]);

angular.module('WebUI').controller('WebConfiguratorCtrl', ['$scope', function ($scope) {
    $scope.ConfirmAdminPassword = $scope.parameters.AdminPassword.value;
    $scope.ConfirmUserPassword = $scope.parameters.UserPassword.value;

}]);


angular.module('WebUI').controller('PCAPLoggingCtrl', ['$scope', '$http', '$filter', 'Popup', 'State', function ($scope, $http, $filter, Popup, State) {
    var fileName = 'tcpdump';

    var checkFileExists = function () {
        $http.head(fileName, {cache: false})
            .then(function () {
                $scope.fileExists = true;
            })
            .catch(function () {
                $scope.fileExists = false;
            });
    };

    checkFileExists();

    $scope.state['Network.PCAP'] = 'init';

    $scope.startLogging = function () {
        $scope.state['Network.PCAP'] = 'start';
        State.set({'Network.PCAP': $scope.state['Network.PCAP']});
    };

    $scope.stopLogging = function () {
        $scope.state['Network.PCAP'] = 'stop';
        State.set({'Network.PCAP': $scope.state['Network.PCAP']});
    };

    $scope.downloadFile = function () {
        $http.get(fileName, {cache: false, responseType: 'blob', restartLogoutTimer: true})
            .then(function (response) {
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), fileName + '.pcap');

                $scope.state['Network.PCAP'] = 'delete';
                State.set({'Network.PCAP': $scope.state['Network.PCAP']});
            })
            .catch(checkFileExists);
    };

    $scope.$watch('state["Network.PCAP"]', function (newValue, oldValue) {
        if (newValue !== oldValue) {
            switch (newValue) {
                case 'start':
                case 'started':
                case 'stop':
                case 'delete':
                    State.get({name: 'Network.PCAP', interval: 2});
                    break;

                case 'exceeded':
                    var popupData = {
                        title: $scope.text.PCAPLogging,
                        message: $filter('Replacement')($scope.text.FileExceedsSizeLimit, [fileName, $filter('FileSize')(3000000)]),
                        buttons: [
                            {text: $scope.text.OK, value: $scope.config.button.ok}
                        ]
                    };

                    Popup.show(popupData, {priority: $scope.config.popup.priority.error});
                /* no break */

                case 'stopped':
                case 'deleted':
                    State.stop('Network.PCAP');
                    checkFileExists();
                    break;
            }
        }
    });

    State.getNetworkPCAPState()
}]);

angular.module('WebUI').controller('SIPLoggingCtrl', ['$scope', '$http', '$interval', 'State', function ($scope, $http, $interval, State) {
    var interval = null,
        skipHeaderRegexp = /((?:[^\n]+\n){1})(?:(?:[\s\S]*?)((?:SIP|ACK|BYE|CANCEL|INFO|INVITE|NOTIFY|OPTIONS|REFER|REGISTER|SUBSCRIBE|UPDATE)(?:[\s\S]*)))/g,
        maxNumberOfPackets = 400;

    $scope.logs = '';
    $scope.state['Network.SIP_PCAP'] = 'init';

    $scope.sendAction = function (action) {
        State.set({'Network.SIP_PCAP': action});
    };

    $scope.$watch('state["Network.SIP_PCAP"]', function (newValue, oldValue) {
        if (newValue !== oldValue) {
            switch (newValue) {
                case 'start':
                case 'stop':
                case 'delete':
                    State.get({name: 'Network.SIP_PCAP', interval: 1});
                    break;

                case 'started':
                    if (interval) {
                        $interval.cancel(interval);
                    }

                    interval = $interval(function () {
                        $http.get('sipdump', {cache: false})
                            .then(function (response) {
                                var rawPackets = response.data.split('\n\n');
                                var printablePackets = [];

                                angular.forEach(rawPackets, function (packet) {
                                    if (packet.length) {
                                        printablePackets.unshift(packet.replace(skipHeaderRegexp, "$1$2").trim());
                                    }
                                });

                                $scope.logs = printablePackets.join('\n\n-------\n\n');

                                if (printablePackets.length >= maxNumberOfPackets) {
                                    $scope.sendAction('stop');
                                }
                            });
                    }, 2000);

                    State.stop('Network.SIP_PCAP');
                    break;

                case 'stopped':
                    if (interval) {
                        $interval.cancel(interval);
                    }

                    State.stop('Network.SIP_PCAP');
                    break;

                case 'deleted':
                    if (interval) {
                        $interval.cancel(interval);
                    }

                    $scope.logs = '';

                    State.stop('Network.SIP_PCAP');
                    break;
            }
        }
    });

    $scope.$on('$destroy', function () {
        if (interval) {
            $interval.cancel(interval);
        }
    });

    State.getSIP_PcapState()
}]);

angular.module('WebUI').controller('RingtonesCtrl', ['$scope', '$http', '$filter', 'FileUploader', 'Popup', 'Tools', function ($scope, $http, $filter, FileUploader, Popup, Tools) {
    $scope.ringtones = [];
    RINGTONE_TYPE = $scope.config.ringtoneType;

    var ringtoneTypes = [RINGTONE_TYPE['RINGTONE_EXTERNAL'],
        RINGTONE_TYPE['RINGTONE_INTERNAL'],
        RINGTONE_TYPE['RINGTONE_GROUP'],
        RINGTONE_TYPE['RINGTONE_DOORSTATION'],
        RINGTONE_TYPE['RINGTONE_EXTERNAL']];

    var popupData = {
        title: $scope.text.Ringtones
    };

    $scope.storedRingtones = {
        data: {},
        totalSize: 0,
        storageLimit: 5 * 1024 * 1024,
        countLimit: 20
    };

    var getRingtones = function () {
        var promise = $http.get('Ringtones', {cache: false});

        promise
            .then(function (response) {
                $scope.ringtones = response.data;

                $scope.storedRingtones.data = $scope.ringtones.filter(function (ringtone) {
                    return ringtone.group === 'user';
                });

                $scope.storedRingtones.totalSize = 0;
                angular.forEach($scope.storedRingtones.data, function (ringtone) {
                    $scope.storedRingtones.totalSize += ringtone.size;
                });
            });

        return promise;
    };

    var correctSelectedRingtones = function () {
        var factoryRingtonesPaths = $scope.ringtones
            .filter(function (ringtone) {
                return (ringtone.group === 'factory');
            })
            .map(function (ringtone) {
                return ringtone.name;
            });

        var userRingtonesPaths = $scope.ringtones
            .filter(function (ringtone) {
                return (ringtone.group === 'user');
            })
            .map(function (ringtone) {
                return ringtone.name;
            });

        var allRingtonesPaths = factoryRingtonesPaths.concat(userRingtonesPaths);

        if (allRingtonesPaths.length > 0) {
            if (allRingtonesPaths.indexOf($scope.parameters.DefaultRingtone.value) === -1) {
                $scope.parameters.DefaultRingtone.value = factoryRingtonesPaths[0];
            }

            angular.forEach($scope.parameters.SIP, function (connection) {
                angular.forEach(ringtoneTypes, function (ringtoneType) {
                    if (allRingtonesPaths.indexOf(connection[ringtoneType].value) === -1) {
                        connection[ringtoneType].value = factoryRingtonesPaths[0];
                    }
                });
            });

            $scope.processForm({
                customParameters: {
                    DefaultRingtone: $scope.parameters.DefaultRingtone,
                    SIP: $scope.parameters.SIP
                },
                ignoreInvalidForm: true,
                disableSuccessPopup: true
            });
        }
    };

    var getRingtoneGroup = function (ringtoneFileName) {
        var group = '';

        $scope.ringtones.some(function (ringtone) {
            if (ringtone.name === ringtoneFileName) {
                group = ringtone.group;
                return true;
            } else {
                return false;
            }
        });

        return group;
    };

    $scope.getRingtoneNameToDisplay = function (ringtone) {
        var match = Tools.getFileNameFromPath(ringtone.name).match(/^([0-9]+_)?(.+)\..+$/);
        var ringtoneName = '';

        if (match) {
            if (ringtone.group === 'factory') {
                ringtoneName = match[2];
            } else {
                ringtoneName = (match[1] ? match[1] + match[2] : match[2]);
            }
        }

        return ringtoneName;
    };

    $scope.getRintoneNameFromPath = function (ringtoneNameOrPath) {
        var match = Tools.getFileNameFromPath(ringtoneNameOrPath).match(/^([0-9]+_)?(.+)\..+$/);
        return match[0];
    };

    $scope.getRingtoneGroupText = function (group) {
        var groups = {
            factory: $scope.text.FactoryRingtones,
            user: $scope.text.StoredRingtones
        };

        return groups[group];
    };

    $scope.setDefaultRingtone = function (autoPropagate, autoSave) {

        if (autoPropagate) {
            angular.forEach($scope.parameters.SIP, function (connection) {
                if (connection.Active.value === $scope.config.checkbox.checked) {
                    angular.forEach(ringtoneTypes, function (ringtoneType) {
                        connection[ringtoneType].value = $scope.parameters.DefaultRingtone.value;
                    });
                }
            });
        }
        if (autoSave) {
            resetAllCallsProperties()
            $scope.processForm({
                customParameters: {
                    DefaultRingtone: $scope.parameters.DefaultRingtone,
                    SIP: $scope.parameters.SIP,
                    OneMelodyRingtoneExternal: $scope.parameters.OneMelodyRingtoneExternal,
                    OneMelodyRingtoneInternal: $scope.parameters.OneMelodyRingtoneInternal,
                    OneMelodyRingtoneGroup: $scope.parameters.OneMelodyRingtoneGroup,
                    OneMelodyRingtoneDoorStation: $scope.parameters.OneMelodyRingtoneDoorStation,
                },
                ignoreInvalidForm: true
            });
        }
    };

    $scope.playRingtone = function (ringtoneFileName) {
        $http.post('Ringtones', {
            'play': ringtoneFileName,
            'group': getRingtoneGroup(ringtoneFileName)
        });
    };

    $scope.deleteRingtone = function (ringtonePath) {
        popupData.message = ringtonePath ? $scope.text.DeleteSelectedRingtone : $scope.text.DeleteAllRingtones;
        popupData.buttons = [
            {text: $scope.text.Yes, value: $scope.config.button.yes},
            {text: $scope.text.No, value: $scope.config.button.no}
        ];

        Popup.show(popupData, {priority: $scope.config.popup.priority.user})
            .then(function (popupButton) {
                switch (popupButton) {
                    case $scope.config.button.yes:
                        popupData.message = $scope.text.PleaseWait;
                        popupData.buttons = [];
                        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

                        $http.post('Ringtones', ringtonePath ? {'delete': Tools.getFileNameFromPath(ringtonePath)} : {'deleteAll': true})
                            .then(function () {
                                popupData.message = ringtonePath ? $scope.text.RingtoneDeleted : $scope.text.RingtonesDeleted;
                                popupData.buttons = [
                                    {text: $scope.text.OK, value: $scope.config.button.ok}
                                ];
                                Popup.show(popupData, {priority: $scope.config.popup.priority.user});
                            })
                            .finally(function () {
                                getRingtones()
                                    .then(correctSelectedRingtones);
                            });
                        break;
                }
            });
    };

    $scope.deleteAllRingtones = $scope.deleteRingtone;

    $scope.uploader = new FileUploader({
        url: 'Ringtones',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['mp3', 'wav', 'ogg'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        },
            {
                name: 'fileSize',
                fn: function (file) {
                    return $scope.storedRingtones.totalSize + file.size < $scope.storedRingtones.storageLimit;
                }
            },
            {
                name: 'fileCount',
                fn: function () {
                    return $scope.storedRingtones.data.length + this.queue.length < $scope.storedRingtones.countLimit;
                }
            }]
    });

    $scope.uploader.onCompleteAll = function () {
        getRingtones();
    };

    $scope.uploader.onWhenAddingFileFailed = function (item, filter, options) {
        popupData.message = $filter('Replacement')($scope.text.FilesWereNotAddedToQueue, [$scope.storedRingtones.countLimit]);
        popupData.buttons = [
            {text: $scope.text.OK, value: $scope.config.button.ok}
        ];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    };

    $scope.uploader.onErrorItem = function (item, response, status, headers) {
        if (status === 400 && response && response.message) {
            popupData.buttons = [
                {text: $scope.text.OK, value: $scope.config.button.ok}
            ];

            switch (response.message) {
                case '#fileType#':
                    popupData.message = $filter('Replacement')($scope.text.FileIsNotSupported, [item.file.name]);
                    break;

                case '#fileSize#':
                    popupData.message = $filter('Replacement')($scope.text.FileIsTooBig, [item.file.name, $filter('FileSize')(item.file.size)]);
                    break;

                case '#fileCount#':
                    popupData.message = $filter('Replacement')($scope.text.FileExceedsLimitOfRingtones, [item.file.name, $scope.storedRingtones.countLimit]);
                    break;
            }

            Popup.show(popupData, {priority: $scope.config.popup.priority.info, chain: true});
        }
    };

    $scope.disableIfConnectionIsInactive = function (connection) {
        return connection.Active.value == '0' ? 'textDisabled' : '';
    }

    $scope.currentRingtones = {
        External: [],
        Internal: [],
        Group: [],
        DoorStation: [],
        Optional: []
    }

    function Ringtone(connectionId, type) {
        this.connectionId = connectionId;
        this.type = type;

        Object.defineProperty(this, 'value', {
            get: function () {
                return $scope.parameters[getOneMelodyParameterKey(this.type)].value != 0
                    ? $scope.parameters[getOneMelodyParameterKey(this.type)].value
                    : $scope.parameters.SIP[this.connectionId][type].value;
            },
            set: function (newValue) {
                $scope.parameters.SIP[this.connectionId][type].value = newValue
                $scope.parameters[getOneMelodyParameterKey(type)].value != 0
                    ? changeRingtoneWhenAllCallsAreSet(type, this.connectionId)
                    : null;
            }
        });
    }

    $scope.initRingtones = function () {
        angular.forEach($scope.getSIPAccounts(), function (connection, connectionId) {
            $scope.currentRingtones.External[connectionId] = new Ringtone(connectionId, RINGTONE_TYPE['RINGTONE_EXTERNAL']);
            $scope.currentRingtones.Internal[connectionId] = new Ringtone(connectionId, RINGTONE_TYPE['RINGTONE_INTERNAL']);
            $scope.currentRingtones.Group[connectionId] = new Ringtone(connectionId, RINGTONE_TYPE['RINGTONE_GROUP']);
            $scope.currentRingtones.DoorStation[connectionId] = new Ringtone(connectionId, RINGTONE_TYPE['RINGTONE_DOORSTATION']);
            $scope.currentRingtones.Optional[connectionId] = new Ringtone(connectionId, RINGTONE_TYPE['RINGTONE_OPTIONAL']);
        });
    };

    function getOneMelodyParameterKey(type) {
        key = '';
        switch (type) {
            case RINGTONE_TYPE['RINGTONE_EXTERNAL']:
                key = 'OneMelodyRingtoneExternal';
                break;
            case  RINGTONE_TYPE['RINGTONE_INTERNAL']:
                key = 'OneMelodyRingtoneInternal';
                break;
            case RINGTONE_TYPE['RINGTONE_GROUP']:
                key = "OneMelodyRingtoneGroup";
                break;
            case RINGTONE_TYPE['RINGTONE_DOORSTATION']:
                key = "OneMelodyRingtoneDoorStation";
                break;
            case RINGTONE_TYPE['RINGTONE_OPTIONAL']:
                key = "OneMelodyRingtoneOptional";
                break;
        }
        return key;
    }

    function changeRingtoneWhenAllCallsAreSet(type, editedConnection) {
        angular.forEach($scope.getSIPAccounts(), function (connection, connectionId) {
            if (connectionId != editedConnection) {
                connection[type].value = $scope.parameters[getOneMelodyParameterKey(type)].value
            }
        })
        $scope.parameters[getOneMelodyParameterKey(type)].value = 0;

    }

    function resetAllCallsProperties() {
        angular.forEach(ringtoneTypes, function (type) {
            $scope.parameters[getOneMelodyParameterKey(type)].value = 0;
        })
    }

    getRingtones();
    $scope.initRingtones()
}]);

angular.module('WebUI').controller('MainMenuCtrl', ['$scope', 'FileUploader', 'Tools', function ($scope, FileUploader, Tools) {
    $scope.uploader = new FileUploader({
        url: 'UploadXML',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['xml'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });
}]);

angular.module('WebUI').controller('PicturesCtrl', ['$scope', '$http', '$filter', 'FileUploader', 'Popup', 'Tools', function ($scope, $http, $filter, FileUploader, Popup, Tools) {
    var popupData = {
        title: $scope.text.Pictures
    };

    var getPictures = function () {
        var promise = $http.get('Pictures', {cache: false});

        promise
            .then(function (response) {
                $scope.pictures = response.data;
                $scope.screensavers.splice(0);

                angular.forEach($scope.pictures, function (picture) {
                    $scope.screensavers.push(function (checked) {
                        if (angular.isDefined(checked)) {
                            $scope.parameters.ScreensaverPictures.value = checked ?
                                Tools.DelimitedString.add($scope.parameters.ScreensaverPictures.value, picture.fileName) :
                                Tools.DelimitedString.remove($scope.parameters.ScreensaverPictures.value, picture.fileName);
                        } else {
                            return Tools.DelimitedString.exist($scope.parameters.ScreensaverPictures.value, picture.fileName);
                        }
                    });
                });
            });

        return promise;
    };

    $scope.screenSaverTimeoutOptions = $scope.parameters.ScreenSaverTimeout.options.filter(function (option) {
        return option !== "1";
    });

    $scope.isMaxwell2 = function () {
        return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 2']);
    }

    $scope.screenSaverOptions = $scope.parameters.Screensaver.options.filter(function (option) {
        if ($scope.isMaxwell2()) {
            return option !== "2" && option !== "3";
        }
        return option;
    });

    $scope.pictures = [];
    $scope.screensavers = [];
    $scope.maxPicturesCount = 20;

    $scope.deletePicture = function (deletedPicture) {
        deletedPictureArray = [deletedPicture]
        popupData.message = $scope.text.PleaseWait;
        popupData.buttons = [];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        $http.post('Pictures', {'delete': deletedPictureArray})
            .then(getPictures)
            .then(function () {
                $scope.parameters.ScreensaverPictures.value = Tools.DelimitedString.remove($scope.parameters.ScreensaverPictures.value, deletedPicture);

                if ($scope.parameters.ScreensaverPictures.value.length === 0 && $scope.parameters.Screensaver.value === '2') {
                    $scope.parameters.Screensaver.value = '0';
                }

                $scope.processForm({
                    customParameters: {
                        Screensaver: $scope.parameters.Screensaver,
                        ScreensaverPictures: $scope.parameters.ScreensaverPictures
                    },
                    ignoreInvalidForm: true,
                    disableSuccessPopup: true
                });
            })
            .finally(function () {
                Popup.close(popupData.layer, popupData.id);
            });
    };

    $scope.handleScreensaverChange = function () {
        if ($scope.parameters.Screensaver.value !== '3') {
            $scope.parameters.ScreensaverHTTPSource.restoreFromCache();
        }
    };

    $scope.uploader = new FileUploader({
        url: 'Pictures',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['jpg', 'jpeg'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        },
            {
                name: 'fileSize',
                fn: function (file) {
                    var fileSizeLimit = 250 * 1024;
                    var availableSpace = 5 * 1024 * 1024;
                    var usedSpace = 0;

                    angular.forEach($scope.pictures, function (pictures) {
                        usedSpace += pictures.size;
                    });

                    return (usedSpace + file.size < availableSpace && file.size < fileSizeLimit);
                }
            },
            {
                name: 'fileCount',
                fn: function () {
                    return $scope.pictures.length + this.queue.length < $scope.maxPicturesCount;
                }
            }]
    });

    $scope.uploader.onCompleteAll = function () {
        getPictures();
    };

    $scope.uploader.onWhenAddingFileFailed = function (item, filter, options) {
        popupData.message = $filter('Replacement')($scope.text.FilesWereNotAddedToQueue, [$scope.maxPicturesCount]);
        popupData.buttons = [
            {text: $scope.text.OK, value: $scope.config.button.ok}
        ];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    };

    $scope.uploader.onErrorItem = function (item, response, status, headers) {
        if (status === 400 && response && response.message) {
            popupData.buttons = [
                {text: $scope.text.OK, value: $scope.config.button.ok}
            ];

            switch (response.message) {
                case '#fileType#':
                    popupData.message = $filter('Replacement')($scope.text.FileIsNotSupported, [item.file.name]);
                    break;

                case '#fileSize#':
                    popupData.message = $filter('Replacement')($scope.text.FileIsTooBig, [item.file.name, $filter('FileSize')(item.file.size)]);
                    break;

                case '#fileCount#':
                    popupData.message = $filter('Replacement')($scope.text.FileExceedsLimitOfPictures, [item.file.name, $scope.maxPicturesCount]);
                    break;
            }

            Popup.show(popupData, {priority: $scope.config.popup.priority.info, chain: true});
        }
    };
    getPictures();
}]);

angular.module('WebUI').controller('ScreencapturesCtrl', ['$scope', '$http', '$filter', 'FileUploader', 'Popup', 'Tools', function ($scope, $http, $filter, FileUploader, Popup, Tools) {

    var popupData = {
        title: $scope.text.screencaptures
    };

    const MAX_SCREEN_CAPTURE_COUNT = 10;
    const URL = '/Screencaptures?screenCapture=';

    $scope.lockCaptureButton = false;
    $scope.screenCaptures = [];
    $scope.selectAllFlag = false;

    var getScreencaptures = function () {

        var promise = $http.get('Screencaptures', {cache: false});

        promise
            .then(function (response) {
                angular.forEach(response.data, function (responeElem, key) {
                    addScreenCaptureFlag = true;
                    angular.forEach($scope.screenCaptures, function (screenCapture, key) {

                        if (screenCapture.fileName == responeElem.fileName) {
                            addScreenCaptureFlag = false;
                        }
                    });

                    if (addScreenCaptureFlag) {
                        responeElem.isSelected = $scope.selectAllFlag;
                        $scope.screenCaptures.unshift(responeElem);
                    }
                });

            });

        return promise;
    };

    getScreencaptures();

    $scope.captureActualScreen = function () {
        $scope.lockCaptureButton = true;

        if (!$scope.isMaxAmountOfScreenCapture()) {
            var promise = $http.post('Screencaptures', {cache: false});
            promise
                .then(function (response) {
                    getScreencaptures()
                })
                .finally(function () {
                    $scope.lockCaptureButton = false
                })
        } else {
            $scope.showMaxPicturesReachedMsg();
            $scope.lockCaptureButton = false;
        }
    };

    $scope.downloadScreenCaptures = function () {
        angular.forEach(getSelectedScreenCaptureNames(), function (elem) {
            fileName = elem.split('.')[0];
            $scope.downloadScreenCapture(fileName);
        })
    }

    $scope.downloadScreenCapture = function (filename) {

        popupData.message = $scope.text.PleaseWait;
        popupData.buttons = [];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});

        url = URL + filename;
        $http.get(url, {cache: false, responseType: 'blob'})
            .then(function (response) {
                saveAs(new Blob([response.data], {type: 'application/octet-stream'}), filename + '.png');
            })
            .catch(function () {
                popupData.title = $scope.text.Error;
                popupData.message = $scope.text.ServerError;
                popupData.buttons = [
                    {text: $scope.text.OK, value: $scope.config.button.ok}
                ];

                Popup.show(popupData, {priority: $scope.config.popup.priority.error});
            })
            .finally(function () {
                Popup.close(popupData.layer, popupData.id);
            });
    };

    $scope.deleteScreencaptures = function () {

        popupData.message = $scope.text.PleaseWait;
        popupData.buttons = [];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
        $http.post('Screencaptures', {'delete': getSelectedScreenCaptureNames(true)})
            .then()
            .finally(function () {
                $scope.selectAllFlag = false;
                Popup.close(popupData.layer, popupData.id);
            });
    };

    $scope.toggle = function (screenCapture) {

        screenCapture.isSelected != screenCapture.isSelected;
        areAllSelectedFlag = true;
        angular.forEach($scope.screenCaptures, function (picture, key) {
            !picture.isSelected ? areAllSelectedFlag = false : null
        })

        areAllSelectedFlag ? $scope.selectAllFlag = true : $scope.selectAllFlag = false;
    };

    $scope.toggleSelectAll = function () {

        if ($scope.selectAllFlag) {
            angular.forEach($scope.screenCaptures, function (picture, key) {
                picture.isSelected = true;
            });
        } else {
            angular.forEach($scope.screenCaptures, function (picture, key) {
                picture.isSelected = false;
            });
        }
    }

    $scope.getAmountOfSelectedPictures = function () {
        counter = 0;
        angular.forEach($scope.screenCaptures, function (picture, key) {
            picture.isSelected ? counter++ : null
        })
        return counter;
    }

    $scope.isMaxAmountOfScreenCapture = function () {
        return $scope.screenCaptures.length == MAX_SCREEN_CAPTURE_COUNT ? true : false;
    }

    $scope.showMaxPicturesReachedMsg = function () {
        popupData.message = $filter('Replacement')($scope.text.ScreencaptureMaxAmount);
        popupData.buttons = [
            {text: $scope.text.OK, value: $scope.config.button.ok}
        ];
        Popup.show(popupData, {priority: $scope.config.popup.priority.info});
    }

    $scope.lockButton = function () {
        return $scope.lockCaptureButton
    }

    getSelectedScreenCaptureNames = function ($_removeFlag) {
        let fileNames = []

        angular.forEach($scope.screenCaptures, function (picture, key) {
            if (picture.isSelected) {
                fileNames.push(picture.fileName);
            }
        })

        if ($_removeFlag) {
            $scope.screenCaptures = $scope.screenCaptures.filter(function (el) {
                return el.isSelected == false;
            })
        }
        return fileNames;
    }

}]);

angular.module('WebUI').controller('CallSettingsCtrl', ['$scope', function ($scope) {
    var areaCodes = {
        Other: {international: {prefix: '', code: ''}, local: {prefix: ''}},
        ALB: {international: {prefix: '00', code: '355'}, local: {prefix: '0'}},
        ARE: {international: {prefix: '00', code: '971'}, local: {prefix: '0'}},
        ARG: {international: {prefix: '00', code: '54'}, local: {prefix: '0'}},
        AUS: {international: {prefix: '0011', code: '61'}, local: {prefix: '0'}},
        AUT: {international: {prefix: '00', code: '43'}, local: {prefix: '0'}},
        BEL: {international: {prefix: '00', code: '32'}, local: {prefix: '0'}},
        BGR: {international: {prefix: '00', code: '359'}, local: {prefix: '0'}},
        BHR: {international: {prefix: '00', code: '973'}, local: {prefix: ''}},
        BIH: {international: {prefix: '00', code: '387'}, local: {prefix: '0'}},
        BLR: {international: {prefix: '810', code: '375'}, local: {prefix: '8'}},
        BOL: {international: {prefix: '00', code: '591'}, local: {prefix: '0'}},
        BRA: {international: {prefix: '00', code: '55'}, local: {prefix: '0'}},
        CAN: {international: {prefix: '011', code: '1'}, local: {prefix: '1'}},
        CHE: {international: {prefix: '00', code: '41'}, local: {prefix: '0'}},
        CHL: {international: {prefix: '00', code: '56'}, local: {prefix: '0'}},
        CHN: {international: {prefix: '00', code: '86'}, local: {prefix: '0'}},
        COL: {international: {prefix: '009', code: '57'}, local: {prefix: '09'}},
        CRI: {international: {prefix: '00', code: '506'}, local: {prefix: ''}},
        CYP: {international: {prefix: '00', code: '357'}, local: {prefix: ''}},
        CZE: {international: {prefix: '00', code: '420'}, local: {prefix: ''}},
        DEU: {international: {prefix: '00', code: '49'}, local: {prefix: '0'}},
        DNK: {international: {prefix: '00', code: '45'}, local: {prefix: ''}},
        ECU: {international: {prefix: '00', code: '593'}, local: {prefix: '0'}},
        EGY: {international: {prefix: '00', code: '20'}, local: {prefix: '0'}},
        ESP: {international: {prefix: '00', code: '34'}, local: {prefix: ''}},
        EST: {international: {prefix: '00', code: '372'}, local: {prefix: ''}},
        FIN: {international: {prefix: '00', code: '358'}, local: {prefix: '0'}},
        FRA: {international: {prefix: '00', code: '33'}, local: {prefix: '0'}},
        GBR: {international: {prefix: '00', code: '44'}, local: {prefix: '0'}},
        GRC: {international: {prefix: '00', code: '30'}, local: {prefix: '0'}},
        HKG: {international: {prefix: '001', code: '852'}, local: {prefix: ''}},
        HRV: {international: {prefix: '00', code: '385'}, local: {prefix: '0'}},
        HUN: {international: {prefix: '00', code: '36'}, local: {prefix: '06'}},
        IDN: {international: {prefix: '001', code: '62'}, local: {prefix: '0'}},
        IND: {international: {prefix: '00', code: '91'}, local: {prefix: '0'}},
        IRL: {international: {prefix: '00', code: '353'}, local: {prefix: '0'}},
        IRN: {international: {prefix: '00', code: '98'}, local: {prefix: '0'}},
        ISL: {international: {prefix: '00', code: '354'}, local: {prefix: ''}},
        ISR: {international: {prefix: '00', code: '972'}, local: {prefix: '0'}},
        ITA: {international: {prefix: '00', code: '39'}, local: {prefix: ''}},
        JOR: {international: {prefix: '00', code: '962'}, local: {prefix: '0'}},
        JPN: {international: {prefix: '010', code: '81'}, local: {prefix: '0'}},
        KOR: {international: {prefix: '001', code: '82'}, local: {prefix: '0'}},
        KWT: {international: {prefix: '00', code: '965'}, local: {prefix: ''}},
        LBN: {international: {prefix: '00', code: '961'}, local: {prefix: '0'}},
        LIE: {international: {prefix: '00', code: '423'}, local: {prefix: ''}},
        LTU: {international: {prefix: '00', code: '370'}, local: {prefix: '0'}},
        LUX: {international: {prefix: '00', code: '352'}, local: {prefix: ''}},
        LVA: {international: {prefix: '00', code: '371'}, local: {prefix: ''}},
        MAR: {international: {prefix: '00', code: '212'}, local: {prefix: '0'}},
        MEX: {international: {prefix: '00', code: '52'}, local: {prefix: '01'}},
        MKD: {international: {prefix: '00', code: '389'}, local: {prefix: '0'}},
        MNE: {international: {prefix: '00', code: '382'}, local: {prefix: '0'}},
        MYS: {international: {prefix: '00', code: '60'}, local: {prefix: '0'}},
        NAM: {international: {prefix: '00', code: '264'}, local: {prefix: '0'}},
        NLD: {international: {prefix: '00', code: '31'}, local: {prefix: '0'}},
        NOR: {international: {prefix: '00', code: '47'}, local: {prefix: ''}},
        NZL: {international: {prefix: '00', code: '64'}, local: {prefix: '0'}},
        PAK: {international: {prefix: '00', code: '92'}, local: {prefix: '0'}},
        PAN: {international: {prefix: '00', code: '507'}, local: {prefix: ''}},
        PER: {international: {prefix: '00', code: '51'}, local: {prefix: '0'}},
        PHL: {international: {prefix: '00', code: '63'}, local: {prefix: '0'}},
        POL: {international: {prefix: '00', code: '48'}, local: {prefix: ''}},
        PRI: {international: {prefix: '011', code: '1'}, local: {prefix: '1'}},
        PRT: {international: {prefix: '00', code: '351'}, local: {prefix: ''}},
        PRY: {international: {prefix: '00', code: '595'}, local: {prefix: '0'}},
        PSE: {international: {prefix: '00', code: '970'}, local: {prefix: '0'}},
        ROU: {international: {prefix: '00', code: '40'}, local: {prefix: '0'}},
        RUS: {international: {prefix: '810', code: '7'}, local: {prefix: '8'}},
        SAU: {international: {prefix: '00', code: '966'}, local: {prefix: '0'}},
        SGP: {international: {prefix: '001', code: '65'}, local: {prefix: ''}},
        SRB: {international: {prefix: '00', code: '381'}, local: {prefix: '0'}},
        SVK: {international: {prefix: '00', code: '421'}, local: {prefix: '0'}},
        SVN: {international: {prefix: '00', code: '386'}, local: {prefix: '0'}},
        SWE: {international: {prefix: '00', code: '46'}, local: {prefix: '0'}},
        THA: {international: {prefix: '001', code: '66'}, local: {prefix: '0'}},
        TUR: {international: {prefix: '00', code: '90'}, local: {prefix: '0'}},
        TWN: {international: {prefix: '002', code: '886'}, local: {prefix: '0'}},
        URY: {international: {prefix: '00', code: '598'}, local: {prefix: '0'}},
        USA: {international: {prefix: '011', code: '1'}, local: {prefix: '1'}},
        VEN: {international: {prefix: '00', code: '58'}, local: {prefix: '0'}},
        VNM: {international: {prefix: '00', code: '84'}, local: {prefix: '0'}},
        ZAF: {international: {prefix: '00', code: '27'}, local: {prefix: '0'}}
    };

    $scope.changeCountryNumbers = function () {
        $scope.parameters.AreaCodesIntPrefix.value = areaCodes[$scope.parameters.AreaCodesCountry.value].international.prefix;
        $scope.parameters.AreaCodesIntCode.value = areaCodes[$scope.parameters.AreaCodesCountry.value].international.code;
        $scope.parameters.AreaCodesLocalPrefix.value = areaCodes[$scope.parameters.AreaCodesCountry.value].local.prefix;
    };

    $scope.activeConnections = $scope.parameters.SIP.filter(function (connection) {
        return connection.Active.value === $scope.config.checkbox.checked;
    });
}]);

angular.module('WebUI').controller('ExpertRebootCtrl', ['$scope', function ($scope) {

    // sample collection of objects
    var weekDays = [
        {Day: 1},
        {Day: 2},
        {Day: 3},
        {Day: 4},
        {Day: 5},
        {Day: 6},
        {Day: 0}
    ];

    $scope.getWeekdays = function () {
        return weekDays;
    };

    $scope.weekDayActive_ = function (value, dayIndex) {
        if (angular.isDefined(value)) {
            if (value == $scope.config.checkbox.checked) {
                $scope.parameters.AutomaticRebootWeekdays.value |= 1 << dayIndex;
                weekDays[dayIndex].Active = $scope.config.checkbox.checked;
            }
            else {
                $scope.parameters.AutomaticRebootWeekdays.value &= ~(1 << dayIndex);
                weekDays[dayIndex].Active = $scope.config.checkbox.unchecked;
            }
        }
        else {
            return ($scope.parameters.AutomaticRebootWeekdays.value & 1 << dayIndex) ? $scope.config.checkbox.checked : $scope.config.checkbox.unchecked;
        }
    };

    $scope.weekDayActive = [
        function (value) {
            return $scope.weekDayActive_(value, 0);
        },
        function (value) {
            return $scope.weekDayActive_(value, 1);
        },
        function (value) {
            return $scope.weekDayActive_(value, 2);
        },
        function (value) {
            return $scope.weekDayActive_(value, 3);
        },
        function (value) {
            return $scope.weekDayActive_(value, 4);
        },
        function (value) {
            return $scope.weekDayActive_(value, 5);
        },
        function (value) {
            return $scope.weekDayActive_(value, 6);
        }
    ];
    $scope.time_ = new Date(2001, 1, 1, $scope.parameters.AutomaticRebootTime.value.split(":")[0], $scope.parameters.AutomaticRebootTime.value.split(":")[1], 0, 0);
    $scope.time = function (value) {
        if (angular.isDefined(value)) {
            $scope.parameters.AutomaticRebootTime.value = value.getHours() + ":" + value.getMinutes();
            $scope.time_.setHours(value.getHours());
            $scope.time_.setMinutes(value.getMinutes());
        } else {
            return $scope.time_;
        }
    };

}]);

angular.module('WebUI').controller('CLICtrl', ['$scope', '$http', '$timeout', function ($scope, $http, $timeout) {
    $scope.cliCommand = "max debug level 9";
    $scope.cliData = "";
    $scope.intervalPending = false;
    var lastLine = 0;
    var lastUpdate = 0;
    var SYSLOG_POLL_INTERVAL = 1000;

    $scope.interval = function () {
        $http.get('SysLog', {cache: false, params: {'lastLine': lastLine, 'lastUpdate': lastUpdate}})
            .then(function (response) {
                if (response && response.data
                    && response.data.syslogAppend.length > 0) {
                    $scope.cliData += response.data.syslogAppend;
                    lastLine = response.data.lastLine;
                    lastUpdate = response.data.lastUpdate;
                }
                if ($scope.intervalPending) {
                    $timeout($scope.interval, SYSLOG_POLL_INTERVAL);
                }
            });
    };

    $scope.run = function () {
        $http.post('CLI', {'cmd': $scope.cliCommand})
            .then(function (response) {
                $timeout($scope.interval, SYSLOG_POLL_INTERVAL);
                $scope.intervalPending = true;
            });

    };

    $scope.stop = function () {
        $scope.intervalPending = false;
    };

    $scope.download = function () {
        $http.get('SysLog', {cache: false, params: {'lastLine': 0, 'lastUpdate': 0}})
            .then(function (response) {
                saveAs(new Blob([response.data.syslogAppend], {type: 'text/html'}), "logs.txt");
            });
    };

    $scope.$on('$destroy', $scope.stop);
}]);

angular.module('WebUI').controller('DisplayCtrl', ['$scope', function ($scope) {
    $scope.getTimeoutOption = function () {
        $scope.screenSaverTimeoutOptions = $scope.parameters.ScreenSaverTimeout.options.filter(function (option) {
            if ($scope.parameters.ScreenSaverTimeout.value == "0") {
                return option !== "1";
            }
            return option !== "1" && option !== "0";
        });

    }
    $scope.isMaxwell2 = function () {
        return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell 2']);
    }

    $scope.isMaxwellBasic = function () {
        return ($scope.parameters.PhoneModel.value === $scope.config.phoneModel['Maxwell Basic']);
    }

    $scope.screenSaverOptions = $scope.parameters.Screensaver.options.filter(function (option) {
        if ($scope.isMaxwell2()) {
            return option !== "2" && option !== "3";
        }
        return option;
    });
    $scope.getTimeoutOption();
    $scope.$on('Form::Set::Success', function (event, data) {
        $scope.getTimeoutOption();

    });
}]);

angular.module('WebUI').controller('CallDivertCtrl', ['$scope', 'Model', function ($scope, Model) {
    PAGE_NAME = 'CallDiverPage'
    $scope.getParametesRelatedWithDeletingAccount(PAGE_NAME, null);

    $scope.getAllCallDivert = function (type) {
        callDivertArray = [];
        for (index = 0; index < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; index++) {
            callDivertArray.push($scope.parameters[type][index])
        }
        return callDivertArray
    }

    getCallDivertParametersArray = function (type) {
        callDivertArray = []
        for (index = 0; index < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; index++) {
            tmpObj = {
                Active: $scope.parameters[type][index].Active.value,
                Target: $scope.parameters[type][index].Target.value,
                VoiceMail: $scope.parameters[type][index].VoiceMail.value
            };
            if (type == $scope.config.callDivertType['NO_ANSWER']) {
                tmpObj['Delay'] = $scope.parameters[type][index].Delay.value;
            }
            callDivertArray.push(tmpObj)
        }
        return callDivertArray
    }

    $scope.setDivertCalls = function () {
        console.log($scope.config.callDivertType['ALL_CALLS'])

        var CallDivertAll = getCallDivertParametersArray($scope.config.callDivertType['ALL_CALLS']);
        var CallDivertBusy = getCallDivertParametersArray($scope.config.callDivertType['WHEN_BUSY']);
        var CallDivertNoAnswer = getCallDivertParametersArray($scope.config.callDivertType['NO_ANSWER']);

        var selectedParameters = {
            CallDivertAll: CallDivertAll,
            CallDivertBusy: CallDivertBusy,
            CallDivertNoAnswer: CallDivertNoAnswer
        };
        $scope.processFormWithParameters(selectedParameters, {ignoreInvalidForm: true});
    };

}]);

angular.module('WebUI').controller('VoicemailCtrl', ['$scope', 'Model', function ($scope, Model) {
    $scope.setVoicemails = function () {
        for (var i = 0; i < $scope.config.sipAccountsCount[Model.getPhoneModelName()]; i++) {
            $scope.parameters.CallDivertAll[i].TargetMail = $scope.parameters.SIP[i].VoiceMailMailbox;
            $scope.parameters.CallDivertBusy[i].TargetMail = $scope.parameters.SIP[i].VoiceMailMailbox;
            $scope.parameters.CallDivertNoAnswer[i].TargetMail = $scope.parameters.SIP[i].VoiceMailMailbox;
        }
        $scope.processForm();
    };

}]);

angular.module('WebUI').controller('VoipCtrl', ['$scope', 'FileUploader', 'Popup', 'Tools', '$window', '$http', 'Model', function ($scope, FileUploader, Popup, Tools, $window, $http, Model) {

    $scope.$parent.$parent.inputType[0] = 'password';

    $scope.RemoveSipCertificate = function () {
        var promise = $http.post('RemoveSipCert', {cache: false});

        promise
            .then(function (response) {
                Model.getParameters({removeCache: true});
            });
    };

    $scope.RemoveSipPrivateKey = function () {
        var promise = $http.post('RemoveSipPrivateKey', {cache: false});

        promise
            .then(function (r) {
                Model.getParameters({removeCache: true});
            });
    };

    $scope.uploader = new FileUploader({
        url: 'UploadSipCert',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['pem'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });
    var onBeforeUploadItem = $scope.uploader.onBeforeUploadItem;

    $scope.uploader.filters.push({
        name: 'globalFileSize',
        fn: function (file) {
            return file.size < $scope.config.maxUploadFileSize;
        }
    });

    $scope.uploader.onBeforeUploadItem = function (item) {
        item.headers = item.headers || {};
        item.headers.Authorization = 'Bearer ' + $window.sessionStorage.token;
        onBeforeUploadItem(item);
    };

    $scope.uploader.onCompleteItem = function (item, response, status, headers) {
        Model.getParameters({removeCache: true});
    };
    $scope.uploader2 = new FileUploader({
        url: 'UploadSipPrivateKey',
        filters: [{
            name: 'fileExt',
            fn: function (file) {
                var allowedExtensions = ['txt'];
                return allowedExtensions.indexOf(Tools.getFileExtensionFromFileName(file.name).toLowerCase()) !== -1;
            }
        }]
    });
    var onBeforeUploadItem2 = $scope.uploader2.onBeforeUploadItem;
    $scope.uploader2.filters.push({
        name: 'globalFileSize',
        fn: function (file) {
            return file.size < $scope.config.maxUploadFileSize;
        }
    });

    $scope.uploader2.onBeforeUploadItem = function (item) {
        item.headers = item.headers || {};
        item.headers.Authorization = 'Bearer ' + $window.sessionStorage.token;
        onBeforeUploadItem2(item);
    };


    $scope.uploader2.onCompleteItem = function (item, response, status, headers) {
        Model.getParameters({removeCache: true});
    };

    $scope.isRtcpXrEnabled = function () {
        return $scope.parameters.SIPRtpUseRTCPXR.value == '1';
    };
}]);

angular.module('WebUI').controller('WatchdogStatisticsCtrl', ['$scope', '$http', function ($scope, $http) {
    $scope.watchdogStats = {}
    $scope.expertStats = {}

    $http.get('WatchdogStats', {cache: false})
        .then(function (response) {
            if (response && response.data) {
                $scope.watchdogStats = response.data;
            }
        });
    $scope.isWatchdogStatsEmpty = function () {
        return angular.equals($scope.watchdogStats, {})
    }

    $http.get('ExpertStat', {cache: false})
        .then(function (response) {
            if (response && response.data) {
                $scope.expertStats = response.data;
                for (var key in $scope.expertStats) {
                    if ($scope.expertStats[key] == null) {
                        $scope.expertStats[key] = $scope.text.NotAvailable;
                    }
                }
            }
        });
}]);

angular.module('WebUI').controller('UptimeCtrl', ['$scope', '$http', function ($scope, $http) {

    $http.get('Uptime', {cache: false})
        .then(function (response) {
                if ((response && response.data)) {
                    $scope.uptime = response.data;
                }
            }
        );
}]);


angular.module('WebUI').controller('TotalUptimeCtrl', ['$scope', '$http', function ($scope, $http) {
    $scope.expertStats = {}
    $scope.totalUptimeDays = null;
    $scope.totalUptimeHours = null;
    $scope.totalUptimeMinutes = null;

    $http.get('ExpertStat', {cache: false})
        .then(function (response) {
            if (response && response.data) {
                $scope.expertStats = response.data;
            }
            $scope.totalUptimeDays = $scope.expertStats['workingTime'] != null ? ($scope.expertStats['workingTime'] / 3600 / 24 - 1 / 2) : null;
            $scope.totalUptimeHours = $scope.expertStats['workingTime'] != null ? ($scope.expertStats['workingTime'] / 3600 % 24) : null;
            $scope.totalUptimeMinutes = $scope.expertStats['workingTime'] != null ? ($scope.expertStats['workingTime'] % 3600 / 60) : null;
        });
}]);

angular.module('WebUI').controller('FunKeyCtrl', ['$scope', function ($scope) {

    $scope.getAvailableFKeyTypes = function(funKeyOptions){

        index = funKeyOptions.indexOf($scope.config.functionKeyType.PbxFeature);
        if( index != -1 && $scope.parameters.PhoneSystem.value != $scope.config.phoneSystem['BINTEC-ELMEG_PBX']){
            funKeyOptionsCopy = [];
            angular.copy(funKeyOptions, funKeyOptionsCopy);
            funKeyOptionsCopy.splice(index, 1);
            funKeyOptions = funKeyOptionsCopy;
        }
        return funKeyOptions
    };

    $scope.getLabelForTargetNumber = function (keyType) {
        let label = '';

        switch (keyType) {
            case $scope.config.functionKeyType.ParkAndRetrive:
                label = $scope.text.ParkingSpace;
                break;
            case $scope.config.functionKeyType.PbxFeature:
                label = $scope.text.ServiceCode;
                break;
            default:
                label = $scope.text.TargetNumber;
        }

        return label;
    }

}]);