angular.module('WebUI').filter('FileSize', function() {
    var units = ['B', 'KB', 'MB', 'GB', 'TB', 'PB'];

    return function(input, precision) {
        if (isNaN(parseFloat(input)) || !isFinite(input)) {
            return '';
        }

        var unit = 0;

        while (input >= 1024) {
            input /= 1024;
            unit++;
        }

        if (angular.isUndefined(precision)) {
            precision = 0;
        }

        return input.toFixed(precision) + ' ' + units[unit];
    };
});

angular.module('WebUI').filter('Reverse', function() {
    return function(items) {
        return (angular.isArray(items) ? items.slice().reverse() : items);
    };
});

angular.module('WebUI').filter('Replacement', function() {
    return function(text, replacements) {
        if (angular.isString(text) && angular.isArray(replacements) && replacements.length > 0) {
            var parts = text.split('%');

            if (angular.isArray(parts)) {
                text = parts.shift();
                angular.forEach(parts, function(part, index) {
                    if (angular.isUndefined(replacements[index]) || replacements[index] === null) {
                        text += '%';
                    } else {
                        text += angular.isArray(replacements[index]) ? replacements[index].join(', ') : replacements[index];
                    }

                    text += part;
                });
            }
        }

        return text;
    };
});

angular.module('WebUI').filter('DateTime', ['$filter', 'Model', 'Config', function($filter, Model, Config) {
	var DateTime = function(input, format) {
        if (Model.parameters &&
            Model.parameters.TimeFormat &&
            Model.parameters.DateOrder &&
            Model.parameters.TimeZone &&
            Model.parameters.AutoAdjustClockForDST)
        {
            switch (format) {
                case 'Time':
                    format = Config.formatTime[Model.parameters.TimeFormat.value];
                    break;

                case 'Date':
                    format = Config.formatDate[Model.parameters.DateOrder.value];
                    break;

                case 'DateTime':
                    format = Config.formatDate[Model.parameters.DateOrder.value] + ' ' + Config.formatTime[Model.parameters.TimeFormat.value];
                    break;
            }

            var timezoneMoment = moment(Number(input)).utc();
            var timezone = Model.parameters.TimeZone.value.split('.')[1];
            timezoneMoment.tz(timezone);

            if (timezoneMoment.isDST() && Model.parameters.AutoAdjustClockForDST.value === 'AutomatedDaylightSavingDisabled') {
                timezoneMoment.utcOffset(timezoneMoment.utcOffset() - 60);
            }

            return $filter('date')(input, format, timezoneMoment.format('ZZ'));
        } else {
            return input;
        }
	};

    DateTime.$stateful = true;

    return DateTime;
}]);