<?php

class certificates
{

  private $certificate;
  private $certdir = array('pending' => PENDINGCERTIFICATESDIR,
       'accepted' => ACCEPTEDCERTIFICATESDIR,
       'rejected' => REJECTEDCERTIFICATESDIR,
       'invalid' => INVALIDCERTIFICATESDIR);
  private $defaultCertName = "/tmp/tmpcert/tmp_cert.pem";
  private $defaultKeyName = "/tmp/tmpcert/tmp_key.pem";

    private function scanDir()
  {
    $certificatearray = array();
    foreach ($this->certdir as $key => $value)
    {
      $dir = new DirectoryIterator($value);
      foreach ($dir as $val)
      {
        if ($val->isDot() == false && file_exists($value . DIRECTORY_SEPARATOR . $val->getFilename()))
        {
          if ($this->isDerCert($value . DIRECTORY_SEPARATOR . $val->getFilename()))
          {
            $this->certificate = $this->parsePemCertificate($this->convertDerToPem($value . DIRECTORY_SEPARATOR . $val->getFilename()));
          }
          else
          {
            $this->certificate = $this->parsePemCertificate(file_get_contents($value . DIRECTORY_SEPARATOR . $val->getFilename()));
          }
          $infoarr = $this->getInfoFromCert();
          $infoarr['file'] = $val->getFilename();
          $infoarr['status'] = $key;
          $infoarr['fingerprints'] = $this->getHash($value . DIRECTORY_SEPARATOR . $val->getFilename());
          $certificatearray[] = $infoarr;
        }
      }
    }
    return $this->makeJson($certificatearray);
  }

  private function scanDirInvalidCertificates()
  {
      $certificatearray = array();
      $dir = new DirectoryIterator(INVALIDCERTIFICATESDIR);
      foreach ($dir as $val)
      {
          if ($val->isDot() == false)
          {
              $certificatearray[] = $val->getFilename();
          }
      }
      return $this->makeJson($certificatearray);
  }

  private function isCA()
  {
    $return = false;
    if (isset($this->certificate['extensions']) && isset($this->certificate['extensions']['basicConstraints']) && strpos($this->certificate['extensions']['basicConstraints'], 'CA:TRUE') !== false)
    {
      $return = true;
    }
    return $return;
  }

  public function uploadCertificate($_file, $_dir)
  {
    try
    {
      $uplo = new filemanager();
      $filename = null;
      $result = $uplo->uploadFile($_file, 'fileoperations::replaceRenameFile', array('TypeFilters' => unserialize(CERTIFICATEMIMETYPE)), array('maxwell' => $_dir), $filename);

      if ($result == FILEOK)
      {



        if ($this->checkIsCertificateFile($_dir . DIRECTORY_SEPARATOR . $filename) != false)
        {
          $result = array('action' => FILEOK, 'filename' => $filename);
        }
        else
        {
          unlink($_dir . DIRECTORY_SEPARATOR . $filename);
          $result = array('action' => FILEERR, 'filename' => $filename);
        }
      }
      else
      {
        $result = array('action' => $result, 'filename' => $filename);
      }

      return $result;
    }
    catch (Exception $ex)
    {
      $exc = new errorexceptionhandler();
      $exc->getException($ex);
      return array('action' => $result, 'filename' => $filename);
    }
  }

  private function pem2DerCert($_file)
  {
    $certificateContent = file_get_contents($_file);
    $begin = "CERTIFICATE-----";
    $end = "-----END";
    $return = false;

    if (strpos($certificateContent, $begin) != false)
    {
      $pem_datas = substr($certificateContent, strpos($certificateContent, $begin) + strlen($begin));
      $pem_data = substr(trim($pem_datas), 0, strpos($pem_datas, $end) - 1);

      $der = base64_decode($pem_data);

      $return = $der;
    }
    return $return;
  }

  private function isDerCert($_file)
  {

    $certificateContent = file_get_contents($_file);
    $begin = "CERTIFICATE-----";
    $return = false;
    if (strpos($certificateContent, $begin) == false)
    {
      $return = true;
    }
    else
    {
      $return = false;
    }
    return $return;
  }

  private function convertDerToPem($_cert)
  {
    $cert = base64_encode(file_get_contents($_cert));
    $pemCert = sprintf(
            '-----BEGIN CERTIFICATE-----%s-----END CERTIFICATE-----', PHP_EOL . wordwrap($cert, 64, "\n", true) . PHP_EOL
    );

    return $pemCert;
  }

  private function checkIsCertificateFile($_file)
  {
    $return = false;
    if ($this->isDerCert($_file))
    {
      $cert = $this->convertDerToPem($_file);
    }
    else
    {
      $cert = file_get_contents($_file);
    }
    try
    {
      if (openssl_x509_read($cert) === false)
      {
        $return = false;
      }
      else
      {
        $return = true;
      }
      return $return;
    }
    catch (Exception $ex)
    {
      return false;
    }
  }
    public function verifyCert($_password) {
        $result = utils::extractKeyCertFromPemFile($this->defaultCertName, $this->defaultKeyName);
        $result &= validation::checkExpired($this->defaultCertName);
        $result &= validation::verifyCertKey($this->defaultCertName, $this->defaultKeyName, $_password);

        return $result;
    }

  private function parsePemCertificate($_cert)
  {
    $parse = openssl_x509_parse($_cert);



    $return = false;
    if ($parse != false)
    {
      $return = $parse;
    }
    else
    {
      $return = false;
    }

    return $return;
  }

  private function makeJson($_data = array())
  {
    return json_encode($_data);
  }

  private function getInfoFromCert()
  {
    return array(
        "isCA" => $this->isCA(),
        "serialNumber" => $this->getSerialNumber(),
        "validity" => array(
            "from" => $this->getValidFrom(),
            "to" => $this->getValidTo()
        ),
        "subject" => $this->getSubject(),
        "issuer" => $this->getIssuer()
    );
  }

  private function getValidFrom()
  {
    return $this->certificate['validFrom_time_t'];
  }

  private function getValidTo()
  {
    return $this->certificate['validTo_time_t'];
  }

  private function getSubject()
  {
    return $this->certificate['subject'];
  }

  private function getIssuer()
  {
    return $this->certificate['issuer'];
  }

  private function getSerialNumber()
  {
    return $this->certificate['serialNumber'];
  }

  private function setHash($_file)
  {
    $pemder = $this->pem2DerCert($_file);
    $der = $this->isDerCert($_file);
    $return = false;
    if ($pemder != false)
    {
      $return = $pemder;
    }
    else if ($der != false)
    {
      $return = $der;
    }
    return $return;
  }

  private function getHash($_file)
  {
    $return = array();
    $dercert = $this->setHash($_file);
    if ($dercert != false)
    {
      $return = array("md5" => md5($dercert), 'sha1' => sha1($dercert));
    }
    return $return;
  }

  public function getInfoCertificates()
  {
    return $this->scanDir();
  }

  public function getInfoInvalidCertificates()
  {
      return $this->scanDirInvalidCertificates();
  }

    public function convertCert($_filetype, $_uploadedfile, $_pass) {
        $result = false;
        $filemanager = new filemanager();
        switch ($_filetype) {
            case 'cert':
                $result = converter::convert($_uploadedfile, $_pass);
                $result &= validation::checkExpired($_uploadedfile);
                $filemanager->renameFile($_uploadedfile, $this->defaultCertName);
                break;
            case 'key':
                $result = converter::convert($_uploadedfile, $_pass);
                $result &= converter::isKeyFile($_uploadedfile, $_pass);
                $filemanager->renameFile($_uploadedfile, $this->defaultKeyName);
                break;
            default:
                $result = converter::convert($_uploadedfile, $_pass);
                $filemanager->renameFile($_uploadedfile, $this->defaultCertName);
                break;
        }
        return $result;
    }
}
