<?php

/**
 * IPC_MQTT class is MQTT client wrapper
 *
 * @author Marcin Rabczynski <marcin.rabczynski@gigaset.com>
 * @package Maxwell-webui
 * @created 09.2016
 */
class IPC_MQTT implements IPC_Interface
{
    private static $request;
    private static $message;
    private static $instance = null;
    /*
     * @var Mosquitto\Client $connection
     */
    private $connection;

    public static function getInstance()
    {
        try
        {
            if (self::$instance === null)
            {
                self::$instance = new IPC();
            }
            return self::$instance;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return 0;
        }
    }

    public function sendMessageToProperty($param, $method = METHOD_GET_VALUE, $value = null)
    {
        $message = array('command' => $method, 'parameters' => array('key' => $param, 'field' => PropertyService::getFields()->webui), 'response_required' => true);

        return $this->sendMessage($param, $method, $value, $message);
    }

    public function sendMessageToPropertyVList($param, $method, $value = null){
        $message = array('command' => $method, 'parameters' => array('list' => $param), 'response_required' => true);
            return $this->sendMessage($param, $method, $value, $message);
    }

    public function sendMessageToState($param, $method = METHOD_GET_VALUE, $value = null)
    {
        $message = array('command' => $method, 'parameters' => array('key' => $param), 'response_required' => true);

        return $this->sendMessage($param, $method, $value, $message);
    }

    public function sendMessageToAudio($value)
    {
        $topicPublish = 'web-ui/audio';

        return $this->sendCustomMessage($topicPublish, $value);
    }

    public function sendMessageToTelephony($value)
    {
        $topicPublish = 'web-ui/telephony';

        return $this->sendCustomMessage($topicPublish, $value);
    }

    public function sendMessageToPhoneServices($value)
    {
        $topicPublish = 'web-ui/phone-services';

        return $this->sendCustomMessage($topicPublish, json_encode($value));
    }

    public function sendMessageToPhoneUI($value)
    {
        $topicPublish = 'web-ui/phone-ui';

        return $this->sendCustomMessage($topicPublish, json_encode($value));
    }

    public function sendMessageToSip($value)
    {
        $topicPublish = 'web-ui/sip-client';

        return $this->sendCustomMessage($topicPublish, json_encode($value));
    }


    public function sendMessageToPhonesystem($message, $defaultTimeout=10000)
    {
        $topicPublish = 'web-ui/phone-services';
        $topicSubscribe = 'phone-services/web-ui';

        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message, false, $defaultTimeout);

        return $responseMessage;
    }

    public function sendMessageToPhoneUiToMakeScreenCapture($defaultTimeout=10000){
        $topicPublish = 'web-ui/phone-ui';
        $topicSubscribe = 'phone-ui/web-ui';
        $message = '{"text":"save_screen"}';

        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message, false, $defaultTimeout);
    }

    public function sendMessageToDownloadLocalPhonebook($defaultTimeout=10000){
        $topicPublish = 'web-ui/sysconf';
        $topicSubscribe = 'sysconf/web-ui';
        $message = '{"command":"DownloadLocalPhonebook", "parameters":{}, "response_required":true}';

        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message, false, $defaultTimeout);
    }

    /**
     * @param $topicPublish string
     * @param $value string
     * @return bool|object
     */
    public function sendCustomMessage($topicPublish, $value)
    {
        $responseMessage = $this->publish($topicPublish, $value);
        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    private function connect()
    {
        try
        {
            $this->connection = new Mosquitto\Client(null, true);
            $this->connection->connect(IPCHOST, 1883, 60);
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function subscribe($_topic)
    {
        try
        {
            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            $this->connection->onMessage(function ($_message)
            {
                $this->messageHandler($_message);
            });
            $this->connection->subscribe($_topic, 1);
            $this->connection->loopForever(5000);
        }
        catch(Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function publish($_topicPublish, $_message)
    {
        try
        {

            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            self::$request = $_message;
            $this->connection->publish($_topicPublish, self::$request);
            logger::debug("DEBUG",__FILE__, __LINE__,$_topicPublish);
            logger::debug("DEBUG",__FILE__, __LINE__,self::$request);

            $this->connection->disconnect();
            return true;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function sendMessage($param, $method = METHOD_GET_VALUE, $value = null, $message)
    {
        $topicPublish = 'web-ui/prop-service';
        $topicSubscribe = 'prop-service/web-ui';
        $responseMessage = null;

        switch ($method)
        {
            case METHOD_GET_VALUE:
            case METHOD_GET_VALUES_LIST:
                $message['parameters']['fields'] = $value;
                $message['parameters']['list'] = is_array($param) ? $param : array($param);
                break;

            case METHOD_SET_VALUE:
                $message['parameters']['value'] = $value;
                break;

            case METHOD_SET_VALUES_LIST:
                //need to convert associative array into list of objects
                foreach ($value as $key=>$val)
                {
                    $object = (object) [$key => $val];
                    $message['parameters']['list'][] = $object;
                }
                break;
            case METHOD_RESET_VALUES_LIST:
                $message['parameters']['list'] = $param;
                break;
            case METHOD_BACKUP:
                $message['parameters']['allSettings'] = $value;
                break;

            default:;
        }

        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message);

        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    private function publishWait($_topicPublish, $_topicSubscribe, $_message, $encodeJson=true, $defaultTimeout=10000)
    {
        try
        {
            if(self::$instance === null)
            {
                self::getInstance();
            }
            //lock here
            $fp = fopen(MQTT_MUTEX, "w+");
            if (flock($fp, LOCK_EX))
            {
                fwrite($fp, "Locked access to MQTT!!!");
            }
            //
            $this->connect();
            /*
            * @var Mosquitto\Client $connection
            */
            $this->connection->onMessage(function ($_message)
            {
                $this->connection->exitLoop();
                $this->messageHandler($_message);
            });

            $this->connection->subscribe($_topicSubscribe, 2);

            if ($encodeJson)
            {
                self::$request = json_encode($_message);
            }
            else
            {
                self::$request = $_message;
            }
            self::$message = "";
            $this->connection->publish($_topicPublish, self::$request);

            //wait maximum 10s for answer from PropertyService
            $this->connection->loopForever($defaultTimeout);
            if (self::$message == "")
            {
              logger::debug("ERROR",__FILE__, __LINE__,'Response message not recived');
            }
            $this->connection->disconnect();

            //unlock here
            flock($fp, LOCK_UN);
            fclose($fp);

            if (isset(self::$message->parameters))
            {
                return self::$message->parameters;
            }
            else
            {
                return self::$message;
            }
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function messageHandler($_message)
    {
        self::$message = json_decode(trim($_message->payload));

        return $_message->payload;
    }
}
