<?php

/**
 * IPC_ZMQ class is ZMQ client wrapper
 *
 * @author Marcin Rabczynski <marcin.rabczynski@gigaset.com>
 * @package Maxwell-webui
 * @created 09.2016
 */
class IPC_ZMQ implements IPC_Interface
{
    private static $_instance = null;
    /*
     * @var ZMQSocket $_connection
     */
    private $_connection;

    /**
     * @return IPC_ZMQ|null
     */
    public static function getInstance()
    {
        if (self::$_instance === null)
        {
            self::$_instance = new IPC();
        }
        return self::$_instance;
    }


    /**
     * @param string $host
     * @return ZMQSocket
     */
    private function connect($host)
    {
        try
        {

            $this->_connection = new ZMQSocket(new ZMQContext(), ZMQ::SOCKET_REQ, 'my socket');
            $this->_connection->setSockOpt(ZMQ::SOCKOPT_LINGER, 0);
            $endpoints = $this->_connection->getEndpoints();
            if (!in_array($host, $endpoints['connect'], TRUE))
            {
                 $this->_connection->connect($host);
            }

            return $this->_connection;

        } catch (ZMQException $ex)
        {

            logger::debug("ERROR", $ex->getFile(), $ex->getLine(), $ex->getMessage() . " " .  $ex->getTraceAsString());
            return false;

        }
    }

    private function disconnect($host)
    {
        $endpoints = $this->_connection->getEndpoints();
        if (!in_array($host, $endpoints['connect'], TRUE))
        {
            $this->_connection->disconnect();
        }
        return true;
    }

    private function send($host, $message)
    {
        try {

            $connection = $this->connect($host);
            $splittedMessage = str_split($message, 255);
            $connection->sendmulti($splittedMessage);
            return $connection->recvMulti();

        } catch (ZMQException $ex)
        {
            logger::debug("ERROR", $ex->getFile(), $ex->getLine(), $ex->getMessage() . " " . $ex->getTraceAsString());
            return array();
        }
    }

    public function sendStateMessageToProperty($param, $method = METHOD_GET_VALUE, $value = null)
    {
        $this->sendMessageToProperty($method, 'State.'.$param, $value);
    }

    private function arrayToJSONObject($array)
    {
        if(is_array($array))
        {
            $string = '';
            foreach ($array as $key => $val) {
                $string .= $val;
            }
        } else {
            $string = $array;
        }
        return json_decode(trim($string));
    }

    public function sendMessageToProperty($param, $method = METHOD_GET_VALUE, $value = null)
    {
        $message = array('method' => $method, 'param' => $param, 'field' => PropertyService::getFields()->webui);
        if ($value !== null) {
            if ($method === METHOD_GET_VALUE || $method === METHOD_GET_VALUES_LIST) {
                $message['fields'] = $value;
            } else if ($method === METHOD_SET_VALUE) {
                $message['value'] = $value;
            } else if ($method === METHOD_SET_VALUES_LIST) {
                $message['list'] = $value;
            }
        }
        $responseMessage = $this->send(NVMHOST, json_encode($message));

        $output = $this->arrayToJSONObject($responseMessage);
        if(is_array($responseMessage) && count($responseMessage) === 1) {
            if (isset($output->result))
            {
                $output = $output->result;
            }
        }

        return $output;
    }

    public function sendMessageToState($param, $method = METHOD_GET_VALUE, $value = null)
    {
        return $this->sendMessageToProperty($param, $method, $value);
    }

    public function sendMessageToAudio($value)
    {
        return $this->sendMessageToPhoneUI($value);
    }

    public function sendMessageToPhoneUI($value)
    {
        return $this->send(NVMHOSTMESSAGE, json_encode($value));
    }

}
