<?php
class ringtones
{

    public function uploadRingtones($_files)
    {
        try
        {
            $result = false;
            $filemanager = new filemanager();
            $filename = null;
            if (($filemanager->sumFilesSize($_files) <=
                $filemanager->availableStorageSize(filemanager::getUserStoragePath(RINGTONESPATH), MAXALLOCATEDSIZE)))
            {
                if ($filemanager->checkFileCount(filemanager::getUserStoragePath(RINGTONESPATH)) <= MAXFILESCOUNT)
                {
                    $result = $filemanager->uploadFile($_files, 'fileoperations::replaceRenameFile',
                        array(
                            'TypeFilters' => unserialize(AUDIOMIMETYPE)
                        ), array(
                            'maxwell' => RINGTONESPATH
                        ), $filename);
                }
                else
                {
                    $result = FILECOUNT;
                    logger::logSyslogMessage(LOG_WARNING, 'Skipping any further uploads due to user ringtones number limit reached');
                }
            }
            else
            {
                $result = ERRFILESIZE;
                logger::logSyslogMessage(LOG_WARNING, 'Skipping any further uploads due to user storage limit reached');
            }
            return $result;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);
            logger::logSyslogMessage(LOG_ERR, $ex);
            return false;
        }
    }

    public function deleteRingtones($_ringtone = array())
    {
        try
        {
            $deletedfile = 0;
            foreach ($_ringtone as $value)
            {
                if (unlink(RINGTONESPATH . DIRECTORY_SEPARATOR . $value))
                {
                    logger::logSyslogMessage(LOG_INFO, 'Deleted user ringtone ' . RINGTONESPATH . DIRECTORY_SEPARATOR . $value);
                    $deletedfile ++;
                }
                else
                {
                    logger::logSyslogMessage(LOG_ERR, 'Failed deleteing user ringtone ' . RINGTONESPATH . DIRECTORY_SEPARATOR . $value);
                }
            }

            $ret = false;
            if ($deletedfile == count($_ringtone))
            {
                $ret = true;
            }
            return $ret;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);
            logger::logSyslogMessage(LOG_ERR, $ex);
            return false;
        }
    }

    public function deleteAllRingtones()
    {
        try
        {
            $ret = false;
            if (file_exists(filemanager::getUserStoragePath(RINGTONESPATH)))
            {
                $iter = new DirectoryIterator(filemanager::getUserStoragePath(RINGTONESPATH));
                $deletedfile = 0;
                $filecount = 0;
                foreach ($iter as $file)
                {
                    if ($file->isFile())
                    {
                        $filecount ++;
                        if (unlink(RINGTONESPATH . DIRECTORY_SEPARATOR . $file->getFilename()))
                        {
                            $deletedfile ++;
                            logger::logSyslogMessage(LOG_INFO, 'Deleted user ringtone ' . RINGTONESPATH . DIRECTORY_SEPARATOR . $file->getFilename());
                        }
                        else
                        {
                            logger::logSyslogMessage(LOG_ERR, 'Failed deleteing user ringtone ' . RINGTONESPATH . DIRECTORY_SEPARATOR . $value);
                        }
                    }
                }
                if ($deletedfile == $filecount)
                {
                    $ret = true;
                }
            }
            return $ret;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);
            logger::logSyslogMessage(LOG_ERR, $ex);
            return false;
        }
    }

    private function getInfoUserRingtones()
    {
        $return = array();
        if (file_exists(filemanager::getUserStoragePath(RINGTONESPATH)))
        {
            $userfile = array();
            $iter = new DirectoryIterator(filemanager::getUserStoragePath(RINGTONESPATH));
            foreach ($iter as $file)
            {
                if (! $file->isDot())
                {
                    $userfile[] = array(
                        'group' => 'user',
                        'name' => $file->getFilename(),
                        'size' => $file->getSize()
                    );
                }
            }
            $return = $userfile;
        }
        return $this->sortArray($return);
    }

    private function sortArray($_array)
    {
        $ret = array();
        if (! empty($_array))
        {
            foreach ($_array as $key => $row)
            {
                $ringtones[$key] = $row['name'];
            }
            array_multisort($ringtones, SORT_NATURAL, $_array);
            $ret = $_array;
        }
        return $ret;
    }

    private function getInfoInternalRingtones()
    {
        $return = array();
        if (file_exists(RINGTONESPATH))
        {
            $internalfile = array();
            $iter = new DirectoryIterator(RINGTONESPATH);
            foreach ($iter as $file)
            {
                if (! $file->isDot())
                {
                    $internalfile[] = array(
                        'group' => 'factory',
                        'name' => $file->getFilename(),
                        'size' => $file->getSize()
                    );
                }
            }
            $return = $internalfile;
        }
        return $return;
    }

    public function getInfoRingtones()
    {
        $mergedarray = array_merge($this->getInfoUserRingtones(), $this->getInfoInternalRingtones());
        return $mergedarray;
    }

    public function playRingtone($_path, $_group)
    {
        if ($_path != '' && $_group != '')
        {
            try
            {
                $nvm = new settingscontroller();
                $ringtonespath = RINGTONESPATH . DIRECTORY_SEPARATOR;
                $volume = $nvm->settingsGetOneParam('Volume');
                $ipc = IPC::getInstance();
                $msg = '{"command": "PLAY_FILE", "parameters": {"VOLUME": ' . $volume . ', "IN_LOOP": false, "PLAY_MODE": 10,  "FILE_NAME": "' .
                    $ringtonespath . $_path . '"}}';
                isMaxwell4() ? $ipc->sendMessageToTelephony($msg) : $ipc->sendMessageToAudio($msg);
                logger::logSyslogMessage(LOG_INFO, 'Ringtone playback ' . $ringtonespath . $_path);
                return true;
            }
            catch (Exception $ex)
            {
                $exc = new errorexceptionhandler();
                $exc->getException($ex);
                logger::logSyslogMessage(LOG_ERR, $ex);
                return false;
            }
        }
    }
}
