<?php
require_once('PropertyService.php');

/**
 * Description of settings
 *
 * @author hampel_ma
 */
class settingscontroller
{
    private $ipc;
    private $view;
    private $state;
    private $ipcresp;
    private $ipc_publish_topic = 'web-ui/prop-service';
    private $ipc_recive_topic = 'prop-service/web-ui';

    public function settingscontroller() {
        $this->ipc = IPC::getInstance();
        $this->view = new viewcontroller();
        $this->state = new tmpsettings();
    }

    /*   * ************ STATE ************* */

    public function settingsSetState($_request) {
        $result = $this->state->setState($_request);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->viewStateForOneObject($result);
        else
            $return = false;

        return $return;
    }

    public function settingsGetState($_request) {
        $result = $this->state->getState($_request);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->viewForOneObject($result);
        else
            $return = false;

        return $return;
    }

    /*   * ************ GET PARAMS ************* */

    private function resp($_user) {

        return $this->view->getAllParamsForUser($_user, 'accessLevelRead');
    }

    private function getNVMKeys($_user) {
        $return = false;
        $nvmrespobj = $this->ipc->sendMessageToState('%', PropertyService::getMethods()->getColsL, PropertyService::getFields()->flags);

        if (!empty($nvmrespobj)) {
            foreach ($nvmrespobj as $v) {
                if (((int)$v->flags) & ACCESS_USER) {
                    $results[$v->key] = $v->flags;
                }
            }
        }
        return $results;
    }

    public function canRestore($restoredParams, $userID) {
        $forbiddenParams = array();
        $allowedParams = $this->getNVMKeys($userID);

        foreach ($restoredParams as $k => $v) {
            if (!isset($allowedParams[$k])) {
                logger::debug("ERROR", __FILE__, __LINE__, "webui restore verification - forbidden param = : " . $k . "==" . $v);
                $forbiddenParams[$k] = $v;
            }
        }

        return count($forbiddenParams) == 0;
    }

    public function settingsGetProvDump() {
        $return = false;
        $nvmrespobj = $this->ipc->sendMessageToProperty('%', PropertyService::getMethods()->getProvDump, PropertyService::getFields()->all);
        if (!empty($nvmrespobj)) {
            $view = $this->view->creator('viewxml');

            if ($this->view->isViewXMLObject($view)) {
                $return = $view->viewGetRequestAllParams($nvmrespobj);
            } else {
                $return = false;
            }
        }
        return $return;
    }

    public function settingsGetPhoneConfigValue($paramName) {

        return trim(shell_exec("phoneconfigcli get $paramName")," \n");
    }

    private function settingsGetPhoneConfigObject($paramName, $webuiName) {
        $val = $this->settingsGetPhoneConfigValue($paramName);

        //view needs objects. E.g. {"flags":8,"value":"http://172.29.8.248/gigaset-prov/netdir/","webui":"XmlServerAddress"}
        return (object)['value' => $val, 'webui' => $webuiName, 'flags' => ACCESS_READONLY];
    }

    private function array_values_recursive($arr, $searchkey) {
        $arr = array_values($arr);
        foreach ($arr as $key => $val)
            if (array_values($val) === $val)
                $arr[$key] = array_values_recursive($val);

        return $arr;
    }

    public function settingsGetAllParams($_user) {
        $return = false;
        $nvmrespobj = $this->ipc->sendMessageToProperty('%', PropertyService::getMethods()->getWebUiDump, PropertyService::getFields()->all);

        if (!empty($nvmrespobj)) {
            $view = $this->view->creator('viewjson');

            if ($this->view->isViewJsonObject($view)) {
                $this->ipcresp = $this->resp($_user);

                $results = array_filter($nvmrespobj, function ($el) {

                    return in_array(explode('.', $el->webui)[0], $this->ipcresp);
                });

                //add non NVM parameters
                $results[] = $this->settingsGetPhoneConfigObject('voip.SrtpEnabled', 'SIPSecurityEnabled');
                $return = $view->viewGetRequestAllParams($results);
                $return['Timestamp'] = $view->viewGetTimestamp();

            } else {
                $return = false;
            }
        }
        return $return;
    }

    public function settingsGetMultiValue($_prefix) {
        return $this->ipc->sendMessageToProperty($_prefix . '%', PropertyService::getMethods()->getColsL, PropertyService::getFields()->all);
    }

    public function settingsGetArrayValue($_prefix, $_user) {

        $nvmrespobj = $this->settingsGetMultiValue($_prefix);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view)) {
            $this->ipcresp = $this->resp($_user);
            $results = array_filter($nvmrespobj, function ($el) {

                return in_array(explode('.', $el->webui)[0], $this->ipcresp);
            }
            );
            $return = $view->viewGetArrayValue($results);
        } else
            $return = false;

        return $return;
    }

    public function settingsGetOneParam($_value) {
        $nvmrespobj = $this->ipc->sendMessageToProperty($_value);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view)) {
            $return = $view->viewForOneObject($nvmrespobj);
        } else {
            $return = false;
        }

        return $return;
    }

    public function settingsGetVList($paramsArray, $fields){
        return $this->ipc->sendMessageToPropertyVList($paramsArray, PropertyService::getMethods()->getColsL, $fields);
    }

    public function settingsResetVlist($paramsArray){
        return $this->ipc->sendMessageToPropertyVList($paramsArray, METHOD_RESET_VALUES_LIST, null);
    }

    public function settingsGetVListArray($paramsArray, $user){
        $nvmrespobj = $this->settingsGetVList($paramsArray, PropertyService::getFields()->all);

        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view)) {
            $this->ipcresp = $this->resp($user);
            $results = array_filter($nvmrespobj, function ($el) {
                return in_array(explode('.', $el->webui)[0], $this->ipcresp);
            }
            );
            $return = $view->viewGetArrayValue($results);
        } else
            $return = false;

        return $return;
    }

    public function settingsSetVlist($paramArray){
        $message = array('command' => METHOD_SET_VALUES_LIST, 'parameters' => array('list' => array($paramArray)), 'response_required' => true);
        $this->ipc->sendCustomMessage($this->ipc_publish_topic, json_encode($message));

    }

    public function settingsGetOneParamWithProperties($_value) {
        $nvmrespobj = $this->ipc->sendMessageToProperty($_value, PropertyService::getMethods()->getColsL, PropertyService::getFields()->all);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->jsonFile($nvmrespobj[0]);
        else
            $return = false;

        return $return;
    }

    public function settingsGetTimestamp() {
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->viewGetTimestamp();
        else
            $return = false;

        return $return;
    }

    /*   * ************ SAVE PARAMS ************* */

    public function settingsSaveOneParam($_params, $_value) {

        if (is_array($_value))
            $_value = implode(',', $_value);

        $nvmresp = $this->ipc->sendMessageToProperty($_params, PropertyService::getMethods()->setV, $_value);

        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->viewSaveOneParam($nvmresp);
        else
            $return = false;

        return $return;
    }

    public function settingsSaveMultiValue($_request) {
        $_request = $this->settingsCheckAccessParams($_request);
        $queryArray = array();
        $objcount = 0;
        $response = array();

        foreach ($_request as $key => $val) {
            //Change number to string
            if (is_numeric($val) || is_bool($val))
                $_request[$key] = (string)$val;



            if ($key == 'Timestamp') {
                $this->settingsSetState(array('System.DateAndTime.Timestamp' => $val));
                $sql = new SQLite3(WEBDB);
                $sql->exec('update webui_users set lastAccess=CURRENT_TIMESTAMP'); //UPDATE lastAccess when user changed timestamp
                if (count($_request) == 1) {
                    return true;
                }
            }
            else if ($key == 'CertPass')
            {
                file_put_contents(CERTPASSFILE, $val);
                unset($_request['CertPass']);
            }
            else {
                if (is_array($val)) {

                    foreach ($val as $k => $v) {
                        //Objects with no properties are no longer considered empty.(stdClass())
                        if (is_object($v) && $v != new stdClass()) {

                            foreach ($v as $keys => $value) {
                                //EXAMPLE JSON {SIP.0.Active:0}
                                $queryArray[$key . '.' . $k . '.' . $keys] = (string)$value;
                            }

                            $objcount++;
                        }
                    }
                    if ($objcount != 0) {

                        foreach ($queryArray as $k => $v) {
                            $response[$k] = $v;
                        }
                    } else {
                        //Change array to string separated by ,
                        $response[$key] = implode(',', $val);
                    }
                } else {
                    $response[$key] = (string)$val;
                }
            }
        }

        unset($_request['Timestamp']); // DELETE Timestamp from array

        $nvmresp = $this->ipc->sendMessageToProperty(null, PropertyService::getMethods()->setVList, $response);

        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view)) {
            $return = $view->viewSaveMultiParams($nvmresp);
        } else {
            $return = false;
        }
        return $return;
    }

    public function settingsCustomRequest($_request) {
        return $this->ipc->sendMessageToProperty(null, $_request['method'], $_request['allSettings']);
    }

    public function settingsGetText($_key) {
        $lang = $this->settingsGetOneParam('WebUILanguage');
        $str = file_get_contents(TEXTPATH . $lang);
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view))
            $return = $view->viewGetText($str, $_key);
        else
            $return = false;

        echo $return;
    }

    public function settingsCheckAccessParams($_request) {
        $view = $this->view->creator('viewjson');

        if ($this->view->isViewJsonObject($view)) {
            $response = $this->view->userParametersAccessSave($_request);
        } else {
            $response = array();
        }

        return $response;
    }

    public function settingsChangePin($_currentpin, $_newpin) {
        $currpin = $this->settingsGetOneParam('PIN');
        if ($_currentpin == $currpin) {
            return $this->settingsSaveOneParam('PIN', $_newpin);
        } else {
            return false;
        }
    }

    public function settingsSendMessage($_topic, $_message) {
        return $this->ipc->sendMessageToProperty($_topic . $_message);
    }

    public function settingsGetCertPassword(){
        return file_exists(CERTPASSFILE) ? file_get_contents(CERTPASSFILE): false;
    }
}
