<?php

require_once($_SERVER['DOCUMENT_ROOT'] . "/conf/autoloader.php");

$_message = NULL;

try {

    header('Content-Type: text/event-stream');
    header('Cache-Control: no-cache');

    ob_end_clean();

    $session = new sessionmanager();
    $token = $session->getToken();
    $userID = $session->verifySession($token);
    $mqtt = new Mosquitto\Client(1);

    register_shutdown_function(function () use ($mqtt, $session) {
        try {
            $mqtt->disconnect();
        } catch (Exception $ex) {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);
            return false;
        }
    });

    if ($userID) {

        $notifications = array();
        $mqtt->connect('localhost', 1883, 60);
        $mqtt->subscribe(PROP_SERVICE_PROP_CLIENT_TOPIC, 1);

        $mqtt->onMessage(function ($_message) use (&$notifications, &$_timeout) {
            $_timeout = microtime(true);
            $id = round($_timeout * 1000);
            $msg = createMessage($id, $_message);
            isMsgNotEmpty($msg) ? $notifications[] = $msg : null;
        });

        while (true) {
            $mqtt->loop();
            $token = $session->getToken();

            if ($session->verifySession($token)) {
                if (count($notifications) > 0) {
                    sendMsgNottify($notifications);
                    $notifications = array();
                } else {
                    echo ":" . PHP_EOL . PHP_EOL;
                }
                flush();
            } else {
                die();
            }
            usleep(300000);
        }
    }
} catch (Exception $ex) {
    $exc = new errorexceptionhandler();
    $exc->getException($ex);
    return false;
}

function sendMsg($msg)
{
    echo PHP_EOL;
    echo "event: " . $msg->type . "\n";
    unset($msg->type);
    echo "data: " .json_encode($msg) . "\n";
    echo PHP_EOL;
}

function sendMsgNottify($notiffy)
{
    foreach ($notiffy as $_msg) {
        sendMsg($_msg);
    }
}

function createMessage($id, $_mqttMessage){
    $mqttPayload = json_decode($_mqttMessage->payload, false);
    $_msg = new stdClass();
    $parameters = [];
    $isStateFlag = false;

    foreach($mqttPayload->parameters->list as $parameter){
        if(!empty($parameter->keyWebui)){
            addParameter($parameters, $parameter->keyWebui, $parameter->value);
        }else if (isState($parameter->key)){
            addParameter($parameters, $parameter->key, $parameter->value);
            $isStateFlag = true;
        }
    }

    if($isStateFlag){
        $_msg->type = STATE_UPDATE_TYPE;
    } else{
        $_msg->type = PARAM_UPDATE_TYPE;
    }

    $_msg->payload = !empty($parameters) ? $parameters : null;
    return $_msg;
}

function addParameter(&$parameters, $key, $value){
    $parameters[] = (object)[
        'key' => $key,
        'value' => $value
    ];
}

function isState($key) {
    $keyAsArray = explode('.', $key);
    return $keyAsArray[0] == 'State' ? true : false;
}

function isMsgNotEmpty($msg)
{
    return (!empty($msg->payload));
}

?>